@@ -146,6 +146,7 @@ void NedAcceleration::draw(cairo_t *cr, double leftx, double topy, double zoom_f
 void NedAcceleration::startContextDialog(GtkWidget *ref) {
 	unsigned int newtempo;
 	unsigned int kind;
+	bool dot;
 	bool state;
 	double start_tempoinverse = getLineStartPoint()->getElement()->determineTempoInverse();
 	double start_tempo = 60000.0 / start_tempoinverse;
@@ -154,9 +155,14 @@ void NedAcceleration::startContextDialog(GtkWidget *ref) {
 	if (newtempo < 10) newtempo = 10;
 	if (newtempo > 999) newtempo = 999;
 	NedTempoDialog *tempodialog = new NedTempoDialog(GTK_WINDOW(ref), NOTE_4, newtempo);
-	tempodialog->getValues(&state, &kind, &newtempo);
+	tempodialog->getValues(&state, &kind, &dot, &newtempo);
 	if (state) {
-		newtempo *= NOTE_4 / kind;
+		if (dot) {
+			newtempo *= 3 * NOTE_4 / 2 / kind;
+		}
+		else {
+			newtempo *= NOTE_4 / kind;
+		}
 		if (newtempo  < 10) newtempo  = 10;
 		if (newtempo  > 999) newtempo  = 999;
 		m_tempodiff = newtempo - start_tempo;
