@@ -36,21 +36,21 @@
 #define BREAK_TIE_BACKWARD (1 << 1)
 
 #define TYPE_REST (1 << 0)
-#define TYPE_NOTE (1 << 1)
+#define TYPE_CHORD (1 << 1)
 #define TYPE_CLEF (1 << 2)
 #define TYPE_KEYSIG (1 << 3)
+#define TYPE_STAFF_SIGN (1 << 5) // otherwise conflic with CLEFS : -((
 #define TYPE_GRACE (1 << 4)
-#define STAFF_ELEM_MASK (TYPE_CLEF)
-#define TYPE_DYNAMIC (1 << 5)
-#define TYPE_TEMPO (1 << 6)
-#define TYPE_SLUR (1 << 7)
-#define TYPE_LINE (1 << 8)
-#define TYPE_LINE3 (1 << 9)
-#define TYPE_FREE_TEXT (1 << 10)
-#define TYPE_SIGN (1 << 11)
-#define TYPE_GUITAR_CHORD (1 << 12)
-#define TYPE_SPACER (1 << 13)
-#define TYPE_GUITAR_CHORDNAME (1 << 14)
+#define TYPE_DYNAMIC (1 << 6)
+#define TYPE_TEMPO (1 << 7)
+#define TYPE_SLUR (1 << 8)
+#define TYPE_LINE (1 << 9)
+#define TYPE_LINE3 (1 << 10)
+#define TYPE_FREE_TEXT (1 << 11)
+#define TYPE_SIGN (1 << 12)
+#define TYPE_GUITAR_CHORD (1 << 13)
+#define TYPE_SPACER (1 << 14)
+#define TYPE_CHORDNAME (1 << 15)
 
 
 #define LYR_CURSOR_LEFT -1
@@ -81,6 +81,7 @@ class BBox;
 class NedCommandList;
 class NedNote;
 class NedFreeReplaceable;
+class NedLyricsEditor;
 class NedSlur;
 class NedMeasure;
 class NedPangoCairoText;
@@ -105,6 +106,7 @@ class NedChordOrRest {
 		NedChordOrRest(NedVoice *voice, int type, bool is_hidden, int line, int dot_count, unsigned int length, int head, unsigned int status, unsigned long long time);
 		NedChordOrRest(NedVoice *voice, int type, unsigned int clef_number, int octave_shift, unsigned long long time, bool dummy); // clef
 		NedChordOrRest(NedVoice *voice, int type, int keysig, int last_keysig, unsigned long long time); // key signature
+		NedChordOrRest(NedVoice *voice, int type, int sub_type, unsigned long long time); // sign 
 		~NedChordOrRest();
 		NedChordOrRest *clone(struct addr_ref_str **addrlist, bool keep_beam = false);
 		NedChordOrRest *clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedVoice *p_voice);
@@ -117,13 +119,14 @@ class NedChordOrRest {
 		void draw(cairo_t *cr, bool *freetexts_or_lyrics_present);
 		void drawTexts(cairo_t *cr, double scale, double last_left_bbox_edge);
 		double getRealXpos() {return m_xpos;}
-		double getXPos() {if (m_tmp_xpos < 0.0) return m_xpos; return m_tmp_xpos;}
+		double getXPos() {/* if (m_tmp_xpos < 0.0) */return m_xpos; /* return m_tmp_xpos; */}
 		double getLeftBboxEdge() {return m_xpos + m_bbox.x;}
 		double getXPosAndMicroshifts() {if (m_tmp_xpos < 0.0) return m_xpos + m_micro_shift; return m_tmp_xpos + m_micro_shift;}
 		void insertNoteAt(NedNote *note, bool echo = FALSE);
 		bool removeNoteAt(int line);
 		bool noConflict(int line);
 		void testTies();
+		void tryTieBackward();
 		void reConfigure();
 		NedMeasure *m_measure;
 		NedNote *getFirstNote();
@@ -133,11 +136,13 @@ class NedChordOrRest {
 		NedPage *getPage();
 		NedSystem *getSystem();
 		NedStaff *getStaff();
+		int getPosition();
 		NedVoice *getVoice() {return m_voice;}
 		void createLyrics(int line, char *text);
 		void setLyrics(int line, char *text);
+		void setLyrics(NedCommandList *command_list, NedLyricsEditor *leditor);
 		void setLyricsFromOldContext(NedMainWindow *main_window, int line, char *text);
-		void appendToLyrics(NedCommandList *command_list, int line, char *text); 
+		void appendToLyrics(NedCommandList *command_list, int line, const char *text); 
 		bool deleteLyricsLetter(NedCommandList *command_list, int line);
 		bool setLyricsCursor(int line, int pos);
 		void selectNextChord(int line, bool lyrics_required);
@@ -158,6 +163,7 @@ class NedChordOrRest {
 		bool hitStemEnd(double x, double y);
 		double computeDistFrom(double x, double y, double factor, double *ydist);
 		bool tryErease (double x, double y, bool *removed);
+		void ereaseNote(NedNote *note);
 		bool testXShift(double x);
 		void moveToLine(int line);
 		bool scaleStem(double y, double old_ypos);
@@ -166,10 +172,12 @@ class NedChordOrRest {
 		double getOldStemLen() {return m_stemheight_copy;}
 		bool ReleaseTmpValue();
 		static int compareMidiTimes(NedChordOrRest *c1, NedChordOrRest *c2);
+		static int compareMidiTimesAndTypes(NedChordOrRest *c1, NedChordOrRest *c2);
 		unsigned int getTime() {return m_time;}
 		unsigned int getDuration(unsigned int measure_duration = -1);
 		unsigned int determineVolume();
 		double determineTempoInverse();
+		void recomputeFreeReplaceables();
 		void zoomFreeReplaceables(double zoom, double scale);
 		void determineTempoInverse(double *tempoinverse);
 		void setXPos(double pos);
@@ -196,6 +204,7 @@ class NedChordOrRest {
 		double getBottomOfChordOrRest();
 		bool testForTiesToDelete(NedCommandList *command_list, unsigned int dir = BREAK_TIE_BACKWARD, bool execute = false);
 		void testForDeleteableConstraints(NedCommandList *command_list);
+		void tieAllNotes(NedChordOrRest *other_chord);
 		static void handleOpenTies(NedCommandList *command_list, GList *pool);
 		bool isTied();
 		void tieCompleteTo(NedChordOrRest *other_chord);
@@ -233,14 +242,17 @@ class NedChordOrRest {
 		void saveTies(FILE *fp, bool *ties_written);
 		void saveFreePlaceables(FILE *fp, bool *free_placeables_written);
 		void addPossibleVolumeChange(GList **volume_changes);
+		void prepareMutedReplay();
+		void resetActiveFlags();
 		void prepareReplay(int clef, int keysig, int octave_shift, unsigned int grace_time);
 		void updateVolume(int staff_nr);
 		void setOffset(char offs_array[115]);
 		int getPitchOfFirstNote();
 		void setInternalPitch(int pitchdist);
-		void hideRests(NedCommandList *command_list, bool unhide);
+		void hideRest(NedCommandList *command_list, bool unhide);
 		void shiftNotes(int linedist);
 		void adjustAccidentals();
+		void changeAccidentals(NedCommandList *command_list, int preferred_offs);
 		void resetMicroshift() {m_micro_shift = 0.0;}
 		double compute_microshift(NedChordOrRest *right_chord, int mnum, int pos);
 		double compute_y_shift(NedChordOrRest *other_chord, int dir);
@@ -259,14 +271,18 @@ class NedChordOrRest {
 		void adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist);
 		char *getLyrics(int line);
 		void testNewMidiVolume(bool before);
-		void exportLilyPond(FILE *fp, int *last_line, unsigned int *midi_len, NedSlur **lily_slur, unsigned int *lyrics_map, bool *chordname);
-		void exportLilyChordnames(FILE *fp, int *last_line, unsigned int *midi_len);
+		void exportLilyPond(FILE *fp, int *last_line, unsigned int *midi_len, NedSlur **lily_slur, unsigned int *lyrics_map,
+			bool *guitar_chordnames, bool *chordname, int *breath_script, bool keep_beams);
+		void exportLilyGuitarChordnames(FILE *fp, int *last_line, unsigned int *midi_len);
+		void exportLilyFreeChordName(FILE *fp, int *last_line, unsigned int *midi_len);
 		void exportLilyLyrics(FILE *fp, int line_nr, int *sil_count);
+		void collectLyrics(NedLyricsEditor *leditor);
 	private:
 		void removeFreeDisplaceable(NedFreeReplaceable *freedisplaceable);
 		void addFreeDislaceable(NedFreeReplaceable *freedisplaceable);
 		void exportLilyAccents(FILE *fp);
-		void exportLilyFreePlaceables(FILE *fp, NedSlur **lily_slur, bool *chordnames);
+		void exportLilyFreePlaceablesBefore(FILE *fp);
+		void exportLilyFreePlaceablesAfter(FILE *fp, NedSlur **lily_slur, bool *guitar_chordnames, bool *chordnames);
 		GList *m_notes;
 		NedBeam *m_beam;
 		NedTuplet *m_tuplet_ptr;
@@ -299,6 +315,7 @@ class NedChordOrRest {
 		int m_slope_offs_copy;
 		unsigned int m_status_copy;
 		bool m_free_texts_present, m_lyrics_present;
+		bool m_has_open_highhat, m_has_closed_highhat;
 	friend class NedNewLyricsCommand;
 	friend class NedDeleteLyricsCommand;
 	friend class NedFreeReplaceable;
@@ -306,6 +323,7 @@ class NedChordOrRest {
 	friend class NedMoveNoteRelativeCommand;
 	friend class NedFlipValuesCommand;
 	friend class NedRemoveConstraintsCommand;
+	friend class NedChangeLyricsCommand;
 };
 
 #endif /* CHORD_OR_REST_H */
