@@ -27,7 +27,7 @@
 
 NedInsertStaffElemCommand::NedInsertStaffElemCommand(NedChordOrRest *staffelem, int clef_number_so_far, int octave_shift_so_far, NedStaff *staff, bool do_adjust) :
 m_staff(staff), m_staff_elem(staffelem), m_line_dist(0), m_do_adjust(do_adjust) {
-	if (m_staff_elem->getType() == TYPE_CLEF) {
+	if (m_staff_elem->getType() & (TYPE_CLEF)) {
 		m_line_dist = NedResource::getClefDist(clef_number_so_far, octave_shift_so_far,
 				m_staff_elem->getLength() /* clef_number */, m_staff_elem->getDotCount() /* octave_shift */);
 	}
@@ -46,7 +46,7 @@ void NedInsertStaffElemCommand::execute(bool adjust /* = false */) {
 	if (m_do_adjust && m_staff_elem->getType() == TYPE_KEYSIG) {
 		m_staff->getMainWindow()->adjustAccidentals();
 	}
-	else if (m_do_adjust && m_staff_elem->getType() == TYPE_CLEF) {
+	else if (m_do_adjust && (m_staff_elem->getType() & (TYPE_CLEF | TYPE_STAFF_SIGN))) {
 			
 		m_staff->getMainWindow()->shiftNotes(m_staff_elem->getMidiTime(), m_line_dist, m_staff->getPage(), m_staff->getSystem(), m_staff->getStaffNumber()); 
 	}
@@ -64,7 +64,7 @@ void NedInsertStaffElemCommand::unexecute(bool adjust /* = false */) {
 	if (m_do_adjust && m_staff_elem->getType() == TYPE_KEYSIG) {
 		m_staff->getMainWindow()->adjustAccidentals();
 	}
-	else if (m_do_adjust && m_staff_elem->getType() == TYPE_CLEF) {
+	else if (m_do_adjust && (m_staff_elem->getType() & (TYPE_CLEF | TYPE_STAFF_SIGN))) {
 		m_staff->getMainWindow()->shiftNotes(m_staff_elem->getMidiTime(), -m_line_dist, m_staff->getPage(), m_staff->getSystem(), m_staff->getStaffNumber()); 
 	}
 }
