@@ -39,12 +39,12 @@ _("F sharp Major; d sharp minor")
 };
 
 
-NedStaffContextDialog::NedStaffContextDialog(GtkWindow *parent, NedMainWindow *main_window,  bool allow_delete_system, int staff_count, int staff_pos, char *staff_name, char *staff_short_name, int clef_number, int octave_shift, int key_signature_number, int numerator, int demoninator, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int play_transposed, int reverb, bool muted) :
+NedStaffContextDialog::NedStaffContextDialog(GtkWindow *parent, NedMainWindow *main_window,  bool allow_delete_system, int staff_count, int staff_pos, char *staff_name, char *staff_short_name, char *group_name, char* group_short_name, int clef_number, int octave_shift, int key_signature_number, int numerator,  int demoninator, unsigned int symbol, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int play_transposed, int reverb, bool muted) :
 m_octave_shift(octave_shift), m_tempo(tempo), m_pan(pan), m_chorus(chorus), m_play_transposed(play_transposed), m_reverb(reverb), m_volume(volume), m_channel_value(channel),
-m_state(false), m_numerator_value(numerator), m_denominator_value(demoninator),
+m_state(false), m_numerator_value(numerator), m_denominator_value(demoninator), m_symbol(symbol),
 m_key_signature_number(key_signature_number), m_current_clef(clef_number), m_midi_program_number(midi_program), m_config_changed(false),
 m_delete_staff(false), m_staff_pos(staff_pos), 
-m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_short_name), m_muted(muted)  {
+m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_short_name), m_group_name(group_name), m_group_short_name(group_short_name), m_muted(muted)  {
 	GtkWidget *dialog;
 	GtkWidget *label_score;
 	GtkWidget *label_midi;
@@ -94,6 +94,9 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	GtkWidget *staff_name_frame;
 	GtkWidget *staff_short_name_frame;
 	GtkWidget *signature_name_vbox;
+	GtkWidget *group_name_frame;
+	GtkWidget *group_short_name_frame;
+	GtkWidget *group_name_vbox;
 	int i;
 	char Str[128];
 	m_newclef = m_current_clef;
@@ -204,6 +207,28 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 		gtk_combo_box_set_active(GTK_COMBO_BOX(m_denominator), 3);
 	}
 
+	m_com_time_bu = gtk_toggle_button_new();
+	gtk_button_set_image(GTK_BUTTON(m_com_time_bu),  gtk_image_new_from_stock("comm-icon", GTK_ICON_SIZE_SMALL_TOOLBAR));
+	m_cut_time_bu = gtk_toggle_button_new();
+	gtk_button_set_image(GTK_BUTTON(m_cut_time_bu),  gtk_image_new_from_stock("cut-icon", GTK_ICON_SIZE_SMALL_TOOLBAR));
+	g_signal_connect(m_com_time_bu, "toggled", G_CALLBACK (OnCommButtonToggled), (void *) this);
+	g_signal_connect(m_cut_time_bu, "toggled", G_CALLBACK (OnCutButtonToggled), (void *) this);
+
+	switch (symbol) {
+		case TIME_SYMBOL_COMMON_TIME: gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_com_time_bu), TRUE);
+						gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_numerator), 4);
+						gtk_combo_box_set_active(GTK_COMBO_BOX(m_denominator), 1);
+						gtk_widget_set_sensitive(m_numerator, FALSE);
+						gtk_widget_set_sensitive(m_denominator, FALSE);
+						break;
+		case TIME_SYMBOL_CUT_TIME: gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_cut_time_bu), TRUE);
+						gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_numerator), 2);
+						gtk_combo_box_set_active(GTK_COMBO_BOX(m_denominator), 0);
+						gtk_widget_set_sensitive(m_numerator, FALSE);
+						gtk_widget_set_sensitive(m_denominator, FALSE);
+						break;
+	}
+
 	staff_name_frame =  gtk_frame_new(_("staff name"));
 	m_staffname = gtk_entry_new_with_max_length(50);
 	if (m_staff_name != NULL) {
@@ -218,6 +243,20 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	}
 	gtk_container_add (GTK_CONTAINER(staff_short_name_frame), m_staffshortname);
 
+	group_name_frame =  gtk_frame_new(_("group name"));
+	m_groupname = gtk_entry_new_with_max_length(50);
+	if (m_group_name != NULL) {
+		gtk_entry_set_text(GTK_ENTRY(m_groupname), m_group_name);
+	}
+	gtk_container_add (GTK_CONTAINER(group_name_frame), m_groupname);
+
+	group_short_name_frame =  gtk_frame_new(_("group short name"));
+	m_groupshortname = gtk_entry_new_with_max_length(50);
+	if (m_group_short_name != NULL) {
+		gtk_entry_set_text(GTK_ENTRY(m_groupshortname), m_group_short_name);
+	}
+	gtk_container_add (GTK_CONTAINER(group_short_name_frame), m_groupshortname);
+
 
 	volume_frame = gtk_frame_new(_("volume"));
 	m_volume_scale = gtk_vscale_new_with_range(0.0, 127.0, 1.0);
@@ -244,7 +283,7 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	gtk_container_add (GTK_CONTAINER(chorus_frame), m_chorus_scale);
 
 	play_transposed_frame = gtk_frame_new(_("transp."));
-	m_play_transposed_scale = gtk_vscale_new_with_range(-12.0, 12.0, 1.0);
+	m_play_transposed_scale = gtk_vscale_new_with_range(-24.0, 24.0, 1.0);
 	gtk_range_set_inverted(GTK_RANGE(m_play_transposed_scale), TRUE);
 	gtk_range_set_value(GTK_RANGE(m_play_transposed_scale), m_play_transposed);
 	gtk_container_add (GTK_CONTAINER(play_transposed_frame), m_play_transposed_scale);
@@ -322,8 +361,10 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	}
 
 	gtk_box_pack_start (GTK_BOX (time_signature_upper_hbox), numerator_label, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (time_signature_upper_hbox), m_com_time_bu, FALSE, FALSE, 0);
 	gtk_box_pack_end (GTK_BOX (time_signature_upper_hbox), m_numerator, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (time_signature_lower_hbox), denominator_label, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (time_signature_lower_hbox), m_cut_time_bu, FALSE, FALSE, 0);
 	gtk_box_pack_end (GTK_BOX (time_signature_lower_hbox), m_denominator, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (time_signature_vbox), time_signature_upper_hbox, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (time_signature_vbox), time_signature_lower_hbox, FALSE, FALSE, 0);
@@ -334,11 +375,16 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	gtk_box_pack_start(GTK_BOX(signature_name_vbox), staff_name_frame, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(signature_name_vbox), staff_short_name_frame, FALSE, FALSE, 0);
 
+	group_name_vbox = gtk_vbox_new(FALSE, 2);
+	gtk_box_pack_start(GTK_BOX(group_name_vbox), group_name_frame, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(group_name_vbox), group_short_name_frame, FALSE, FALSE, 0);
+
 	score_hbox = gtk_hbox_new(FALSE, 2);
 	midi_hbox = gtk_hbox_new(FALSE, 2);
 	gtk_box_pack_start(GTK_BOX(score_hbox), clef_frame, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(score_hbox), key_frame, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(score_hbox), signature_name_vbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(score_hbox), group_name_vbox, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), volume_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), tempo_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), pan_frame, FALSE, TRUE, 0);
@@ -379,6 +425,7 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 	int newnum, newdenom, newkeysig, newtempo;
 	int newpan, newchorus, newplaytransposed, newreverb, new_octave_shift;
 	const char *newname, *newshortname;
+	const char *newgroupname, *newshortgroupname;
 	bool diff;
 
 	NedStaffContextDialog *context_dialog = (NedStaffContextDialog *) data;
@@ -495,6 +542,39 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 	else {
 		context_dialog->m_staff_short_name = NULL;
 	}
+	newgroupname = gtk_entry_get_text(GTK_ENTRY(context_dialog->m_groupname));
+	if (strlen(newgroupname) < 1) {
+		newgroupname = NULL;
+	}
+	diff = (newgroupname == NULL && context_dialog->m_group_name != NULL) || 
+		(newgroupname != NULL && context_dialog->m_group_name == NULL);
+	if (newgroupname  != NULL && context_dialog->m_group_name != NULL) {
+		diff = diff || strcmp(newgroupname, context_dialog->m_group_name) != 0;
+	}
+	if (diff) {
+		context_dialog->m_config_changed = TRUE;
+	}
+	if (newgroupname != NULL && strlen(newgroupname) > 0) {
+		context_dialog->m_group_name = strdup(newgroupname);
+	}
+	else {
+		context_dialog->m_group_name = NULL;
+	}
+	newshortgroupname = gtk_entry_get_text(GTK_ENTRY(context_dialog->m_groupshortname));
+	diff = (newshortgroupname == NULL && context_dialog->m_group_short_name != NULL) || 
+		(newshortgroupname != NULL && context_dialog->m_group_short_name == NULL);
+	if (newshortgroupname  != NULL && context_dialog->m_group_short_name != NULL) {
+		diff = diff || strcmp(newshortgroupname, context_dialog->m_group_short_name) != 0;
+	}
+	if (diff) {
+		context_dialog->m_config_changed = TRUE;
+	}
+	if (newshortgroupname != NULL && strlen(newshortgroupname) > 0) {
+		context_dialog->m_group_short_name = strdup(newshortgroupname);
+	}
+	else {
+		context_dialog->m_group_short_name = NULL;
+	}
 	selection =  gtk_tree_view_get_selection (GTK_TREE_VIEW(context_dialog->m_instrument_list));
 	selected_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
 	if ( g_list_length(selected_rows) > 0) {
@@ -538,10 +618,21 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 		context_dialog->m_channel_value = newchannel;
 		context_dialog->m_config_changed = true;
 	}
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_com_time_bu))) {
+		context_dialog->m_symbol = TIME_SYMBOL_COMMON_TIME;
+		context_dialog->m_numerator_value = context_dialog->m_denominator_value = 4;
+	}
+	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_cut_time_bu))) {
+		context_dialog->m_symbol = TIME_SYMBOL_CUT_TIME;
+		context_dialog->m_numerator_value = context_dialog->m_denominator_value = 2;
+	}
+	else {
+		context_dialog->m_symbol = TIME_SYMBOL_NONE;
+	}
 	gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 
-void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, bool *delete_system, int *staff_pos, char **staff_name, char **staff_short_name, int *clef_number, int *octave_shift, int *key_signature_number, bool *ajdust_notes, int *numerator, int *demoninator, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *play_transposed, int *reverb, bool *config_changed, bool *muted) {
+void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, bool *delete_system, int *staff_pos, char **staff_name, char **staff_short_name, char **group_name, char **group_short_name, int *clef_number, int *octave_shift, int *key_signature_number, bool *ajdust_notes, int *numerator, int *demoninator,  unsigned int *symbol, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *play_transposed, int *reverb, bool *config_changed, bool *muted) {
 	*delete_staff = m_delete_staff;
 	*delete_system = m_delete_system;
 	*state = m_state;
@@ -553,6 +644,8 @@ void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, bool *del
 	*ajdust_notes = m_do_adjust;
 	*staff_name = m_staff_name;
 	*staff_short_name = m_staff_short_name;
+	*group_name = m_group_name;
+	*group_short_name = m_group_short_name;
 	*clef_number = m_current_clef;
 	*key_signature_number = m_key_signature_number;
 	*numerator = m_numerator_value;
@@ -567,6 +660,7 @@ void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, bool *del
 	*midi_program = m_midi_program_number;
 	*channel = m_channel_value;
 	*muted = m_muted;
+	*symbol = m_symbol;
 	if (m_state) {
 		*config_changed = m_config_changed;
 	}
@@ -576,7 +670,7 @@ void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, bool *del
 }
 
 gboolean NedStaffContextDialog::draw_clefs(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
-#define ZOOM_LEVEL 8
+#define ZOOM_LEVEL 9
 	NedStaffContextDialog *the_dialog = (NedStaffContextDialog *) data;
 	cairo_scaled_font_t *scaled_font;
 	cairo_glyph_t glyph;
@@ -652,8 +746,8 @@ gboolean NedStaffContextDialog::draw_clefs(GtkWidget *widget, GdkEventExpose *ev
 				  }
 				glyph.index = BASE + 1; y_offs = -DEMO_LINE_DIST; break;
 		case NEUTRAL_CLEF1: 
-				glyph.index = BASE + 27; break;
 		case NEUTRAL_CLEF2: 
+		case NEUTRAL_CLEF3: 
 				glyph.index = BASE + 27; break;
 	}
 	glyph.x = 50.0;
@@ -670,6 +764,11 @@ gboolean NedStaffContextDialog::draw_clefs(GtkWidget *widget, GdkEventExpose *ev
 					glyph.y = 120.0;
 					cairo_show_glyphs(cr, &glyph, 1);
 					break;
+		case NEUTRAL_CLEF3: glyph.index = NUMBERBASE + 3; 
+					glyph.x = 60.0;
+					glyph.y = 120.0;
+					cairo_show_glyphs(cr, &glyph, 1);
+					break;
 	}
 	if (octave_shift) {
 		glyph.x = 50.0 + oct_x_offs;
@@ -744,3 +843,36 @@ void NedStaffContextDialog::octave_shift_change(GtkSpinButton *spinbutton, gpoin
 	rect.height = context_dialog->m_clef_show_window->allocation.height;
 	gdk_window_invalidate_rect (context_dialog->m_clef_show_window->window, &rect, FALSE);
 }
+
+void NedStaffContextDialog::OnCommButtonToggled(GtkToggleButton *togglebutton, gpointer data) {
+	NedStaffContextDialog *context_dialog = (NedStaffContextDialog *) data;
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_com_time_bu))) {
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context_dialog->m_cut_time_bu), FALSE);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(context_dialog->m_numerator), 4);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(context_dialog->m_denominator), 1);
+		gtk_widget_set_sensitive(context_dialog->m_numerator, FALSE);
+		gtk_widget_set_sensitive(context_dialog->m_denominator, FALSE);
+	}
+	else if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_cut_time_bu))) {
+		gtk_widget_set_sensitive(context_dialog->m_numerator, TRUE);
+		gtk_widget_set_sensitive(context_dialog->m_denominator, TRUE);
+	}
+}
+
+void NedStaffContextDialog::OnCutButtonToggled(GtkToggleButton *togglebutton, gpointer data) {
+	NedStaffContextDialog *context_dialog = (NedStaffContextDialog *) data;
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_cut_time_bu))) {
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context_dialog->m_com_time_bu), FALSE);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(context_dialog->m_numerator), 2);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(context_dialog->m_denominator), 0);
+		gtk_widget_set_sensitive(context_dialog->m_numerator, FALSE);
+		gtk_widget_set_sensitive(context_dialog->m_denominator, FALSE);
+	}
+	else if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_com_time_bu))) {
+		gtk_widget_set_sensitive(context_dialog->m_numerator, TRUE);
+		gtk_widget_set_sensitive(context_dialog->m_denominator, TRUE);
+	}
+}
+
