@@ -34,13 +34,15 @@ class NedMainWindow;
 
 class NedStaffContextDialog {
 	public:
-		NedStaffContextDialog(GtkWindow *parent,  NedMainWindow *main_window, bool allow_delete_system, int staff_count, int staff_pos, char *staff_name, char *staffshortname, int clef_number, int octave_shift, int key_signature_number, int numerator, int demoninator, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int play_transposed, int reverb, bool muted);
-		void getValues(bool *state, bool *delete_staff, bool *delete_system, int *staff_pos, char **staff_name, char **staff_short_name, int *clef_number, int *octave_shift, int *key_signature_number, bool *ajdust_notes, int *numerator, int *demoninator, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *play_transposed, int *reverb, bool *config_changed, bool *muted);
+		NedStaffContextDialog(GtkWindow *parent,  NedMainWindow *main_window, bool allow_delete_system, int staff_count, int staff_pos, char *staff_name, char *staffshortname, char *group_name, char* group_short_name, int clef_number, int octave_shift, int key_signature_number, int numerator, int demoninator, unsigned int symbol, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int play_transposed, int reverb, bool muted);
+		void getValues(bool *state, bool *delete_staff, bool *delete_system, int *staff_pos, char **staff_name, char **staff_short_name, char **group_name, char **group_short_name, int *clef_number, int *octave_shift, int *key_signature_number, bool *ajdust_notes, int *numerator, int *demoninator, unsigned int *symbol, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *play_transposed, int *reverb, bool *config_changed, bool *muted);
 	private:
 		static void OnClose(GtkDialog *dialog, gint result, gpointer data);
 		static gboolean OnScroll(GtkWidget *widget, GdkEventScroll *event, gpointer data);
 		static gboolean draw_clefs(GtkWidget *widget,
 			 GdkEventExpose *event, gpointer data);
+		static void OnCommButtonToggled(GtkToggleButton *togglebutton, gpointer data);
+		static void OnCutButtonToggled(GtkToggleButton *togglebutton, gpointer data);
 		static void decrease_clef_number (GtkButton *button, gpointer user_data);
 		static void increase_clef_number (GtkButton *button, gpointer user_data);
 		static void octave_shift_change(GtkSpinButton *spinbutton, gpointer user_data);
@@ -49,6 +51,8 @@ class NedStaffContextDialog {
 		GtkWidget *m_sig_radio_buttons[14];
 		GtkWidget *m_denominator;
 		GtkWidget *m_numerator;
+		GtkWidget *m_com_time_bu;
+		GtkWidget *m_cut_time_bu;
 		GtkWidget *m_clef_bu_up;
 		GtkWidget *m_clef_bu_down;
 		GtkWidget *m_instrument_list;
@@ -65,6 +69,8 @@ class NedStaffContextDialog {
 		GtkWidget *m_position_list;
 		GtkWidget *m_staffname;
 		GtkWidget *m_staffshortname;
+		GtkWidget *m_groupname;
+		GtkWidget *m_groupshortname;
 		GtkWidget *m_table;
 		GtkObject *m_instrument_adjustment;
 		int m_octave_shift;
@@ -78,6 +84,8 @@ class NedStaffContextDialog {
 		bool m_state;
 		int m_numerator_value;
 		int m_denominator_value;
+		unsigned int m_symbol;
+		int m_time_symbol;
 		int m_key_signature_number;
 		int m_current_clef, m_newclef;
 		bool m_do_adjust;
@@ -89,9 +97,12 @@ class NedStaffContextDialog {
 		int m_staff_count;
 		char *m_staff_name;
 		char *m_staff_short_name;
+		char *m_group_name;
+		char *m_group_short_name;
 		bool m_muted;
 
 	friend class NedKeysigDialog;
+	friend class NedMidiRecordConfigDialog;
 
 		
 };
