@@ -74,6 +74,8 @@
 #define MORDENT_BUTTON       [2][12]
 #define ARPEGGIO_BUTTON      [2][13]
 
+#define O_HIGH_NOTE_BUTTON   [3][ 2]
+#define C_HIGH_NOTE_BUTTON   [3][ 3]
 #define NORMAL_NOTE_BUTTON   [3][ 4]
 #define CROSS_NOTE1_BUTTON   [3][ 5]
 #define CROSS_NOTE2_BUTTON   [3][ 6]
@@ -107,7 +109,7 @@ NedTools::button_descr NedTools::m_mini_button_descriptions[4] = {
 NedTools::button_descr NedTools::m_button_descriptions[NUM_PANELS][17] = {
    {
 	{ &nullbuf, EMPTY_BUTTON, SCOPE_NONE, false, NULL }, { &nullbuf, EMPTY_BUTTON, SCOPE_NONE, false, NULL },
-		{&NedResource::m_64thnote_icon, TOGGLE_BUTTON, SCOPE_NOTE_LENGTH, false, &m_note_length_group}, {&NedResource::m_arrow_right_icon, PUSH_BUTTON, false, NULL }, 
+		{&NedResource::m_64thnote_icon, TOGGLE_BUTTON, SCOPE_NOTE_LENGTH, false, &m_note_length_group}, {&NedResource::m_arrow_right_icon, PUSH_BUTTON, SCOPE_NONE, false, NULL }, 
 	{&NedResource::m_natural_icon, TOGGLE_BUTTON, SCOPE_NOTE_STATE, false, &m_accidentals}, {&NedResource::m_cross_icon, TOGGLE_BUTTON, SCOPE_NOTE_STATE, false, &m_accidentals},
 		{&NedResource::m_flat_icon, TOGGLE_BUTTON, SCOPE_NOTE_STATE, false, &m_accidentals}, {&NedResource::m_fermata_icon, TOGGLE_BUTTON, SCOPE_STATE, false, NULL},
 	{&NedResource::m_fullnote_icon, TOGGLE_BUTTON, SCOPE_NOTE_LENGTH, false, &m_note_length_group}, {&NedResource::m_halfnote_icon, TOGGLE_BUTTON, SCOPE_NOTE_LENGTH, false, &m_note_length_group},
@@ -143,7 +145,7 @@ NedTools::button_descr NedTools::m_button_descriptions[NUM_PANELS][17] = {
 
    {
 	{ &nullbuf, EMPTY_BUTTON, SCOPE_NONE, false, NULL }, {&NedResource::m_arrow_left_icon, PUSH_BUTTON, SCOPE_NONE, false, NULL }, 
-		{ &nullbuf, EMPTY_BUTTON, SCOPE_NONE, false, NULL }, { &nullbuf, EMPTY_BUTTON, SCOPE_NONE, false, NULL },
+		{ &NedResource::m_drum11_icon, TOGGLE_BUTTON, SCOPE_NOTE_HEAD, false, &m_note_head_group}, { &NedResource::m_drum12_icon, TOGGLE_BUTTON, SCOPE_NOTE_HEAD, false, &m_note_head_group },
 	{&NedResource::m_normal_note_icon, TOGGLE_BUTTON, SCOPE_NOTE_HEAD, true, &m_note_head_group}, {&NedResource::m_drum1_icon, TOGGLE_BUTTON, SCOPE_NOTE_HEAD, false, &m_note_head_group},
 		{&NedResource::m_drum2_icon, TOGGLE_BUTTON, SCOPE_NOTE_HEAD, false, &m_note_head_group}, {&NedResource::m_drum3_icon, TOGGLE_BUTTON, SCOPE_NOTE_HEAD, false, &m_note_head_group},
 	{&NedResource::m_drum4_icon, TOGGLE_BUTTON, SCOPE_NOTE_HEAD, false, &m_note_head_group}, {&NedResource::m_drum5_icon, TOGGLE_BUTTON, SCOPE_NOTE_HEAD, false, &m_note_head_group},
@@ -249,6 +251,18 @@ bool X11button::handle_release(gdouble xd, gdouble yd) {
 
 X11buttonGroup::X11buttonGroup(bool all_inactive_allowed) : m_buttons(NULL), m_all_inactive_allowed(all_inactive_allowed) {}
 
+void X11buttonGroup::removeAllButtons()  {
+	if (m_buttons != NULL) {
+/*
+		while ((lptr = g_list_first(m_buttons)) != NULL) {
+			delete (X11button *) lptr->data;
+		}
+*/
+		g_list_free(m_buttons);
+		m_buttons = NULL;
+	}
+}
+
 void X11buttonGroup::addX11button(X11button *button) {
 	if (button->m_button_group != NULL) {
 		NedResource::Abort("X11buttonGroup::addX11button(1)");
@@ -320,16 +334,21 @@ void X11buttonGroup::resetAllButtons() {
 #define CREATE_X11_EMPTY(back, up) createX11PixmapFromGdkPixbuffer(dpy, screen, d, ggc, NULL, back, up);
 
 NedTools::NedTools() :
-	m_current_row(-1), m_current_column(-1), m_current_panel(0) {
+	m_back_pix(NULL), m_current_row(-1), m_current_column(-1), m_current_panel(0), m_buttons_created(false)
+		 {
 	GdkColor bgcolor;
+	int wx, wy;
+	NedMainWindow *last_main;
+	GdkDisplay *display = NULL;
+	GdkScreen *screen = NULL;
 
 	m_toolbox = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title (GTK_WINDOW (m_toolbox), _("Toolbox"));
+    	g_signal_connect (m_toolbox, "delete-event", G_CALLBACK (close_toolbox), this);
 
 	
 	g_signal_connect(m_toolbox, "delete-event", G_CALLBACK (OnDelete), (void *) this);
-        g_signal_connect (m_toolbox, "key-press-event",
-		      G_CALLBACK (key_press_handler), (void *) this);
+        g_signal_connect (m_toolbox, "key-press-event", G_CALLBACK (key_press_handler), (void *) this);
 
 	m_back_img_width = gdk_pixbuf_get_width(NedResource::m_background1);
 	m_back_img_height = gdk_pixbuf_get_height(NedResource::m_background1);
@@ -337,20 +356,31 @@ NedTools::NedTools() :
 	m_back_miniimg_height = gdk_pixbuf_get_height(NedResource::m_mini_background1);
 	m_mini_spaces = (4 * m_back_img_width - 4 * m_back_miniimg_width) / 5;
 
+	if ((last_main = NedResource::getMainwWindowWithLastFocus()) != NULL) {
+		display = gdk_display_get_default ();
+		screen = gdk_display_get_default_screen (display);
+		gdk_window_get_root_origin(GDK_WINDOW(last_main->getWindow()->window), &wx, &wy);
+		gtk_window_move(GTK_WINDOW (m_toolbox), wx + NedResource::m_toolboxX, wy + NedResource::m_toolboxY);
+	}
 
 	m_button_frame = gtk_frame_new(_("Toolbox"));
 	m_button_area = gtk_drawing_area_new ();
 	bgcolor.pixel = 1;
 	bgcolor.red = 0x0000; bgcolor.green =  0x5555; bgcolor.blue =  0xcccc;
 	gtk_widget_modify_bg(GTK_WIDGET(m_button_area), GTK_STATE_NORMAL, &bgcolor);
+        gtk_widget_set_double_buffered(m_button_area, FALSE);
 	gtk_drawing_area_size(GTK_DRAWING_AREA(m_button_area), 4 * m_back_img_width, MINI_BUTTON_AREA_HEIGHT + 5 * m_back_img_height);
+	
+	
 	gtk_container_add (GTK_CONTAINER(m_button_frame), m_button_area);
 
-	g_signal_connect (m_button_frame, "expose-event", G_CALLBACK (draw_menu), (void *) this);
+	g_signal_connect (m_button_area, "expose-event", G_CALLBACK (draw_menu), (void *) this);
 	gtk_widget_add_events(m_button_area, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK);
 	g_signal_connect (m_button_area, "button-press-event", G_CALLBACK (OnButtonPress), (void *) this);
 	g_signal_connect (m_button_area, "button-release-event", G_CALLBACK (OnButtonRelease), (void *) this);
     	gtk_container_add (GTK_CONTAINER (m_toolbox), m_button_frame);
+	gtk_window_set_resizable (GTK_WINDOW(m_toolbox), FALSE);
+    	gtk_widget_show_all (m_toolbox);
 }
 
 
@@ -364,15 +394,49 @@ NedTools::~NedTools() {
 		m_button_arrays[panel][bu_nr] = NULL;
           }
 	}
+	m_accidentals.removeAllButtons();
+	m_note_length_group.removeAllButtons();
+	m_note_head_group.removeAllButtons();
+	m_dot_group.removeAllButtons();
 }
 
+void NedTools::updatePosition() {
+	NedMainWindow *last_main;
+	int wx, wy, tx, ty;
+
+	if ((last_main = NedResource::getMainwWindowWithLastFocus()) != NULL) {
+		gdk_window_get_root_origin(GDK_WINDOW(last_main->getWindow()->window), &wx, &wy);
+		gdk_window_get_root_origin(GDK_WINDOW(m_toolbox->window), &tx, &ty);
+		NedResource::m_toolboxX = tx - wx;
+		NedResource::m_toolboxY = ty - wy;
+	}
+}
+
+bool NedTools::close_toolbox(GtkWidget *widget, GdkEvent  *event, gpointer data) {
+	NedTools *tools = (NedTools *) data;
+	tools->updatePosition();
+
+	return FALSE;
+}
+
+
 void NedTools::Show() {
 	gtk_widget_show_all (m_toolbox);
 }
 
 
 void NedTools::setToolboxTransient(NedMainWindow *m_win) {
+	int wx, wy;
+	GdkDisplay *display = NULL;
+	GdkScreen *screen = NULL;
+
 	gtk_window_set_transient_for(GTK_WINDOW (m_toolbox), (GTK_WINDOW (m_win->getWindow())));
+
+	display = gdk_display_get_default ();
+	screen = gdk_display_get_default_screen (display);
+	gdk_window_get_root_origin(GDK_WINDOW(m_win->getWindow()->window), &wx, &wy);
+	gtk_window_move(GTK_WINDOW (m_toolbox), wx + NedResource::m_toolboxX, wy + NedResource::m_toolboxY);
+
 }
 
 unsigned int NedTools::getCurrentLength() {
@@ -394,6 +458,8 @@ unsigned int NedTools::getCurrentLength() {
 int NedTools::getCurrentNoteHead() {
 	if (m_button_arrays[0][0] == NULL) return NORMAL_NOTE;
 
+	if (m_button_arrays O_HIGH_NOTE_BUTTON ->m_active) return OPEN_HIGH_HAT;
+	if (m_button_arrays C_HIGH_NOTE_BUTTON ->m_active) return CLOSED_HIGH_HAT;
 	if (m_button_arrays NORMAL_NOTE_BUTTON ->m_active) return NORMAL_NOTE;
 	if (m_button_arrays CROSS_NOTE1_BUTTON ->m_active) return CROSS_NOTE1;
 	if (m_button_arrays CROSS_NOTE2_BUTTON ->m_active) return CROSS_NOTE2;
@@ -454,8 +520,10 @@ int NedTools::getDotCount() {
 
 bool NedTools::getTiedMode() {
 	if (m_button_arrays[0][0] == NULL) return 0;
+	bool ret;
 
-	return m_button_arrays TIE_BUTTON ->m_active;
+	ret = m_button_arrays TIE_BUTTON ->m_active;
+	return ret;
 }
 
 void NedTools::setButtons(unsigned int length, int dotcount, unsigned int chord_status, unsigned int note_status, bool isRest, bool tie_forward, int note_head) {
@@ -511,6 +579,8 @@ void NedTools::setButtons(unsigned int length, int dotcount, unsigned int chord_
 	else m_button_arrays QUARTER_NOTE_BUTTON ->m_active = true;
 
 	if (note_head == NORMAL_NOTE) m_button_arrays NORMAL_NOTE_BUTTON ->m_active = true;
+	else if (note_head == OPEN_HIGH_HAT) m_button_arrays O_HIGH_NOTE_BUTTON ->m_active = true;
+	else if (note_head == CLOSED_HIGH_HAT) m_button_arrays C_HIGH_NOTE_BUTTON ->m_active = true;
 	else if (note_head == CROSS_NOTE1) m_button_arrays CROSS_NOTE1_BUTTON ->m_active = true;
 	else if (note_head == CROSS_NOTE2) m_button_arrays CROSS_NOTE2_BUTTON ->m_active = true;
 	else if (note_head == RECT_NOTE1) m_button_arrays RECT_NOTE1_BUTTON ->m_active = true;
@@ -526,7 +596,7 @@ void NedTools::setButtons(unsigned int length, int dotcount, unsigned int chord_
 	else m_button_arrays NORMAL_NOTE_BUTTON ->m_active = true;
 
 	m_button_arrays TIE_BUTTON ->m_active = tie_forward;
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 
 }
 
@@ -553,7 +623,7 @@ void NedTools::setNoteLengthButtons(unsigned int length) {
 	else if (length == NOTE_64) m_button_arrays NOTE_64_BUTTON ->m_active = true;
 	else m_button_arrays QUARTER_NOTE_BUTTON ->m_active = true;
 
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 
 }
 
@@ -569,6 +639,8 @@ void NedTools::resetSomeButtons() {
 	m_button_arrays DDOT_BUTTON ->m_active = false;
 	m_button_arrays DOT_BUTTON ->m_active = false;
 
+	m_button_arrays TIE_BUTTON ->m_active = false;
+
 	m_button_arrays FERMATA_BUTTON ->m_active = false;
 	m_button_arrays STACCATO_BUTTON ->m_active = false;
 	m_button_arrays STACCATISSIMO_BUTTON ->m_active = false;
@@ -584,7 +656,13 @@ void NedTools::resetSomeButtons() {
 	m_button_arrays OPEN_BUTTON ->m_active = false;
 	m_button_arrays MORDENT_BUTTON ->m_active = false;
 	m_button_arrays ARPEGGIO_BUTTON ->m_active = false;
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
+}
+
+void NedTools::resetTieButton() {
+	if (m_button_arrays[0][0] == NULL) return;
+	m_button_arrays TIE_BUTTON ->m_active = false;
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 void NedTools::resetButtons() {
@@ -596,29 +674,29 @@ void NedTools::resetButtons() {
 			m_button_arrays[p][i]->m_active = m_button_descriptions[p][i].m_default_state;
 		}
 	}
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 void NedTools::toggleButton(int nr) {
 	m_button_arrays[m_current_panel][nr]->toggle();
 	testForPanelChange(m_button_arrays[m_current_panel][nr]);
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 void NedTools::toggleRest() {
 	m_button_arrays REST_BUTTON ->m_active = !(m_button_arrays REST_BUTTON ->m_active);
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 void NedTools::toggleTie() {
 	m_button_arrays TIE_BUTTON ->m_active = !(m_button_arrays TIE_BUTTON ->m_active);
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 void NedTools::toggleDot() {
 	m_button_arrays DOT_BUTTON ->m_active = !(m_button_arrays DOT_BUTTON ->m_active);
 	if (m_button_arrays DOT_BUTTON ->m_active) m_button_arrays DDOT_BUTTON ->m_active = false;
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 void NedTools::toggleSharp() {
@@ -629,7 +707,7 @@ void NedTools::toggleSharp() {
 		m_button_arrays DSHARP_BUTTON ->m_active = false;
 		m_button_arrays DFLAT_BUTTON ->m_active = false;
 	}
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 void NedTools::toggleNatural() {
@@ -640,7 +718,7 @@ void NedTools::toggleNatural() {
 		m_button_arrays DSHARP_BUTTON ->m_active = false;
 		m_button_arrays DFLAT_BUTTON ->m_active = false;
 	}
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 
@@ -652,7 +730,7 @@ void NedTools::toggleFlat() {
 		m_button_arrays DSHARP_BUTTON ->m_active = false;
 		m_button_arrays DFLAT_BUTTON ->m_active = false;
 	}
-	gdk_window_invalidate_rect(m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(m_button_area->window, NULL, FALSE);
 }
 
 
@@ -784,6 +862,7 @@ void NedTools::createButtons(Display *dpy, Window win, GC xgc, GdkGC *ggc,  GdkD
 		determine_image_and_dimensions(dpy, win, ggc, d, screen, panel, row, column, bu_nr, *(m_button_descriptions[panel][bu_nr].m_pixbuf), img, &w, &h);
 		m_button_arrays[panel][bu_nr] = new X11button(dpy, win, xgc, screen, img, column * m_back_img_width, MINI_BUTTON_AREA_HEIGHT+ row * m_back_img_height,
 				w, h, column, row, panel, m_button_descriptions[panel][bu_nr].m_type, m_button_descriptions[panel][bu_nr].m_scope, m_button_descriptions[panel][bu_nr].m_type != EMPTY_BUTTON, m_button_descriptions[panel][bu_nr].m_default_state);
+
 		if (m_button_descriptions[panel][bu_nr].m_button_group != NULL) {
 			m_button_descriptions[panel][bu_nr].m_button_group->addX11button(m_button_arrays[panel][bu_nr]);
 		}
@@ -823,19 +902,33 @@ gboolean NedTools::draw_menu(GtkWidget *widget, GdkEventExpose *event, gpointer
 	Display *dpy;
 	Window win;
 	GC xgc;
+	GdkColor bgcolor;
 	int screen;
 	int bu_nr;
 	
 	if (m_button_arrays[0][0] == NULL) {
-		dpy = gdk_x11_drawable_get_xdisplay(the_dialog->m_button_area->window);
-		win = gdk_x11_drawable_get_xid(the_dialog->m_button_area->window);
+		the_dialog->m_back_pix = gdk_pixmap_new(the_dialog->m_button_area->window, 
+	4 * the_dialog->m_back_img_width, MINI_BUTTON_AREA_HEIGHT + 5 * the_dialog->m_back_img_height,
+		-1);
+		the_dialog->m_copy_gc = gdk_gc_new(the_dialog->m_button_area->window);
+		the_dialog->m_blue_gc = gdk_gc_new(the_dialog->m_back_pix);
+		bgcolor.pixel = 1;
+		bgcolor.red = 0x0000; bgcolor.green =  0x5555; bgcolor.blue =  0xcccc;
+		gdk_gc_set_rgb_fg_color (the_dialog->m_blue_gc, &bgcolor);
+		dpy = gdk_x11_drawable_get_xdisplay(the_dialog->m_back_pix);
+		win = gdk_x11_drawable_get_xid(the_dialog->m_back_pix);
 		screen = DefaultScreen(dpy);
 		xgc = DefaultGC(dpy, screen);
-		the_dialog->createButtons(dpy, win, xgc, the_dialog->m_button_area->style->white_gc, the_dialog->m_button_area->window, screen);
-		the_dialog->createMiniButtons(dpy, win, xgc, the_dialog->m_button_area->style->white_gc, the_dialog->m_button_area->window, screen);
-
+		the_dialog->createButtons(dpy, win, xgc, the_dialog->m_button_area->style->white_gc, the_dialog->m_back_pix, screen);
+		the_dialog->createMiniButtons(dpy, win, xgc, the_dialog->m_button_area->style->white_gc, the_dialog->m_back_pix, screen);
 	}
 
+	if (the_dialog->m_back_pix == NULL) return TRUE;
+
+        gdk_draw_rectangle(the_dialog->m_back_pix, the_dialog->m_blue_gc,
+			TRUE, 0, 0,
+	4 * the_dialog->m_back_img_width, MINI_BUTTON_AREA_HEIGHT + 5 * the_dialog->m_back_img_height);
+
 	for (bu_nr = 0; bu_nr < 17; bu_nr++) {
 		m_button_arrays[the_dialog->m_current_panel][bu_nr]->draw();
 	}
@@ -843,7 +936,13 @@ gboolean NedTools::draw_menu(GtkWidget *widget, GdkEventExpose *event, gpointer
 		m_mini_button_arrays[bu_nr]->draw();
 	}
 
-	return FALSE;
+	GdkImage *img1 =  gdk_drawable_get_image(the_dialog->m_back_pix, 0, 0,
+	4 * the_dialog->m_back_img_width, MINI_BUTTON_AREA_HEIGHT + 5 * the_dialog->m_back_img_height);
+
+
+	gdk_draw_image(the_dialog->m_button_area->window, the_dialog->m_copy_gc, img1, 0, 0, 0, 0,
+	4 * the_dialog->m_back_img_width, MINI_BUTTON_AREA_HEIGHT + 5 * the_dialog->m_back_img_height);
+	return TRUE;
 }
 
 
@@ -910,7 +1009,6 @@ XImage *NedTools::createX11PixmapFromGdkPixbuffer(Display *dpy, int screen, GdkD
 }
 
 bool NedTools::OnButtonPress(GtkWidget *widget, GdkEventButton *event, gpointer data) {
-
 	NedTools *the_dialog = (NedTools *) data;
 	int bu_nr;
 
@@ -920,7 +1018,7 @@ bool NedTools::OnButtonPress(GtkWidget *widget, GdkEventButton *event, gpointer
 	for (bu_nr = 0; bu_nr < 4; bu_nr++) {
 		m_mini_button_arrays[bu_nr]->handle_click(event->x, event->y);
 	}
-	gdk_window_invalidate_rect(the_dialog->m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(the_dialog->m_button_area->window, NULL, FALSE);
 	return FALSE;
 }
 
@@ -938,7 +1036,7 @@ bool NedTools::OnButtonRelease(GtkWidget *widget, GdkEventButton *event, gpointe
 			the_dialog->m_current_panel = bu_nr;
 		}
 	}
-	gdk_window_invalidate_rect(the_dialog->m_button_frame->window, NULL, FALSE);
+	gdk_window_invalidate_rect(the_dialog->m_button_area->window, NULL, FALSE);
 	return FALSE;
 }
 
