@@ -30,6 +30,8 @@
 #include "resource.h"
 #include "paths.h"
 
+//#define WITH_BANNER
+
 class NedPage;
 class NedNote;
 class NedCommandHistory;
@@ -40,6 +42,8 @@ class NedStaff;
 class NedFreeReplaceable;
 class NedFreeText;
 class NedImporter;
+class NedMidiRecorder;
+class NedLyricsEditor;
 class NedMusicXMLImport;
 class NedPangoCairoText;
 class ScoreInfo;
@@ -48,6 +52,8 @@ struct paper_info_struct;
 struct staff_context_str {
 	NedPangoCairoText *m_staff_name;
 	NedPangoCairoText *m_staff_short_name;
+	NedPangoCairoText *m_group_name;
+	NedPangoCairoText *m_group_short_name;
 	int m_key_signature_number;
 	int m_clef_number;
 	int m_clef_octave_shift;
@@ -59,6 +65,7 @@ struct staff_context_str {
 	char m_play_transposed;
 	unsigned char m_midi_program;
 	unsigned char m_midi_channel;
+	unsigned char m_effective_channel;
 	unsigned int m_flags;
 #define BRACE_START   (1 << 0)
 #define BRACE_END     (1 << 1)
@@ -77,9 +84,10 @@ struct staff_context_str {
 
 class SpecialMeasure {
 	public:
-		SpecialMeasure() {type = 0; measure = NULL;}
-		void setTimeSignature(unsigned short num, unsigned short denom) {
+		SpecialMeasure() {type = 0; measure = NULL; symbol = TIME_SYMBOL_NONE;}
+		void setTimeSignature(unsigned short num, unsigned short denom, unsigned int sym) {
 			numerator = num; denominator = denom;
+			symbol = sym;
 			measure_duration = numerator * WHOLE_NOTE / denominator;
 		}
 #define FIELD_BITS 8
@@ -109,6 +117,7 @@ class SpecialMeasure {
 		unsigned int measure_duration;
 		NedMeasure *measure;
 		unsigned short numerator, denominator;
+		unsigned int symbol;
 		bool isNear(double x, double y);
 };
 
@@ -131,8 +140,11 @@ typedef struct {
 	char *root_name;
 	char *up_name;
 	char *down_name;
+	double fontsize;
+	double ydist;
 } chord_name_info_struct;
 
+
 typedef union {
 	int m_special_sub_type_int;
 	chord_info_struct m_chord_info;
@@ -153,6 +165,7 @@ class NedMainWindow {
 		bool getShowHidden();
 		bool hasFocus();
 		int getLyricsMode();
+		void collectLyrics(NedLyricsEditor *leditor, int staff_nr);
 		NedCommandHistory *getCommandHistory() {return m_command_history;}
 		double getLeftX();
 		double getTopY();
@@ -182,6 +195,7 @@ class NedMainWindow {
 		int getDenominator() {return m_denominator;}
 		bool isTimsigChangingMeasure(int meas_num);
 		unsigned int getMeasureDuration(int meas_num);
+		double getGlopabSpreadFac() {return m_global_spacement_fac;}
 		bool findTimeOfMeasure(int meas_num, unsigned long long *meas_time);
 		void changeLength();
 		void changeState();
@@ -191,6 +205,10 @@ class NedMainWindow {
 		void changeNoteState();
 		int getNumerator(int meas_num);
 		int getDenominator(int meas_num);
+		unsigned int getTimesigSymbol(int meas_num);
+		unsigned int getTimesigSymbolOfThisMeasure(int meas_num);
+		int getNumeratorOfThisMeasure(int meas_num);
+		int getDenominatorOfThisMeasure(int meas_num);
 		int getPreStaffCount() {return m_pre_staffcount;}
 		void stopReplay();
 		void setVisibleSystem(NedSystem *system);
@@ -207,9 +225,11 @@ class NedMainWindow {
 		int getStaffCount() {return m_staff_count;}
 		void do_remove_last_staff();
 		bool do_import_from_other(NedImporter *im);
-		void do_lily_export(FILE *fp, bool with_break, bool *selected_staves);
+		void do_lily_export(FILE *fp, bool with_break, bool *selected_staves, bool *kept_beams);
 		void do_transpose(int pitchdist, bool *staff_list, GList *selected_group);
 		void shiftNotes(unsigned long long start_time, int linedist, NedPage *start_page, NedSystem *start_system, int staff_number);
+		void ProgramChannels();
+		int setChannelFromCursor();
 		void deleteStaff(int staff_number);
 		void shiftStaff(int staff_number, int position);
 		void restoreStaff(int staff_number, staff_context_str *staff_context);
@@ -217,6 +237,7 @@ class NedMainWindow {
 		double determineTempoInverse(NedChordOrRest *element);
 		void setInternalPitches();
 		void adjustAccidentals();
+		void insertNoteWithPitch(int num_midi_input_notes, int *midi_input_chord);
 		GtkWidget *getDrawingArea() {return m_drawing_area;}
 		GtkWidget *getWindow() {return m_main_window;}
 		struct paper_info_struct *getCurrentPaper() {return m_current_paper;}
@@ -239,6 +260,7 @@ class NedMainWindow {
 		SpecialMeasure *getSpecialMeasure(int meas_num);
 		void resetSpecialType() {m_special_type = m_special_sub_type.m_special_sub_type_int = -1;
 				m_special_sub_type.m_chord_info.chord_ptr = NULL;}
+		void setChorNameInfo(char *rootname, char *uptext, char *downtext, double fontsize, double ydist);
 		void setAndUpdateClefTypeAndKeySig();
 		double get2ndNettoIndent() {return m_2ndnetto_indent;}
 		double getNettoIndent() {return m_netto_indent;}
@@ -247,21 +269,24 @@ class NedMainWindow {
 		unsigned int getUpBeatInverse() {return m_upbeat_inverse;}
 		int getFileVersion() {return m_file_version;}
 		void turnOffKeyboardCtrlMode() { m_keyboard_ctrl_mode= false; }
+		void resetMidiInput();
 		void setAllUnpositioned();
 		NedFreeReplaceable *m_freetext;
 		void drawVisibleRectangle(cairo_t *cr, NedNote *note);
+		void resetActiveFlags();
 	private: 
 		GList *m_special_measures;
-		int setSpecial(int measnum, int mtype, bool hide, bool *oldhide);
-		void setSpecialTimesig(int measnum, int num, int denom, int *oldnum, int *olddenom, bool do_reposit = true);
+		int setSpecial(int measnum, int mtype, bool hide, bool *oldhide, bool overwrite);
+		void setSpecialTimesig(int measnum, int num, int denom, unsigned int newsym, int *oldnum, int *olddenom, unsigned int *oldsym, bool do_reposit = true);
 		static gint compare_spec_measures_according_measnum(gconstpointer ptr1, gconstpointer ptr2);
 		void readSpecMeasures(FILE *fp, GList **new_spec_measures);
 		bool conflictWithOhterSpecMeasure(int measnum, int dir);
 		void testAlternativeState(NedCommandList *command_list, NedMeasure *measure, bool alter1, bool alter2);
 		void checkForElementsToSplit(NedCommandList *command_list);
-		void import_from_other(NedImporter *im, char* filename);
+		bool import_from_other(NedImporter *im, char* filename);
 		void deleteEmptyPagesAtEnd();
 		void deletePagesWithoutSystem();
+		void removeEmptyInteriourPages();
 		void rezoomScoreInfo();
 		void adjustAfterZoom();
 		void find_new_cursor_pos(guint keyval, int *x, int *y);
@@ -276,6 +301,7 @@ class NedMainWindow {
 		bool deleteEnclosedBras(unsigned int starts, unsigned int ends);
 		bool deleteEnclosedConnections();
 		void computeSystemIndent();
+		void tryTieBackward();
 		void modeChange(unsigned int stat, GtkAction *action);
 		char *m_start_file_name;
 		int m_last_line_during_key_insertion;
@@ -290,6 +316,7 @@ class NedMainWindow {
 		t_special_subtype m_special_sub_type;
 		GtkWidget *m_drawing_area;
 		GtkWidget *m_page_selector;
+		GtkIMContext *m_im_context;
 		static cairo_status_t writefunc(void *fdv,  const unsigned char *data, unsigned int length);
 #define STREAM_FILE_TYPE_PS 1
 #define STREAM_FILE_TYPE_PDF 2
@@ -304,6 +331,7 @@ class NedMainWindow {
 		void setToPage(int page_nr);
 		void computeScoreTextExtends();
 		int m_numerator, m_denominator;
+		unsigned int m_timesig_symbol;
 		int m_staff_count, m_pre_staffcount;
 		void replay(bool on);
 		bool m_config_changed;
@@ -319,6 +347,7 @@ class NedMainWindow {
 		GtkAction *m_undo_action, *m_redo_action;
 		GtkToggleAction *m_insert_erease_mode_action;
 		GtkToggleAction *m_shift_action, *m_keyboard_insert_action;
+		GtkToggleAction *m_midi_input_action, *m_midi_record_action;
 		GtkToggleAction *m_show_hidden_action;
 		GtkToggleAction *m_replay_action;
 		GtkToggleAction *m_show_measure_numbers_action;
@@ -336,6 +365,7 @@ class NedMainWindow {
 		bool setPageRelative(int y_incr);
 		void setButtons();
 		void resetButtons();
+		void prepareInsertion();
 
 		char m_current_filename[4096];
 
@@ -363,7 +393,6 @@ class NedMainWindow {
 		static void print_file(GtkWidget *widget, void *data);
 		static void import_musicxml(GtkWidget *widget, void *data);
 		static void import_midi(GtkWidget *widget, void *data);
-		static void import_midi2(GtkWidget *widget, void *data);
 		static void export_midi(GtkWidget *widget, void *data);
 		static void export_lily(GtkWidget *widget, void *data);
 		static void insert_tuplet(GtkWidget *widget, void *data);
@@ -372,6 +401,7 @@ class NedMainWindow {
 		static void set_connected_bar_line(GtkWidget *widget, void *data);
 		static void remove_system_delimiter(GtkWidget *widget, void *data);
 		static void remove_tuplet(GtkWidget *widget, void *data);
+		static void OnCommit(GtkIMContext *context, const gchar  *str,  void *data);
 /*		static void append_page(GtkWidget *widget, void *data); 
 		static void insert_page(GtkWidget *widget, void *data); */
 		static void insert_keysig(GtkWidget *widget, void *data);
@@ -384,6 +414,7 @@ class NedMainWindow {
 		static void insert_accelerando(GtkWidget *widget, void *data);
 		static void insert_ritardando(GtkWidget *widget, void *data);
 		static void append_staff(GtkWidget *widget, void *data);
+		static void lyrics_editor(GtkWidget *widget, void *data);
 		static void insert_ppp(GtkAction *widget, gpointer data);
 		static void insert_pp(GtkAction *widget, gpointer data);
 		static void insert_p(GtkAction *widget, gpointer data);
@@ -405,15 +436,22 @@ class NedMainWindow {
 		static void remove_empty_pages(GtkWidget *widget, void *data);
 		static void change_spacement(GtkWidget *widget, void *data);
 		static void change_scale(GtkWidget *widget, void *data);
+		static void config_record(GtkWidget *widget, void *data);
+		static void config_drums(GtkWidget *widget, void *data);
 		static void popup_tools(GtkWidget *widget, void *data);
 		static void set_upbeat_measure(GtkWidget *widget, void *data);
 		static void set_upbeat_start(GtkWidget *widget, void *data);
 		static void edit_score_info(GtkWidget *widget, void *data);
 		static void mute_staves(GtkWidget *widget, void *data);
 		static void transpose(GtkWidget *widget, void *data);
+		static void set_all_halfs_to_sharp(GtkWidget *widget, void *data);
+		static void set_all_halfs_to_flat(GtkWidget *widget, void *data);
 		static void hide_rests(GtkWidget *widget, void *data);
 		static void unhide_rests(GtkWidget *widget, void *data);
-		static void config_midi(GtkWidget *widget, void *data);
+		static void hide_rests_of_all_voices(GtkWidget *widget, void *data);
+		static void unhide_rests_of_all_voices(GtkWidget *widget, void *data);
+		static void config_midi_out(GtkWidget *widget, void *data);
+		static void config_midi_in(GtkWidget *widget, void *data);
 		static void config_paper(GtkWidget *widget, void *data);
 		static void show_about(GtkWidget *widget, void *data);
 #ifdef YELP_PATH
@@ -429,6 +467,7 @@ class NedMainWindow {
 		static void paste_data(GtkWidget *widget, void *data);
 		static void delete_block(GtkWidget *widget, void *data);
 		static void append_system(GtkWidget *widget, void *data);
+		static void importRecorded(GtkWidget *widget, void *data);
 		static void insert_block(GtkWidget *widget, void *data);
 		static void insert_clef(GtkWidget *widget, void *data);
 		static void reposit_all(GtkWidget *widget, void *data);
@@ -442,6 +481,8 @@ class NedMainWindow {
 		static void set_paint_colored(GtkAction *action, gpointer data);
 		static void toggle_shift_mode(GtkAction *action, gpointer data);
 		static void toggle_show_hidden(GtkAction *action, gpointer data);
+		static void toggle_midi_input(GtkAction *action, gpointer data);
+		static void toggle_record_midi(GtkAction *action, gpointer data);
 		static void size_change_handler(GtkWidget *widget, GtkRequisition *requisition, gpointer data);
 		static gboolean handle_delete(GtkWidget *widget, GdkEvent *event, gpointer data);
 		static void handle_page_request(GtkSpinButton *spinbutton, gpointer data);
@@ -454,6 +495,7 @@ class NedMainWindow {
 		bool m_avoid_feedback_action;
 		int m_lyrics_mode;
 
+		static void OnSetFocus(GtkWindow *window, GtkWidget *widget, gpointer data);  
 		static gboolean handle_button_press (GtkWidget *widget,
 			 GdkEventButton *event, gpointer data);
 		static gboolean handle_button_release (GtkWidget *widget,
@@ -483,6 +525,7 @@ class NedMainWindow {
 		static bool close_window2(GtkWidget *widget, GdkEvent  *event, gpointer data);
 		static void close_window(GtkWidget  *widget, void *data);
 		static void open_recent(GtkAction *action, gpointer user); 
+		NedMidiRecorder *m_midi_recorder;
 		double m_pointer_xpos, m_pointer_ypos;
 		struct paper_info_struct *m_current_paper;
 		bool m_paint_colored;
@@ -497,10 +540,21 @@ class NedMainWindow {
 		gint m_last_cursor_x, m_last_cursor_y;
 		NedChordOrRest *m_last_obj;
 		NedSystem *m_last_y_adjustment;
+		int m_record_midi_numerator, m_record_midi_denominator;
+		int m_record_midi_pgm, m_record_midi_tempo_inverse, m_record_midi_metro_volume;
+		bool m_record_midi_triplet_recognition, m_record_midi_f_piano, m_record_midi_dnt_split;
+		int m_record_midi_keysig;
 #ifdef YELP_PATH
 		char *m_docu;
 #endif
 		int m_file_version;
+#ifdef WITH_BANNER
+		NedPangoCairoText *m_banner1;
+		NedPangoCairoText *m_banner2;
+		NedPangoCairoText *m_banner3;
+		NedPangoCairoText *m_banner4;
+		NedPangoCairoText *m_banner5;
+#endif
 
 	friend class NedAppendNewPageCommand;
 	friend class NedInsertNewPageCommand;
