@@ -20,7 +20,7 @@
 /****************************************************************************************/
 
 #include "midiimporter.h"
-#include "instrumenttable.h"
+#include "idiotseditor.h"
 #include "resource.h"
 #include <string.h>
 
@@ -44,12 +44,11 @@
 #define META_KEYSIG            0x59
 #define META_TEMPO             0x51
 
-NedMidiImporter::NedMidiImporter(bool with_triplets, bool with_single_triplets,  bool with_alto_clef, int voice_num,
-	int tempo_change_density, int volume_change_density, bool force_piano, bool force_one_staff) : NedImporter(), m_error_string(NULL), m_current_time(0)
+NedMidiImporter::NedMidiImporter(bool with_triplets, int tempo_change_density, int volume_change_density, bool force_piano, bool sort_instruments, bool dont_split) :
+	NedImporter(), m_error_string(NULL), m_current_time(0), m_last_event_time(0), m_with_triplets(with_triplets)
 {
-	m_instrument_table = new NedInstrumentTable(with_triplets, with_single_triplets, with_alto_clef, voice_num, tempo_change_density, volume_change_density, force_piano, force_one_staff);
+	m_idiots_editor = new NedIdiotsEditor(tempo_change_density, volume_change_density, force_piano, sort_instruments, dont_split);
 	for (int i = 0; i < 16; m_current_midi_instrument[i++] = 0);
-	m_current_midi_instrument[9] = -1;
 }
 
 bool NedMidiImporter::import() {
@@ -59,6 +58,7 @@ bool NedMidiImporter::import() {
 	read_header(m_FILE);
 	for (track_nr = 0; track_nr < m_ntracks; track_nr++) {
 		readTrack(m_FILE, track_nr, 0);
+		if (m_current_time > m_last_event_time) m_last_event_time = m_current_time;
 		if (m_error_string != NULL) break;
 	}
 
@@ -66,10 +66,8 @@ bool NedMidiImporter::import() {
 		NedResource::DbgMsg(DBG_TESTING, "%s\n", m_error_string);
 		return false;
 	}
-
-	m_instrument_table->computeTimeSigMeasureNumbers(m_main_window);
+	/*
 	fseek(m_FILE, 0, SEEK_SET);
-
 	read_header(m_FILE);
 	for (track_nr = 0; track_nr < m_ntracks; track_nr++) {
 		readTrack(m_FILE, track_nr, 1);
@@ -80,45 +78,53 @@ bool NedMidiImporter::import() {
 		NedResource::DbgMsg(DBG_TESTING, "%s\n", m_error_string);
 		return false;
 	}
-	m_instrument_table->prepareNotes();
-	/*
-	for (i = 0; i < VOICE_COUNT; i++) {
-		m_parts[0].voices[i] = m_instrument_table->convertToNtEdNotes(i);
-	}
 	*/
+	m_idiots_editor->setLastTime(m_last_event_time);
+	m_idiots_editor->computeTimeSigMeasureNumbers(m_main_window);
+	m_idiots_editor->computeTempo();
+	m_idiots_editor->computeVolumes();
+	m_idiots_editor->snapStartOfNotes();
+	if (m_with_triplets) {
+		m_idiots_editor->recognizeTriplets();
+	}
+	m_idiots_editor->trySplitInstruments();
+	m_tempo_inverse = m_idiots_editor->m_tempo_inverse;
+
 	m_upbeat_inverse = 0;
-	m_partcount = m_instrument_table->getPartCount();
-	m_tempo_inverse = m_instrument_table->m_tempo_inverse;
+	m_partcount = m_idiots_editor->getPartCount();
 	for (i = 0; i < m_partcount; i++) {
-		m_parts[i].voices[0] = m_instrument_table->convertToNtEdNotes(i, 0);
+		m_parts[i].voices[0] = m_idiots_editor->convertToNtEdNotes(i, 0);
 		for (j = 1; j < VOICE_COUNT; j++) {
-			m_parts[i].voices[j] = m_instrument_table->convertToNtEdNotes(i, j);
+			m_parts[i].voices[j] = m_idiots_editor->convertToNtEdNotes(i, j);
 		}
 		strcpy(m_parts[i].id, "part1");
-		cptr = m_instrument_table->getInstrumentName(i);
+		cptr = m_idiots_editor->getInstrumentName(i);
 		if (cptr == NULL) {
 			m_parts[i].name[0] = '\0';
-			NedResource::DbgMsg(DBG_TESTING, "Stelle X: NULL\n");
 		}
 		else {
-			NedResource::DbgMsg(DBG_TESTING, "Stelle X: name = %s\n", cptr);
-			m_parts[i].name[0] = '\0';
-			//strcpy(m_parts[i].name, cptr);
+			if (*cptr == '\0') {
+				m_parts[i].name[0] = '\0';
+			}
+			else {
+				strcpy(m_parts[i].name, cptr);
+			}
 		}
-		m_parts[i].clef = m_instrument_table->getClef(i);
-		m_parts[i].clef_octave = m_instrument_table->getOctaveShift(i);
-		m_parts[i].key_signature = m_instrument_table->getKeySig(i);
-		m_parts[i].numerator = m_instrument_table->getNumerator();
-		m_parts[i].denominator = m_instrument_table->getDenominator();
-		m_parts[i].midi_pgm = m_instrument_table->getMidiPgm(i);
-		m_parts[i].midi_channel = m_instrument_table->getChannel(i);
-		m_parts[i].volume = m_instrument_table->getVolume(i);
+		m_parts[i].clef = m_idiots_editor->getClef(i);
+		m_parts[i].clef_octave = m_idiots_editor->getOctaveShift(i);
+		m_parts[i].key_signature = m_idiots_editor->getKeySig(i);
+		m_parts[i].numerator = m_idiots_editor->getNumerator();
+		m_parts[i].denominator = m_idiots_editor->getDenominator();
+		m_parts[i].midi_pgm = m_idiots_editor->getMidiPgm(i);
+		m_parts[i].midi_channel = m_idiots_editor->getChannel(i);
+		m_parts[i].volume = m_idiots_editor->getVolume(i);
 	}
 	return true;
 }
 
 NedMidiImporter::~NedMidiImporter() {
-	delete m_instrument_table;
+	//printf("NedMidiImporter::~NedMidiImporter\n"); fflush(stdout);
+	delete m_idiots_editor;
 }
 
 
@@ -189,11 +195,19 @@ void NedMidiImporter::readTrack(FILE *fp, int track_nr, int pass) {
 	unsigned char status_byte, last_status;
 	char sbyte, data1, data2;
 	int num, denom;
-	NedTableTimesig *timesig;
 	unsigned int chan;
-	NedTableNote *note;
-	bool tempo_set = false;
-	bool keysig_set = false;
+	struct {
+		long long time;
+		int key;
+		int state;
+#define KEYSIG_INVALID 0
+#define KEYSIG_SET 1
+#define KEYSIG_NOT_RELEVANT 2
+	} first_keysig;
+
+	first_keysig.state = KEYSIG_INVALID;
+	first_keysig.time = 0;
+	first_keysig.key = 0;
 
 	if (fread(buffer, 1, 4, fp) != 4) {
 		m_error_string = "unexpected EOF(2)";
@@ -210,7 +224,6 @@ void NedMidiImporter::readTrack(FILE *fp, int track_nr, int pass) {
 		return;
 	}
 
-	m_instrument_table->removeLastInserted();
 	last_status = 0;
 	m_current_time = 0;
 	while (len > 0) {
@@ -238,18 +251,11 @@ void NedMidiImporter::readTrack(FILE *fp, int track_nr, int pass) {
 			}
 			len -= meta_len + 1;
 			switch (type) {
-				case META_TRACK_NAME: if (pass < 1) break;
+				case META_TRACK_NAME: 
 						  buffer[meta_len] = '\0';
-						  buffer[10] = '\0';
-						  NedResource::DbgMsg(DBG_TESTING, "trackname = %s\n", buffer);
-						  m_instrument_table->setInstrumentName(buffer);
+						  m_idiots_editor->setInstrumentName(buffer);
 						  break;
-				case META_KEYSIG: if (pass < 1) break;
-						  if (keysig_set) break;
-						   m_instrument_table->addKeySig(m_current_time, buffer[0]);
-						   keysig_set = true;
-						   break;
-				case META_TIMESIG: if (pass > 0) break;
+				case META_TIMESIG:
 						   num  = buffer[0];
 						   switch(buffer[1]) {
 						   	case 0: denom =  1; break;
@@ -260,19 +266,21 @@ void NedMidiImporter::readTrack(FILE *fp, int track_nr, int pass) {
 						   }
 						   NedResource::DbgMsg(DBG_TESTING, "TIMESIG: %llu(%llu): num = %d, dmon = %d\n",
 						   		m_current_time, m_current_time / NOTE_64, num, denom);
-						   timesig = new NedTableTimesig(m_current_time, num, denom);
-						   m_instrument_table->addTimeSig(timesig);
+						   m_idiots_editor->addTimeSigChange(m_current_time, num, denom);
+						   break;
+				case META_TEMPO: tempo = ((0xff & buffer[0]) << 16) | ((0xff & buffer[1]) << 8) | (0xff & buffer[2]);
+						 m_idiots_editor->addTempoChange(m_current_time, (int) (60000000.0 / (double) tempo));
+						 break;
+				case META_KEYSIG:
+						if (track_nr > 0 && first_keysig.state == KEYSIG_INVALID) { // avoid keysigs of empry tracks
+							first_keysig.time = m_current_time;
+							first_keysig.key = buffer[0];
+							first_keysig.state = KEYSIG_SET;
+							break;
+						   }
+						   m_idiots_editor->addKeySig(m_current_time, 0, buffer[0]);
 						   break;
 
-				case META_TEMPO: if (pass > 0) break; 
-						tempo = ((0xff & buffer[0]) << 16) | ((0xff & buffer[1]) << 8) | (0xff & buffer[2]);
-						if (tempo_set) {
-							m_instrument_table->addTempoSig(new NedTableTemposig(m_current_time, (int) (60000000.0 / (double) tempo)));
-							break;
-						}
-						m_instrument_table->m_tempo_inverse = (double) tempo / 1000.0;
-						tempo_set = true;
-						break;
 			}
 		}
 		else if (status_byte == SYS_EXCLUSIVE_MESSAGE1 || status_byte == SYS_EXCLUSIVE_MESSAGE2) {
@@ -294,13 +302,23 @@ void NedMidiImporter::readTrack(FILE *fp, int track_nr, int pass) {
 			}
 			len -= 2;
 			chan = status_byte & 0x0f;
-			if (pass > 0) {
+			if (chan == 9) {
+				m_current_midi_instrument[chan] = -1;
+			}
+			if (pass == 0) {
 				if (data2 == 0) {
-					m_instrument_table->closeNote(data1, m_current_time, m_current_midi_instrument[chan]);
+					m_idiots_editor->closeNote(data1, m_current_time, m_current_midi_instrument[chan], chan);
 				}
 				else {
+				/*
 					note = new NedTableNote(m_current_time, 0, data1, data2);
-					m_instrument_table->addNote(note, m_current_midi_instrument[chan]);
+					m_idiots_editor->addNote(note, m_current_midi_instrument[chan]);
+					*/
+					if (first_keysig.state == KEYSIG_SET) {
+						m_idiots_editor->addKeySig(first_keysig.time, 0, first_keysig.key);
+						first_keysig.state = KEYSIG_NOT_RELEVANT;
+					}
+					m_idiots_editor->addNote(data1, data2, m_current_time, m_current_midi_instrument[chan], chan);
 				}
 			}
 		}
@@ -315,8 +333,8 @@ void NedMidiImporter::readTrack(FILE *fp, int track_nr, int pass) {
 			}
 			len -= 2;
 			chan = status_byte & 0x0f;
-			if (pass > 0) {
-				m_instrument_table->closeNote(data1, m_current_time, m_current_midi_instrument[chan]);
+			if (pass == 0) {
+				m_idiots_editor->closeNote(data1, m_current_time, m_current_midi_instrument[chan], chan);
 			}
 		}
 		else if ((status_byte & 0xf0) == AFTER_TOUCH) {
