@@ -31,6 +31,8 @@
 #include "commandhistory.h"
 #include "tienotescommand.h"
 #include "untieforwardcommand.h"
+#include "changeaccidentalcmd.h"
+#include "druminfo.h"
 
 
 #define X_POS_PAGE_REL(p) ((getPage()->getContentXpos() + (p)) * zoom_factor - leftx)
@@ -76,7 +78,7 @@ int NedNote::signToBase[][2] = {
 int NedNote::diaDists[] = {-1, 0, 2, 4, 5, 7, 9, 11, 12, 14};
 
 NedNote::NedNote(NedChordOrRest *chord_or_rest, int line, int head, unsigned int status) :
-m_active(FALSE), m_acc_dist(0.0), m_line(line), m_tmp_line(UNKNOWN_LINE), m_status(status), m_head(head), m_chord_or_rest(chord_or_rest),
+m_active(NOTE_ACTIVE_STATE_NONE), m_xpos(0.0), m_ypos(0.0), m_tmp_ypos(0.0), m_acc_dist(0.0), m_line(line), m_tmp_line(UNKNOWN_LINE), m_status(status), m_head(head), m_chord_or_rest(chord_or_rest),
 m_tie_forward(NULL), m_tie_backward(NULL), m_tie_y_offs1(0.0), m_tie_y_offs2(0.0), m_tie_xpos0(0.0), m_tie_xpos1(0.0), m_tie_back_xpos0(0.0),
 m_tie_back_xpos1(0.0), m_tie_offs(0.0) {
 	m_ypos = - line * LINE_DIST/2.0;
@@ -114,7 +116,7 @@ void NedNote::draw(cairo_t *cr) {
 	}
 	*/
 
-	if (getMainWindow()->m_selected_note == this || m_active) {
+	if (getMainWindow()->m_selected_note == this || m_active == NOTE_ACTIVE_STATE_ON) {
 		if (getMainWindow()->doPaintColored()) {
 			switch(getVoice()->getVoiceNumber()) {
 				case 0: cairo_set_source_rgb (cr, DSCOLOR(V1RED), DSCOLOR(V1GREEN), DSCOLOR(V1BLUE)); break;
@@ -158,6 +160,8 @@ void NedNote::draw(cairo_t *cr) {
 			glyph.index = BASE + 25; break;
 		case TRIAG_NOTE2:
 			glyph.index = BASE + 26; break;
+		case OPEN_HIGH_HAT:
+		case CLOSED_HIGH_HAT:
 		case CROSS_NOTE3:
 			glyph.index = BASE + 49; break;
 		case CROSS_NOTE4:
@@ -180,6 +184,13 @@ void NedNote::draw(cairo_t *cr) {
 
 
 	cairo_show_glyphs(cr, &glyph, 1);
+	if (m_active) {
+		cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+		if (m_active == NOTE_ACTIVE_STATE_OFF) {
+			m_active = NOTE_ACTIVE_STATE_NONE;
+		}
+		return;
+	}
 
 	switch (m_status & ACCIDENTAL_MASK) {
 		case STAT_DFLAT: glyph.index = BASE + 29;
@@ -258,7 +269,7 @@ void NedNote::draw(cairo_t *cr) {
 		}
 			
 	}
-	if (getMainWindow()->m_selected_note == this || m_active || getMainWindow()->doPaintColored()) {
+	if (getMainWindow()->m_selected_note == this || getMainWindow()->doPaintColored()) {
 		cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
 	}
 	if (m_tie_forward != NULL) {
@@ -385,7 +396,7 @@ void NedNote::setOffset(char offs_array[OFFS_ARRAY_SIZE]) {
 	}
 }
 
-int NedNote::pitchToLine(int pitch, int clef, int keysig, int octave_shift, int *offs) {
+int NedNote::pitchToLine(int pitch, int clef, int keysig, int octave_shift, int *offs, int preferred_offs /* = 0 */) {
 	int oct = 0;
 	int line = 0;
 	int x_oct_shift= 0;
@@ -410,6 +421,7 @@ int NedNote::pitchToLine(int pitch, int clef, int keysig, int octave_shift, int
 	}
 	switch (clef) {
 		case NEUTRAL_CLEF1:
+		case NEUTRAL_CLEF3:
 		case TREBLE_CLEF: clef_offs = TREBLE_LINE_OFFS; break;
 		case NEUTRAL_CLEF2:
 		case BASS_CLEF: clef_offs = BASS_LINE_OFFS; break;
@@ -418,54 +430,90 @@ int NedNote::pitchToLine(int pitch, int clef, int keysig, int octave_shift, int
 		case SOPRAN_CLEF: clef_offs = SOPRAN_LINE_OFFS; break;
 	}
 
-	if (keysig < 0) {
-		switch (pitch) {
-			case 60: line = -2; break;
-			case 61: line = -1; *offs = -1; break;
-			case 62: line = -1; break;
-			case 63: line =  0; *offs = -1; break;
-			case 64: line =  0; break;
-			case 65: line =  1; break;
-			case 66: line =  2; *offs = -1; break;
-			case 67: line =  2; break;
-			case 68: line =  3; *offs = -1; break;
-			case 69: line =  3; break;
-			case 70: line =  4; *offs = -1; break;
-			case 71: line =  4; break;
-		}
+	switch (preferred_offs) {
+		case -1:
+			switch (pitch) {
+				case 60: line = -2; break;
+				case 61: line = -1; *offs = -1; break;
+				case 62: line = -1; break;
+				case 63: line =  0; *offs = -1; break;
+				case 64: line =  0; break;
+				case 65: line =  1; break;
+				case 66: line =  2; *offs = -1; break;
+				case 67: line =  2; break;
+				case 68: line =  3; *offs = -1; break;
+				case 69: line =  3; break;
+				case 70: line =  4; *offs = -1; break;
+				case 71: line =  4; break;
+			}
+			break;
+		case 1:
+			switch (pitch) {
+				case 60: line = -2; break;
+				case 61: line = -2; *offs = 1; break;
+				case 62: line = -1; break;
+				case 63: line = -1; *offs = 1; break;
+				case 64: line =  0; break;
+				case 65: line =  1; break;
+				case 66: line =  1; *offs = 1; break;
+				case 67: line =  2; break;
+				case 68: line =  2; *offs = 1; break;
+				case 69: line =  3; break;
+				case 70: line =  3; *offs = 1; break;
+				case 71: line =  4; break;
+			}
+			break;
+		default:
+			if (keysig < 0) {
+				switch (pitch) {
+					case 60: line = -2; break;
+					case 61: line = -1; *offs = -1; break;
+					case 62: line = -1; break;
+					case 63: line =  0; *offs = -1; break;
+					case 64: line =  0; break;
+					case 65: line =  1; break;
+					case 66: line =  2; *offs = -1; break;
+					case 67: line =  2; break;
+					case 68: line =  3; *offs = -1; break;
+					case 69: line =  3; break;
+					case 70: line =  4; *offs = -1; break;
+					case 71: line =  4; break;
+				}
+			}
+			else {
+				switch (pitch) {
+					case 60: line = -2; break;
+					case 61: line = -2; *offs = 1; break;
+					case 62: line = -1; break;
+					case 63: line = -1; *offs = 1; break;
+					case 64: line =  0; break;
+					case 65: line =  1; break;
+					case 66: line =  1; *offs = 1; break;
+					case 67: line =  2; break;
+					case 68: line =  2; *offs = 1; break;
+					case 69: line =  3; break;
+					case 70: line =  3; *offs = 1; break;
+					case 71: line =  4; break;
+				}
+			}
 	}
-	else {
-		switch (pitch) {
-			case 60: line = -2; break;
-			case 61: line = -2; *offs = 1; break;
-			case 62: line = -1; break;
-			case 63: line = -1; *offs = 1; break;
-			case 64: line =  0; break;
-			case 65: line =  1; break;
-			case 66: line =  1; *offs = 1; break;
-			case 67: line =  2; break;
-			case 68: line =  2; *offs = 1; break;
-			case 69: line =  3; break;
-			case 70: line =  3; *offs = 1; break;
-			case 71: line =  4; break;
-		}
-	}
-
 	return line + 7 * oct - x_oct_shift + clef_offs;
 }
 
-int NedNote::getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bool *flat) {
+int NedNote::getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bool *flat, bool use_transpose) {
+	if (clef == NEUTRAL_CLEF3) return getDrumPitch();
 	int p;
 	int offs = 0;
 	char offs_array[OFFS_ARRAY_SIZE];
 	*flat = false;
 
 	if (m_tie_backward != 0) {
-		return m_tie_backward->getPitch(clef, keysig, octave_shift, pitch_offs, flat);
+		return m_tie_backward->getPitch(clef, keysig, octave_shift, pitch_offs, flat, use_transpose);
 	}
 
 	switch (clef) {
 		case NEUTRAL_CLEF1:
+		case NEUTRAL_CLEF3:
 		case TREBLE_CLEF: offs = TREBLE_PITCH_OFFS; break;
 		case NEUTRAL_CLEF2:
 		case BASS_CLEF: offs = BASS_PITCH_OFFS; break;
@@ -507,7 +555,21 @@ int NedNote::getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bo
 			if (offs_array[m_line + OFFS_ARRAY_HALF] < 0) *flat = true;
 			break;
 	} 
-	return p + octave_shift + pitch_offs + getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_play_transposed;
+	if (use_transpose) {
+		return p + octave_shift + pitch_offs + getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_play_transposed;
+	}
+	return p + octave_shift + pitch_offs;
+}
+
+int NedNote::getDrumPitch() {
+	struct d_info_str *d_inf_ptr;
+
+	for (d_inf_ptr = current_dinfo; d_inf_ptr->name; d_inf_ptr++) {
+		if (m_line == d_inf_ptr->line && m_head == d_inf_ptr->note_head) {
+			return d_inf_ptr->pitch;
+		}
+	}
+	return 60;
 }
 
 	
@@ -518,6 +580,7 @@ void NedNote::removeUnneededAccidental(int clef, int keysig, int octave_shift) {
 
 	switch (clef) {
 		case NEUTRAL_CLEF1:
+		case NEUTRAL_CLEF3:
 		case TREBLE_CLEF: offs = TREBLE_PITCH_OFFS; break;
 		case NEUTRAL_CLEF2:
 		case BASS_CLEF: offs = BASS_PITCH_OFFS; break;
@@ -636,7 +699,7 @@ unsigned int NedNote::determineState(int pitch, int line, int half_offs, char of
 void NedNote::setInternalPitch(int pitchdist, int clef, int keysig, int octave_shift) {
 	bool dummy;
 
-	m_pitch = getPitch(clef, keysig, octave_shift, 0, &dummy) + pitchdist;
+	m_pitch = getPitch(clef, keysig, octave_shift, 0, &dummy, false) + pitchdist;
 }
 
 void NedNote::adjustAccidental(int clef, int keysig, int octave_shift) {
@@ -654,6 +717,29 @@ void NedNote::adjustAccidental(int clef, int keysig, int octave_shift) {
 	}
 }
 
+void NedNote::changeAccidental(NedCommandList *command_list, int preferred_offs, int clef, int keysig, int octave_shift) {
+	int offs;
+	int line;
+	unsigned int status;
+	NedChangeAccidentalCommand *ch_acc_cmd;
+
+
+	line = pitchToLine(m_pitch, clef, keysig, octave_shift, &offs, preferred_offs);
+	status = (m_status & (~(ACCIDENTAL_MASK)));
+
+	switch (offs) {
+		case -1: status |= STAT_FLAT; break;
+		case  1: status |= STAT_SHARP; break;
+		default: status |= STAT_NATURAL; break;
+	}
+
+	if (status != m_status || line != m_line) {
+		ch_acc_cmd = new NedChangeAccidentalCommand(this, status, line);
+		ch_acc_cmd->execute();
+		command_list->addCommand(ch_acc_cmd);
+	}
+}
+
 
 void NedNote::prepareReplay(unsigned long long midi_start_time, int clef, int keysig, int octave_shift, int pitch_offs, 
 		 unsigned int grace_time, bool is_grace, bool is_stacc, unsigned short segno_sign) {
@@ -661,62 +747,62 @@ void NedNote::prepareReplay(unsigned long long midi_start_time, int clef, int ke
 	unsigned int i;
 	bool dummy;
 
-	pitch = getPitch(clef, keysig, octave_shift, pitch_offs, &dummy);
+	pitch = getPitch(clef, keysig, octave_shift, pitch_offs, &dummy, true);
 	if (is_grace) {
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 			midi_start_time - grace_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			midi_start_time - grace_time + GRACE_DURATION, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 	else if (is_stacc && m_chord_or_rest->getLength() > NOTE_32 && grace_time == 0) {
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() / 2, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_TRILL) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
 		pitch2 = findPitchUpDown(keysig, pitch, true);
 		for (tt = 0, i = 0; i < m_chord_or_rest->getDuration() / NOTE_32; i++, tt = 1 - tt) {
-			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 			midi_start_time + i * NOTE_32, SND_SEQ_EVENT_NOTEON, tt ? pitch : pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 			m_chord_or_rest->getMidiTime() + (i + 1) * NOTE_32, SND_SEQ_EVENT_NOTEOFF, tt ? pitch : pitch2, 0, segno_sign, this);
 		}
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_PRALL) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
 		pitch2 = findPitchUpDown(keysig, pitch, true);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_MORDENT) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
 		pitch2 = findPitchUpDown(keysig, pitch, false);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 	else {
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
+		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
 			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() - grace_time, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 }
@@ -757,7 +843,8 @@ void NedNote::do_tie(NedNote *other_note) {
 	NedNote *n0, *n1;
 	if (getChord()->getType() == TYPE_GRACE || other_note->getChord()->getType() == TYPE_GRACE) return;
 	if (other_note == this) return;
-	if (other_note->m_line != m_line) return;
+	// it sufficies the notes are on the same line if (!isTheSame(other_note)) return;
+	if (m_line != other_note->m_line) return;
 	sorting = m_chord_or_rest->getSorting(other_note->m_chord_or_rest);
 	switch (sorting) {
 		case SORTING_NONE: return;
@@ -778,7 +865,12 @@ void NedNote::setTies(NedNote *n1, NedNote *n2) {
 	n2->m_tie_backward = n1;
 }
 
-
+void NedNote::tryTieBackward() {
+	NedChordOrRest *previousChord = m_chord_or_rest->getPreviousChordOrRest();
+	if (previousChord == NULL) return;
+	if (previousChord->getType() != TYPE_CHORD) return;
+	previousChord->tieAllNotes(m_chord_or_rest);
+}
 
 bool NedNote::noConflict() {
 	return m_chord_or_rest->noConflict(m_tmp_line);
@@ -796,6 +888,10 @@ bool NedNote::trySelect(double x, double y) {
 	double xl = X_POS_INVERS_PAGE_REL(x);
 	double yl = Y_POS_INVERS_STAFF_REL(y);
 	double xnote = m_xpos + m_chord_or_rest->getXPosAndMicroshifts();
+	if (m_chord_or_rest->getXPos() < -800.0) {
+		printf("x = %f\n", m_chord_or_rest->getXPos());
+		exit(1);
+	}
 
 	if (m_ypos + extention->y_bearing / zoom_factor < yl &&
 	    m_ypos + (extention->y_bearing  +  extention->height) / zoom_factor > yl &&
@@ -840,26 +936,30 @@ bool NedNote::testYShift(double y) {
 
 void NedNote::setStatus(unsigned int status) {
 	int clef, keysig, octave_shift;
+	int pitch;
 	bool dummy;
 	staff_context_str *context;
 	m_status = status;
 	context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
 	getStaff()->getCurrentClefAndKeysig(m_chord_or_rest->getMidiTime(), &clef, &keysig, &octave_shift);
 	if (NedResource::m_midi_echo) {
-		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, getPitch(clef, keysig, octave_shift, 0, &dummy), context->m_midi_volume);
+		pitch = getPitch(clef, keysig, octave_shift, 0, &dummy, true);
+		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, 1, &pitch, context->m_midi_volume);
 	}
 }
 void NedNote::moveUpDown(int line) {
 	staff_context_str *context;
 	bool dummy;
 	int clef, keysig, octave_shift;
+	int pitch;
 	getStaff()->getCurrentClefAndKeysig(m_chord_or_rest->getMidiTime(), &clef, &keysig, &octave_shift);
 	m_line = line;
 	m_ypos = - m_line * LINE_DIST/2.0;
 	m_chord_or_rest->sortNotes();
 	if (NedResource::m_midi_echo) {
 		context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
-		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, getPitch(clef, keysig, octave_shift, 0, &dummy), context->m_midi_volume);
+		pitch = getPitch(clef, keysig, octave_shift, 0, &dummy, true);
+		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, 1, &pitch, context->m_midi_volume);
 	}
 	m_tmp_line = UNKNOWN_LINE;
 }
@@ -922,6 +1022,13 @@ void NedNote::computeTie() {
 			}
 			m_tie_xpos0 = m_chord_or_rest->getXPosAndMicroshifts() + m_xpos + TIE_X_START;
 			m_tie_xpos1 = m_tie_forward->m_chord_or_rest->getXPosAndMicroshifts() + m_tie_forward->m_xpos - TIE_X_START;
+			if (m_tie_xpos1 - m_tie_xpos0 > MIN_TIE_LEN) {
+				m_tie_xpos0 += TIE_X_START_LARGE;
+				m_tie_xpos1 -= TIE_X_START_LARGE;
+			}
+			else if (m_chord_or_rest->getDotCount()) {
+				m_tie_xpos0 += DOT2POS;
+			}
 		}
 		else {
 			m_tie_xpos1 = getPage()->getContentWidth() + TIE_X_RIGHT_OVER;
@@ -1100,8 +1207,12 @@ void NedNote::exportLilyPond(FILE *fp, int *last_line, bool *tie) {
 	staff_context_str *context;
 	context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
 	getStaff()->getCurrentClefAndKeysig(m_chord_or_rest->getMidiTime(), &clef, &keysig, &octave_shift);
-	if (clef == NEUTRAL_CLEF1 || clef == NEUTRAL_CLEF2) clef = ALTO_CLEF;
-	pitch = getPitch(clef, keysig, octave_shift, 0, &flat);
+	if (clef == NEUTRAL_CLEF3) {
+		exportLilyPondDrums(fp);
+		return;
+	}
+	if (clef == NEUTRAL_CLEF1 || clef == NEUTRAL_CLEF2 || clef == NEUTRAL_CLEF3) clef = ALTO_CLEF;
+	pitch = getPitch(clef, keysig, octave_shift, 0, &flat, false);
 
 	while (pitch < 60) pitch += 12;
 	while (pitch > 71) pitch -= 12;
@@ -1112,13 +1223,13 @@ void NedNote::exportLilyPond(FILE *fp, int *last_line, bool *tie) {
 		case 62: fprintf(fp, " d"); break;
 		case 63: fprintf(fp, " %s", flat ? "es" : "dis"); break;
 		case 64: fprintf(fp, " e"); break;
-		case 65: fprintf(fp, " f"); break;
+		case 65: fprintf(fp, " %s", (keysig > 5) ? "eis" : "f"); break;
 		case 66: fprintf(fp, " %s", flat ? "ges" : "fis"); break;
 		case 67: fprintf(fp, " g"); break;
 		case 68: fprintf(fp, " %s", flat ? "as" : "gis"); break;
 		case 69: fprintf(fp, " a"); break;
 		case 70: fprintf(fp, " %s", flat ? "bes" : "ais"); break;
-		case 71: fprintf(fp, " b"); break;
+		case 71: fprintf(fp, " %s", (keysig < -5) ? "ces" : "b"); break;
 	}
 	line = m_line;
 	while (line > *last_line + 3) {
@@ -1134,3 +1245,15 @@ void NedNote::exportLilyPond(FILE *fp, int *last_line, bool *tie) {
 	}
 	*last_line = m_line;
 }
+
+void NedNote::exportLilyPondDrums(FILE *fp) {
+	struct d_info_str *d_inf_ptr;
+
+	for (d_inf_ptr = current_dinfo; d_inf_ptr->name; d_inf_ptr++) {
+		if (m_line == d_inf_ptr->line && m_head == d_inf_ptr->note_head) {
+			fprintf(fp, " %s", d_inf_ptr->lily_short_name);
+			return;
+		}
+	}
+	fprintf(fp, " r");
+}
