@@ -125,6 +125,16 @@ void NedPage::adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str
 	}
 }
 
+#ifdef XXX
+void NedPage::removeAllBeams() {
+	GList *lptr;
+
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->removeAllBeams();
+	}
+}
+#endif
+
 
 double NedPage::getTopPos() {return (2 * default_border +  m_height) * m_main_window->getCurrentZoomFactor();}
 double NedPage::getLeftPos() {return (( m_xpos + 2*default_border +  m_width)) * m_main_window->getCurrentZoomFactor();}
@@ -137,7 +147,7 @@ NedSystem *NedPage::appendSystem() {
 	m_systems = g_list_append(m_systems, system = new NedSystem(this, 0,
 	                                m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER), system_nr, 1, TRUE));
 
-	placeStaffs(false, 0);
+	placeStaffs(0);
 
 	return system;
 }
@@ -441,6 +451,14 @@ void NedPage::zoomFreeReplaceables(double zoom, double scale) {
 	}
 }
 
+void NedPage::recomputeFreeReplaceables() {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->recomputeFreeReplaceables();
+	}
+}
+
 
 void NedPage::testTies() {
 	GList *lptr;
@@ -451,22 +469,35 @@ void NedPage::testTies() {
 }
 
 bool NedPage::exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
-	NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames) {
+	NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map,
+		bool with_break, bool *guitar_chordnames, bool *chordnames, int *breath_script, bool keep_beams) {
 	GList *lptr;
 	
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		((NedSystem *) lptr->data)->exportLilyPond(fp, staff_nr, voice_nr, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map, with_break, chordnames);
+		((NedSystem *) lptr->data)->exportLilyPond(fp, staff_nr, voice_nr, last_line, midi_len, last_system,
+			end_time, in_alternative, lily_slur, lyrics_map, with_break, guitar_chordnames, chordnames, breath_script, keep_beams);
 		if ((NedSystem *) lptr->data == last_system) return false;
 	}
 	return true;
 }
 
-bool NedPage::exportLilyChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+bool NedPage::exportLilyGuitarChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
 	NedSystem *last_system, unsigned long long end_time, bool *in_alternative, bool with_break) {
 	GList *lptr;
 	
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		((NedSystem *) lptr->data)->exportLilyChordnames(fp, staff_nr, last_line, midi_len, last_system, end_time, in_alternative, with_break);
+		((NedSystem *) lptr->data)->exportLilyGuitarChordnames(fp, staff_nr, last_line, midi_len, last_system, end_time, in_alternative, with_break);
+		if ((NedSystem *) lptr->data == last_system) return false;
+	}
+	return true;
+}
+
+bool NedPage::exportLilyFreeChordName(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+	NedSystem *last_system, unsigned long long end_time, bool *in_alternative, bool with_break) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->exportLilyFreeChordName(fp, staff_nr, last_line, midi_len, last_system, end_time, in_alternative, with_break);
 		if ((NedSystem *) lptr->data == last_system) return false;
 	}
 	return true;
@@ -482,6 +513,30 @@ bool NedPage::exportLilyLyrics(FILE *fp, int staff_nr, int voice_nr, int line_nr
 	return true;
 }
 
+void NedPage::collectLyrics(NedLyricsEditor *leditor, int staff_nr) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->collectLyrics(leditor, staff_nr);
+	}
+}
+
+void NedPage::setLyrics(NedCommandList *command_list, NedLyricsEditor *leditor, int staff_nr) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->setLyrics(command_list, leditor, staff_nr);
+	}
+}
+
+void NedPage::copyDataOfWholeStaff(int staff_nr) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->copyDataOfWholeStaff(staff_nr);
+	}
+}
+
 void NedPage::handleStaffElements() {
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
@@ -503,6 +558,13 @@ void NedPage::adjustAccidentals() {
 	}
 }
 
+void NedPage::changeAccidentals(NedCommandList *command_list, int preferred_offs, bool *staff_list, GList *selected_group) {
+	GList *lptr;
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->changeAccidentals(command_list, preferred_offs, staff_list, selected_group);
+	}
+}
+
 void NedPage::transpose(int pitchdist, bool *staff_list, GList *selected_group) {
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
@@ -838,17 +900,19 @@ double NedPage::convertX(double xpos) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
 	double leftx = m_main_window->getLeftX();
 
-	return X_POS_PAGE_REL(xpos);
+	return m_main_window->getCurrentScale() * X_POS_PAGE_REL(xpos);
 }
 	
 
-bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*  = NULL */, bool force_rest /* = false */) {
+bool NedPage::tryInsertOrErease(double x, double y, int num_midi_input_notes, int *midi_input_chord /* given if insert from midikeyboard */, NedChordOrRest **newObj /*  = NULL */, bool force_rest /* = false */) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
 	double current_scale = m_main_window->getCurrentScale();
 	double leftx = m_main_window->getLeftX();
 	NedChordOrRest *element, *element1, *element2;
+	NedChordName *chordname;
 	double dummy, dummy0;
 	double topy, xx, yy, yyy;
+	double xx2, yy2;
 	unsigned int kind, tempo;
 	NedSlurPoint *sp[3];
 	NedCommandList *command_list;
@@ -867,163 +931,174 @@ bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*
 	GList *lptr;
 	double mindist, d;
 
-	switch(m_main_window->getSpecialType()) {
-	case TYPE_DYNAMIC:
-	case TYPE_TEMPO:
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		yy = Y_POS_INVERS(y);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		command_list = new NedCommandList(m_main_window);
-		if (m_main_window->getSpecialType() == TYPE_DYNAMIC) {
-			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedVolumeSign(m_main_window->getSpecialSubTypeInt()), element, xx, yy));
+	if (midi_input_chord == NULL) { // x, y given
+		switch(m_main_window->getSpecialType()) {
+		case TYPE_DYNAMIC:
+		case TYPE_TEMPO:
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			yy = Y_POS_INVERS(y);
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			command_list = new NedCommandList(m_main_window);
+			if (m_main_window->getSpecialType() == TYPE_DYNAMIC) {
+				command_list->addCommand(new NedInsertFreePlaceableCommand(new NedVolumeSign(m_main_window->getSpecialSubTypeInt()), element, xx, yy));
+			}
+			else {
+				kind = (m_main_window->getSpecialSubTypeInt() & 0xffff) * FACTOR;
+				tempo = ((m_main_window->getSpecialSubTypeInt() >> 16) & 0xffff);
+				command_list->addCommand(new NedInsertFreePlaceableCommand(new NedTempoSign(kind, tempo), element, xx, yy));
+			}
+			command_list->execute();
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			m_main_window->resetSpecialType();
+			return TRUE;
+		case TYPE_SIGN:
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			yy = Y_POS_INVERS(y);
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			command_list = new NedCommandList(m_main_window);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeSign(m_main_window->getSpecialSubTypeInt()), element, xx, yy));
+			command_list->execute();
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			m_main_window->resetSpecialType();
+			return TRUE;
+		case TYPE_GUITAR_CHORD:
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			yy = Y_POS_INVERS(y);
+#define CHOR_NAME_Y_DIST 0.4
+#define CHOR_NAME_X_DIST 0.4
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			xx2 = element->getXPos() - CHOR_NAME_X_DIST;
+			yy2 = element->getSystem()->getYPos() + element->getStaff()->getTopPos() - CHOR_NAME_Y_DIST;
+			command_list = new NedCommandList(m_main_window);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeChord(m_main_window->getSpecialSubTypeChordInfo().chord_ptr,
+					m_main_window->getSpecialSubTypeChordInfo().chord_name_num, 
+					m_main_window->getSpecialSubTypeChordInfo().status, m_main_window), element, xx2, yy2));
+			command_list->execute();
+			m_main_window->setAllUnpositioned();
+			m_main_window->reposit(command_list);
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			m_main_window->resetSpecialType();
+			return TRUE;
+		case TYPE_SPACER:
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			yy = Y_POS_INVERS(y);
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			command_list = new NedCommandList(m_main_window);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeSpacer(m_main_window), element, xx, yy));
+			command_list->execute();
+			m_main_window->setAllUnpositioned();
+			m_main_window->reposit(command_list);
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			m_main_window->resetSpecialType();
+			return TRUE;
+		case TYPE_CHORDNAME:
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			yy = Y_POS_INVERS(y);
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			xx2 = element->getXPos() - CHOR_NAME_X_DIST;
+			yy2 = element->getSystem()->getYPos() +  element->getStaff()->getTopPos() - m_main_window->getSpecialSubTypeChordNameInfo().ydist;
+			command_list = new NedCommandList(m_main_window);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(chordname = new NedChordName(m_main_window->getDrawingArea(),m_main_window->getSpecialSubTypeChordNameInfo().root_name,
+						m_main_window->getSpecialSubTypeChordNameInfo().up_name,m_main_window->getSpecialSubTypeChordNameInfo().down_name, m_main_window->getSpecialSubTypeChordNameInfo().fontsize), element, xx2, yy2));
+
+			chordname->setZoom(zoom_factor, current_scale);
+			command_list->execute();
+			m_main_window->setAllUnpositioned();
+			m_main_window->reposit(command_list);
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			//m_main_window->resetSpecialType();
+			return TRUE;
+		case TYPE_FREE_TEXT:
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			yy = Y_POS_INVERS(y);
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			command_list = new NedCommandList(m_main_window);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(m_main_window->m_freetext, element, xx, yy));
+			m_main_window->m_freetext->setZoom(zoom_factor, current_scale);
+			command_list->execute();
+			m_main_window->m_freetext = NULL;
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			m_main_window->resetSpecialType();
+			return TRUE;
+		case TYPE_SLUR:
+	#define SLUR_START_X_DIST 1.0
+	#define SLUR_START_Y_DIST ( 2 * LINE_DIST )
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			if (xx > m_width / 2.0) xx -= (3 * SLUR_START_X_DIST) / zoom_factor;
+			yy = Y_POS_INVERS(y);
+			command_list = new NedCommandList(m_main_window);
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			yyy = element->getSystem()->getYPos() + element->getStaff()->getMidPos();
+			command_list->addCommand(new NedInsertFreePlaceableCommand(sp[0] = new NedSlurPoint(NULL, 0), element, xx, yyy));
+			dummy0 = 10000000.0;
+			element1 = element->getStaff()->findNearestElement(xx + SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST, NULL, &dummy0, &dummy);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(sp[1] = new NedSlurPoint(NULL, 1), element1, xx + SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST));
+			dummy0 = 10000000.0;
+			element2 = element->getStaff()->findNearestElement(xx + 2 * SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST / 2, NULL, &dummy0, &dummy);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(sp[2] = new NedSlurPoint(NULL, 2), element2, xx + 2 * SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST / 2));
+			command_list->execute();
+			new NedSlur(sp);
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			m_main_window->resetSpecialType();
+			m_main_window->m_selected_free_replaceable = sp[2];
+			return TRUE;
+		case TYPE_LINE:
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			if (xx > m_width / 2.0) xx -= 2 * SLUR_START_X_DIST;
+			yy = Y_POS_INVERS(y);
+			command_list = new NedCommandList(m_main_window);
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(lp0 = new NedLinePoint(NULL, 0), element, xx, yy));
+			dummy0 = 10000000.0;
+			element = element->getStaff()->findNearestElement(xx + LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(lp1 = new NedLinePoint(NULL, 1), element, xx + LINE_START_X_DIST, yy));
+			command_list->execute();
+			switch (m_main_window->getSpecialSubTypeInt()) {
+				case LINE_CRESCENDO:
+				case LINE_DECRESCENDO:
+						new NedCrescendo(lp0, lp1, m_main_window->getSpecialSubTypeInt() == LINE_DECRESCENDO);
+						break;
+				case LINE_OCTAVATION1:
+				case LINE_OCTAVATION_1:
+				case LINE_OCTAVATION2:
+				case LINE_OCTAVATION_2:
+						new NedOctavation(lp0, lp1, m_main_window->getSpecialSubTypeInt());
+						break;
+						
+			}
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			m_main_window->resetSpecialType();
+			m_main_window->m_selected_free_replaceable = lp0;
+			return TRUE;
+		case TYPE_LINE3:
+			topy = m_main_window->getTopY();
+			xx = X_POS_INVERS_PAGE_REL(x);
+			//if (xx > m_width / 2.0) xx -= 3 * SLUR_START_X_DIST;
+			yy = Y_POS_INVERS(y);
+			command_list = new NedCommandList(m_main_window);
+			element = findNearestElement(-1, xx, yy, NULL, &dummy);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(lp0 = new NedLinePoint(NULL, 0), element, xx, yy));
+			dummy0 = 10000000.0;
+			element = element->getStaff()->findNearestElement(xx + LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(lp1 = new NedLinePoint(NULL, 1), element, xx + LINE_START_X_DIST, yy));
+			dummy0 = 10000000.0;
+			element = element->getStaff()->findNearestElement(xx + 2 * LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
+			command_list->addCommand(new NedInsertFreePlaceableCommand(lp2 = new NedLinePoint(NULL, 1), element, xx + 2 * LINE_START_X_DIST, yy));
+			command_list->execute();
+			new NedAcceleration(lp0, lp1, lp2, m_main_window->getSpecialSubTypeInt() == LINE_RITARDANDO);
+			m_main_window->getCommandHistory()->addCommandList(command_list);
+			m_main_window->resetSpecialType();
+			m_main_window->m_selected_free_replaceable = lp0;
+			return TRUE;
 		}
-		else {
-			kind = (m_main_window->getSpecialSubTypeInt() & 0xffff) * FACTOR;
-			tempo = ((m_main_window->getSpecialSubTypeInt() >> 16) & 0xffff);
-			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedTempoSign(kind, tempo), element, xx, yy));
-		}
-		command_list->execute();
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		return TRUE;
-	case TYPE_SIGN:
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		yy = Y_POS_INVERS(y);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		command_list = new NedCommandList(m_main_window);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeSign(m_main_window->getSpecialSubTypeInt()), element, xx, yy));
-		command_list->execute();
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		return TRUE;
-	case TYPE_GUITAR_CHORD:
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		yy = Y_POS_INVERS(y);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		command_list = new NedCommandList(m_main_window);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeChord(m_main_window->getSpecialSubTypeChordInfo().chord_ptr,
-				m_main_window->getSpecialSubTypeChordInfo().chord_name_num, 
-				m_main_window->getSpecialSubTypeChordInfo().status, m_main_window), element, xx, yy));
-		command_list->execute();
-		m_main_window->setAllUnpositioned();
-		m_main_window->reposit(command_list);
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		return TRUE;
-	case TYPE_SPACER:
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		yy = Y_POS_INVERS(y);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		command_list = new NedCommandList(m_main_window);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeSpacer(m_main_window), element, xx, yy));
-		command_list->execute();
-		m_main_window->setAllUnpositioned();
-		m_main_window->reposit(command_list);
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		return TRUE;
-	case TYPE_GUITAR_CHORDNAME:
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		yy = Y_POS_INVERS(y);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		command_list = new NedCommandList(m_main_window);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedChordName(m_main_window->getDrawingArea(),m_main_window->getSpecialSubTypeChordNameInfo().root_name, m_main_window->getSpecialSubTypeChordNameInfo().up_name,m_main_window->getSpecialSubTypeChordNameInfo().down_name), element, xx, yy));
-		command_list->execute();
-		m_main_window->setAllUnpositioned();
-		m_main_window->reposit(command_list);
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		return TRUE;
-	case TYPE_FREE_TEXT:
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		yy = Y_POS_INVERS(y);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		command_list = new NedCommandList(m_main_window);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(m_main_window->m_freetext, element, xx, yy));
-		m_main_window->m_freetext->setZoom(zoom_factor, current_scale);
-		command_list->execute();
-		m_main_window->m_freetext = NULL;
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		return TRUE;
-	case TYPE_SLUR:
-#define SLUR_START_X_DIST 1.0
-#define SLUR_START_Y_DIST ( 2 * LINE_DIST )
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		if (xx > m_width / 2.0) xx -= 3 * SLUR_START_X_DIST;
-		yy = Y_POS_INVERS(y);
-		command_list = new NedCommandList(m_main_window);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		yyy = element->getSystem()->getYPos() + element->getStaff()->getMidPos();
-		command_list->addCommand(new NedInsertFreePlaceableCommand(sp[0] = new NedSlurPoint(NULL, 0), element, xx, yyy));
-		dummy0 = 10000000.0;
-		element1 = element->getStaff()->findNearestElement(xx + SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST, NULL, &dummy0, &dummy);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(sp[1] = new NedSlurPoint(NULL, 1), element1, xx + SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST));
-		dummy0 = 10000000.0;
-		element2 = element->getStaff()->findNearestElement(xx + 2 * SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST / 2, NULL, &dummy0, &dummy);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(sp[2] = new NedSlurPoint(NULL, 2), element2, xx + 2 * SLUR_START_X_DIST, yyy - SLUR_START_Y_DIST / 2));
-		command_list->execute();
-		new NedSlur(sp);
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		m_main_window->m_selected_free_replaceable = sp[2];
-		return TRUE;
-	case TYPE_LINE:
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		if (xx > m_width / 2.0) xx -= 3 * SLUR_START_X_DIST;
-		yy = Y_POS_INVERS(y);
-		command_list = new NedCommandList(m_main_window);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(lp0 = new NedLinePoint(NULL, 0), element, xx, yy));
-		dummy0 = 10000000.0;
-		element = element->getStaff()->findNearestElement(xx + LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(lp1 = new NedLinePoint(NULL, 1), element, xx + LINE_START_X_DIST, yy));
-		command_list->execute();
-		switch (m_main_window->getSpecialSubTypeInt()) {
-			case LINE_CRESCENDO:
-			case LINE_DECRESCENDO:
-					new NedCrescendo(lp0, lp1, m_main_window->getSpecialSubTypeInt() == LINE_DECRESCENDO);
-					break;
-			case LINE_OCTAVATION1:
-			case LINE_OCTAVATION_1:
-			case LINE_OCTAVATION2:
-			case LINE_OCTAVATION_2:
-					new NedOctavation(lp0, lp1, m_main_window->getSpecialSubTypeInt());
-					break;
-					
-		}
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		m_main_window->m_selected_free_replaceable = lp0;
-		return TRUE;
-	case TYPE_LINE3:
-		topy = m_main_window->getTopY();
-		xx = X_POS_INVERS_PAGE_REL(x);
-		if (xx > m_width / 2.0) xx -= 3 * SLUR_START_X_DIST;
-		yy = Y_POS_INVERS(y);
-		command_list = new NedCommandList(m_main_window);
-		element = findNearestElement(-1, xx, yy, NULL, &dummy);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(lp0 = new NedLinePoint(NULL, 0), element, xx, yy));
-		dummy0 = 10000000.0;
-		element = element->getStaff()->findNearestElement(xx + LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(lp1 = new NedLinePoint(NULL, 1), element, xx + LINE_START_X_DIST, yy));
-		dummy0 = 10000000.0;
-		element = element->getStaff()->findNearestElement(xx + 2 * LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(lp2 = new NedLinePoint(NULL, 1), element, xx + 2 * LINE_START_X_DIST, yy));
-		command_list->execute();
-		new NedAcceleration(lp0, lp1, lp2, m_main_window->getSpecialSubTypeInt() == LINE_RITARDANDO);
-		m_main_window->getCommandHistory()->addCommandList(command_list);
-		m_main_window->resetSpecialType();
-		m_main_window->m_selected_free_replaceable = lp0;
-		return TRUE;
 	}
 	lptr = g_list_first(m_systems);
 	NedSystem *nearest_system = (NedSystem *) lptr->data;
@@ -1034,10 +1109,10 @@ bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*
 			mindist = d;
 		}
 	}
-	if (nearest_system->tryInsertOrErease(x, y, newObj, force_rest)) {
-		return TRUE;
+	if (nearest_system->tryInsertOrErease(x, y, num_midi_input_notes, midi_input_chord, newObj, force_rest)) {
+		return true;
 	}
-	return FALSE;
+	return false;
 }
 
 
@@ -1137,6 +1212,14 @@ void NedPage::renumberSystems() {
 	}
 }
 
+void NedPage::resetActiveFlags() {
+	GList *lptr;
+
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->resetActiveFlags();
+	}
+}
+
 NedPage *NedPage::getNextPage() {
 	return m_main_window->getNextPage(this);
 }
@@ -1158,7 +1241,7 @@ bool NedPage::placeStaffs(int pass, NedCommandList *command_list /* = NULL */) {
 		offs += getMainWindow()->getFirstPageYOffs() / getMainWindow()->getCurrentScale();
 	}
 
-	if (pass == 0) m_ping_pong = FALSE;
+	if (pass == 0) m_ping_pong = false;
 	do_place_staffs(offs);
 	if (pass == 0 && m_system_diff < 0) {
 		if (command_list != NULL && g_list_length(m_systems) > 1) {
@@ -1207,6 +1290,14 @@ bool NedPage::placeStaffs(int pass, NedCommandList *command_list /* = NULL */) {
 	return changed;
 }
 
+void  NedPage::computeTuplets() {
+	GList *lptr;
+
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->computeTuplets();
+	}
+}
+
 int NedPage::getSystemCount() {
 	return g_list_length(m_systems);
 }
@@ -1218,7 +1309,7 @@ void NedPage::do_place_staffs(double offs) {
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
 		staffpos = ((NedSystem *) lptr->data)->placeStaffs(staffpos);
 	}
-	m_system_diff = m_height - offs - staffpos;
+	m_system_diff = m_height - staffpos;
 }
 
 NedSystem *NedPage::getNextSystem(NedSystem *system, NedCommandList *command_list /* = NULL */) {
@@ -1231,7 +1322,7 @@ NedSystem *NedPage::getNextSystem(NedSystem *system, NedCommandList *command_lis
 	if (isEmpty()) return NULL;
 
 	if ((lptr = g_list_find(m_systems, system)) == NULL) {
-		printf("gesucht war: 0x%x\n", system); fflush(stdout);
+		printf("gesucht war: 0x%p\n", system); fflush(stdout);
 		NedResource::Abort("NedPage::getNextSystem");
 	}
 
