@@ -34,6 +34,7 @@ class NedSystem;
 class NedStaff;
 class NedMeasure;
 class NedCommandList;
+class NedLyricsEditor;
 class NedSlur;
 class part;
 
@@ -44,6 +45,8 @@ class NedPage {
 		NedPage *clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedMainWindow *p_main_window, bool *staves);
 		void adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist);
 		NedSystem *appendSystem();
+		void testBeams(char *s);
+		void testForKeysig(char *s);
 		void removeSystem(NedSystem *system);
 		void insertSystem(NedSystem *system);
 		void appendSystem(NedCommandList *command_list);
@@ -61,9 +64,10 @@ class NedPage {
 		double getContentXpos();
 		double getContentWidth() {return m_width - 2 * LEFT_RIGHT_BORDER;}
 		NedMainWindow *getMainWindow() {return m_main_window;}
-		void appendElementsOfMeasureLength(part *parts, unsigned int meas_duration);
+		void copyDataOfWholeStaff(int staff_nr);
 		void handleStaffElements();
 		void adjustAccidentals();
+		void changeAccidentals(NedCommandList *command_list, int preferred_offs, bool *staff_list, GList *selected_group);
 		void setInternalPitches();
 		void transpose(int pitchdist, bool *staff_list, GList *selected_group);
 		void hideRests(NedCommandList *command_list, bool unhide, int staff_nt, int voice_nr);
@@ -71,7 +75,7 @@ class NedPage {
 		void removeUnneededAccidentals();
 		bool trySelect(double x, double y, bool only_free_placeables = false);
 		NedChordOrRest *findNearestElement(int staff_nr, double x, double y, NedStaff *oldstaff, double *ydist);
-		bool tryInsertOrErease(double x, double y, NedChordOrRest **newObj = NULL, bool force_rest = false);
+		bool tryInsertOrErease(double x, double y, int num_midi_input_notes, int *midi_input_chord /* given if insert from midikeyboard */, NedChordOrRest **newObj = NULL, bool force_rest = false);
 		void setAllUnpositioned();
 		void zoomFreeReplaceables(double zoom, double scale);
 		double convertX(double xpos);
@@ -93,6 +97,7 @@ class NedPage {
 		NedPage *getNextPage();
 		NedPage *getPreviousPage();
 		void renumberSystems();
+		void resetActiveFlags();
 		bool isFirst(NedSystem *system);
 		bool isLastPage();
 		bool isLast(NedSystem *system);
@@ -100,6 +105,7 @@ class NedPage {
 		NedSystem *getLastSystem();
 		bool tryChangeLength(NedChordOrRest *chord_or_rest);
 		bool placeStaffs(int pass, NedCommandList *command_list = NULL);
+		void computeTuplets();
 		int getNumberOfStaffs();
 		NedStaff *findStaff(double x, double y, NedMeasure **measure);
 		void savePage(FILE *fp);
@@ -130,10 +136,16 @@ class NedPage {
 		void testTies();
 		int getSystemCount();
 		bool exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
-			NedSystem *last_system, unsigned long long end_time, bool *in_alernative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames);
-		bool exportLilyChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+			NedSystem *last_system, unsigned long long end_time, bool *in_alernative, NedSlur **lily_slur,
+				unsigned int *lyrics_map, bool with_break, bool *guitar_chordnames, bool *chordnames, int *breath_script, bool keep_beams);
+		bool exportLilyGuitarChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+			NedSystem *last_system, unsigned long long end_time, bool *in_alernative, bool with_break);
+		bool exportLilyFreeChordName(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
 			NedSystem *last_system, unsigned long long end_time, bool *in_alernative, bool with_break);
 		bool exportLilyLyrics(FILE *fp, int staff_nr, int voice_nr, int line_nr, NedSystem *last_system, unsigned long long end_time, int *sil_count);
+		void collectLyrics(NedLyricsEditor *leditor, int staff_nr);
+		void setLyrics(NedCommandList *command_list, NedLyricsEditor *leditor, int staff_nr);
+		void recomputeFreeReplaceables();
 	private:
 		void do_place_staffs(double offs);
 		GList *m_systems;
