@@ -50,7 +50,117 @@
 #define NEUTRAL_X_OFFSET 0.01
 #define NEUTRAL_Y_OFFSET 0.3
 
-NedPangoCairoText::NedPangoCairoText(GdkDrawable *d, char *text, const char* font_family, PangoStyle style, PangoWeight weight, double size, double zoom, double scale, bool hide_trailing_hyphen) :
+#define HIDDEN_HYPHEN_X_OFFSET 0.01
+#define HODDEN_HYPHEN_Y_OFFSET 0.3
+
+struct NedPangoCairoText::utf8_element NedPangoCairoText::utf8code_array[] = {
+{"_", {    0,    0,  0xc2,   0xa0}},     //		NO-BREAK SPACE                     
+{NULL,  {    0,    0,  0xc2,   0xa1}},	//		INVERTED EXCLAMATION MARK	¡
+{NULL,  {    0,    0,  0xc2,   0xa2}},	//		CENT SIGN	¢
+{NULL,  {    0,    0,  0xc2,   0xa3}},	//		POUND SIGN	£
+{NULL,  {    0,    0,  0xc2,   0xa4}},	//		CURRENCY SIGN	¤
+{NULL,  {    0,    0,  0xc2,   0xa5}},	//		YEN SIGN	¥
+{NULL,  {    0,    0,  0xc2,   0xa6}},	//		BROKEN BAR	¦
+{NULL,  {    0,    0,  0xc2,   0xa7}},	//		SECTION SIGN	§
+{NULL,  {    0,    0,  0xc2,   0xa8}},	//		DIAERESIS	¨
+{"c", {    0,    0,  0xc2,   0xa9}},	//		COPYRIGHT SIGN	©
+{NULL,  {    0,    0,  0xc2,   0xaa}},	//		FEMININE ORDINAL INDICATOR	ª
+{NULL,  {    0,    0,  0xc2,   0xab}},	//		LEFT-POINTING DOUBLE ANGLE QUOTATION MARK	«
+{NULL,  {    0,    0,  0xc2,   0xac}},	//		NOT SIGN	¬
+{NULL,  {    0,    0,  0xc2,   0xad}},	//		SOFT HYPHEN	­
+{"r",  {    0,    0,  0xc2,   0xae}},	//		REGISTERED SIGN	®
+{NULL,  {    0,    0,  0xc2,   0xaf}},	//		MACRON	¯
+{NULL,  {    0,    0,  0xc2,   0xb0}},	//		DEGREE SIGN	°
+{NULL,  {    0,    0,  0xc2,   0xb1}},	//		PLUS-MINUS SIGN	±
+{NULL,  {    0,    0,  0xc2,   0xb2}},	//		SUPERSCRIPT TWO	²
+{NULL,  {    0,    0,  0xc2,   0xb3}},	//		SUPERSCRIPT THREE	³
+{NULL,  {    0,    0,  0xc2,   0xb4}},	//		ACUTE ACCENT	´
+{NULL,  {    0,    0,  0xc2,   0xb5}},	//		MICRO SIGN	µ
+{NULL,  {    0,    0,  0xc2,   0xb6}},	//		PILCROW SIGN	¶
+{NULL,  {    0,    0,  0xc2,   0xb7}},	//		MIDDLE DOT	·
+{NULL,  {    0,    0,  0xc2,   0xb8}},	//		CEDILLA	¸
+{NULL,  {    0,    0,  0xc2,   0xb9}},	//		SUPERSCRIPT ONE	¹
+{NULL,  {    0,    0,  0xc2,   0xba}},	//		MASCULINE ORDINAL INDICATOR	º
+{NULL,  {    0,    0,  0xc2,   0xbb}},	//		RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK	»
+{NULL,  {    0,    0,  0xc2,   0xbc}},	//		VULGAR FRACTION ONE QUARTER	¼
+{NULL,  {    0,    0,  0xc2,   0xbd}},	//		VULGAR FRACTION ONE HALF	½
+{NULL,  {    0,    0,  0xc2,   0xbe}},	//		VULGAR FRACTION THREE QUARTERS	¾
+{NULL,  {    0,    0,  0xc2,   0xbf}},	//		INVERTED QUESTION MARK	¿
+{NULL,  {    0,    0,  0xc3,   0x80}},	//		LATIN CAPITAL LETTER A WITH GRAVE	À
+{NULL,  {    0,    0,  0xc3,   0x81}},	//		LATIN CAPITAL LETTER A WITH ACUTE	Á
+{NULL,  {    0,    0,  0xc3,   0x82}},	//		LATIN CAPITAL LETTER A WITH CIRCUMFLEX	Â
+{NULL,  {    0,    0,  0xc3,   0x83}},	//		LATIN CAPITAL LETTER A WITH TILDE	Ã
+{NULL,  {    0,    0,  0xc3,   0x84}},	//		LATIN CAPITAL LETTER A WITH DIAERESIS	Ä
+{NULL,  {    0,    0,  0xc3,   0x85}},	//		LATIN CAPITAL LETTER A WITH RING ABOVE	Å
+{NULL,  {    0,    0,  0xc3,   0x86}},	//		LATIN CAPITAL LETTER AE	Æ
+{NULL,  {    0,    0,  0xc3,   0x87}},	//		LATIN CAPITAL LETTER C WITH CEDILLA	Ç
+{NULL,  {    0,    0,  0xc3,   0x88}},	//		LATIN CAPITAL LETTER E WITH GRAVE	È
+{NULL,  {    0,    0,  0xc3,   0x89}},	//		LATIN CAPITAL LETTER E WITH ACUTE	É
+{NULL,  {    0,    0,  0xc3,   0x8a}},	//		LATIN CAPITAL LETTER E WITH CIRCUMFLEX	Ê
+{NULL,  {    0,    0,  0xc3,   0x8b}},	//		LATIN CAPITAL LETTER E WITH DIAERESIS	Ë
+{NULL,  {    0,    0,  0xc3,   0x8c}},	//		LATIN CAPITAL LETTER I WITH GRAVE	Ì
+{NULL,  {    0,    0,  0xc3,   0x8d}},	//		LATIN CAPITAL LETTER I WITH ACUTE	Í
+{NULL,  {    0,    0,  0xc3,   0x8e}},	//		LATIN CAPITAL LETTER I WITH CIRCUMFLEX	Î
+{NULL,  {    0,    0,  0xc3,   0x8f}},	//		LATIN CAPITAL LETTER I WITH DIAERESIS	Ï
+{NULL,  {    0,    0,  0xc3,   0x90}},	//		LATIN CAPITAL LETTER ETH	Ð
+{NULL,  {    0,    0,  0xc3,   0x91}},	//		LATIN CAPITAL LETTER N WITH TILDE	Ñ
+{NULL,  {    0,    0,  0xc3,   0x92}},	//		LATIN CAPITAL LETTER O WITH GRAVE	Ò
+{NULL,  {    0,    0,  0xc3,   0x93}},	//		LATIN CAPITAL LETTER O WITH ACUTE	Ó
+{NULL,  {    0,    0,  0xc3,   0x94}},	//		LATIN CAPITAL LETTER O WITH CIRCUMFLEX	Ô
+{NULL,  {    0,    0,  0xc3,   0x95}},	//		LATIN CAPITAL LETTER O WITH TILDE	Õ
+{NULL,  {    0,    0,  0xc3,   0x96}},	//		LATIN CAPITAL LETTER O WITH DIAERESIS	Ö
+{NULL,  {    0,    0,  0xc3,   0x97}},	//		MULTIPLICATION SIGN	×
+{NULL,  {    0,    0,  0xc3,   0x98}},	//		LATIN CAPITAL LETTER O WITH STROKE	Ø
+{NULL,  {    0,    0,  0xc3,   0x99}},	//		LATIN CAPITAL LETTER U WITH GRAVE	Ù
+{NULL,  {    0,    0,  0xc3,   0x9a}},	//		LATIN CAPITAL LETTER U WITH ACUTE	Ú
+{NULL,  {    0,    0,  0xc3,   0x9b}},	//		LATIN CAPITAL LETTER U WITH CIRCUMFLEX	Û
+{NULL,  {    0,    0,  0xc3,   0x9c}},	//		LATIN CAPITAL LETTER U WITH DIAERESIS	Ü
+{NULL,  {    0,    0,  0xc3,   0x9d}},	//		LATIN CAPITAL LETTER Y WITH ACUTE	Ý
+{NULL,  {    0,    0,  0xc3,   0x9e}},	//		LATIN CAPITAL LETTER THORN	Þ
+{NULL,  {    0,    0,  0xc3,   0x9f}},	//		LATIN SMALL LETTER SHARP S	ß
+{NULL,  {    0,    0,  0xc3,   0xa0}},	//		LATIN SMALL LETTER A WITH GRAVE	à
+{NULL,  {    0,    0,  0xc3,   0xa1}},	//		LATIN SMALL LETTER A WITH ACUTE	á
+{NULL,  {    0,    0,  0xc3,   0xa2}},	//		LATIN SMALL LETTER A WITH CIRCUMFLEX	â
+{NULL,  {    0,    0,  0xc3,   0xa3}},	//		LATIN SMALL LETTER A WITH TILDE	ã
+{NULL,  {    0,    0,  0xc3,   0xa4}},	//		LATIN SMALL LETTER A WITH DIAERESIS	ä
+{NULL,  {    0,    0,  0xc3,   0xa5}},	//		LATIN SMALL LETTER A WITH RING ABOVE	å
+{NULL,  {    0,    0,  0xc3,   0xa6}},	//		LATIN SMALL LETTER AE	æ
+{NULL,  {    0,    0,  0xc3,   0xa7}},	//		LATIN SMALL LETTER C WITH CEDILLA	ç
+{NULL,  {    0,    0,  0xc3,   0xa8}},	//		LATIN SMALL LETTER E WITH GRAVE	è
+{NULL,  {    0,    0,  0xc3,   0xa9}},	//		LATIN SMALL LETTER E WITH ACUTE	é
+{NULL,  {    0,    0,  0xc3,   0xaa}},	//		LATIN SMALL LETTER E WITH CIRCUMFLEX	ê
+{NULL,  {    0,    0,  0xc3,   0xab}},	//		LATIN SMALL LETTER E WITH DIAERESIS	ë
+{NULL,  {    0,    0,  0xc3,   0xac}},	//		LATIN SMALL LETTER I WITH GRAVE	ì
+{NULL,  {    0,    0,  0xc3,   0xad}},	//		LATIN SMALL LETTER I WITH ACUTE	í
+{NULL,  {    0,    0,  0xc3,   0xae}},	//		LATIN SMALL LETTER I WITH CIRCUMFLEX	î
+{NULL,  {    0,    0,  0xc3,   0xaf}},	//		LATIN SMALL LETTER I WITH DIAERESIS	ï
+{NULL,  {    0,    0,  0xc3,   0xb0}},	//		LATIN SMALL LETTER ETH	ð
+{NULL,  {    0,    0,  0xc3,   0xb1}},	//		LATIN SMALL LETTER N WITH TILDE	ñ
+{NULL,  {    0,    0,  0xc3,   0xb2}},	//		LATIN SMALL LETTER O WITH GRAVE	ò
+{NULL,  {    0,    0,  0xc3,   0xb3}},	//		LATIN SMALL LETTER O WITH ACUTE	ó
+{NULL,  {    0,    0,  0xc3,   0xb4}},	//		LATIN SMALL LETTER O WITH CIRCUMFLEX	ô
+{NULL,  {    0,    0,  0xc3,   0xb5}},	//		LATIN SMALL LETTER O WITH TILDE	õ
+{NULL,  {    0,    0,  0xc3,   0xb6}},	//		LATIN SMALL LETTER O WITH DIAERESIS	ö
+{NULL,  {    0,    0,  0xc3,   0xb7}},	//		DIVISION SIGN	÷
+{NULL,  {    0,    0,  0xc3,   0xb8}},	//		LATIN SMALL LETTER O WITH STROKE	ø
+{NULL,  {    0,    0,  0xc3,   0xb9}},	//		LATIN SMALL LETTER U WITH GRAVE	ù
+{NULL,  {    0,    0,  0xc3,   0xba}},	//		LATIN SMALL LETTER U WITH ACUTE	ú
+{NULL,  {    0,    0,  0xc3,   0xbb}},	//		LATIN SMALL LETTER U WITH CIRCUMFLEX	û
+{NULL,  {    0,    0,  0xc3,   0xbc}},	//		LATIN SMALL LETTER U WITH DIAERESIS	ü
+{NULL,  {    0,    0,  0xc3,   0xbd}},	//		LATIN SMALL LETTER Y WITH ACUTE	ý
+{NULL,  {    0,    0,  0xc3,   0xbe}},	//		LATIN SMALL LETTER THORN	þ
+{NULL,  {    0,    0,  0xc3,   0xbf}},	//		LATIN SMALL LETTER Y WITH DIAERESIS	ÿ
+{"p", {    0,    0xe2, 0x84, 0x97}},     //              SOUND RECORDING COPYRIGHT  ℗      
+{"b", {    0,    0xe2, 0x99, 0xad}},     //              MUSIC FLAT SIGN            ♭      
+{"n", {    0,    0xe2, 0x99, 0xae}},     //              MUSIC NATURAL SIGN         ♮      
+{"#", {    0,    0xe2, 0x99, 0xaf}},     //              MUSIC SHARP SIGN           ♯      
+{"--", {   0,    0xe2, 0x80, 0x94}},     //              EM DASH                    —       
+{"...", {  0,    0xe2, 0x80, 0xa6}},     //              HORIZONTAL ELLIPSIS        …       
+{NULL,  {    0,    0,    0,      0}}
+};
+
+
+NedPangoCairoText::NedPangoCairoText(GdkDrawable *d, const char *text, const char* font_family, PangoStyle style, PangoWeight weight, double size, double zoom, double scale, bool hide_trailing_hyphen) :
 m_display_layout(NULL), m_measure_layout(NULL), m_drawable(d), m_hidden_hyphen(false), m_text(NULL), m_measure_text(NULL), m_font_family(font_family), m_style(style), m_weight(weight), m_size(size * PANGO_INTERNAL_FONT_FACTOR), m_width(0), m_height(0), m_hide_hyphen(hide_trailing_hyphen),
 	m_cursor_idx(0), m_spec_letters(NULL) {
 	if (text == NULL || strlen(text) < 1) {
@@ -71,7 +181,7 @@ m_display_layout(NULL), m_measure_layout(NULL), m_drawable(d), m_hidden_hyphen(f
 	cairo_t *cr = gdk_cairo_create (m_drawable);
 	createLayout(cr, zoom, scale);
 	cairo_destroy (cr);
-	m_cursor_idx = strlen(m_text);
+	m_cursor_idx = g_utf8_strlen(m_text, -1);
 	computeCursorPosition();
 }
 	
@@ -273,8 +383,10 @@ void NedPangoCairoText::computeDimensions() {
 	cairo_destroy (cr);
 }
 
-void NedPangoCairoText::insertAtCursorPosition(char *text, double zoom) {
+
+void NedPangoCairoText::insertAtCursorPosition(const char *text, double zoom) {
 	char str1[1024], str2[1024];
+	gchar *endp;
 	//text = "ע";
 	if (text == NULL || strlen(text) < 1) return;
 	bool RTL = pango_find_base_dir(m_text, -1) == PANGO_DIRECTION_RTL;
@@ -287,7 +399,7 @@ void NedPangoCairoText::insertAtCursorPosition(char *text, double zoom) {
 		computeCursorPosition();
 		return;
 	}
-	if (m_cursor_idx == strlen(m_measure_text)) {
+	if (m_cursor_idx == g_utf8_strlen(m_measure_text, -1)) {
 		strcpy(str1, m_text);
 		strcat(str1, text);
 		setText(str1, zoom, m_hide_hyphen);
@@ -296,9 +408,11 @@ void NedPangoCairoText::insertAtCursorPosition(char *text, double zoom) {
 		computeDimensions();
 		return;
 	}
-	strncpy(str1, m_text, m_cursor_idx);
-	str1[m_cursor_idx] = '\0';
-	strcpy(str2, m_text + m_cursor_idx);
+	g_utf8_strncpy(str1, m_text, m_cursor_idx);
+	endp = g_utf8_offset_to_pointer(str1, m_cursor_idx);
+	*endp = '\0';
+	endp = g_utf8_offset_to_pointer(m_text, m_cursor_idx);
+	strcpy(str2, endp);
 	strcat(str1, text);
 	strcat(str1, str2);
 	setText(str1, zoom, m_hide_hyphen);
@@ -346,14 +460,14 @@ void NedPangoCairoText::setCursorToLastPos() {
 	if (m_text == NULL) return;
 	bool RTL = (pango_find_base_dir(m_text, -1) == PANGO_DIRECTION_RTL);
 
-	m_cursor_idx = strlen(m_measure_text);
+	m_cursor_idx = g_utf8_strlen(m_measure_text, -1);
 	if (RTL) {
 		m_cursor_idx = 0;
 		computeCursorPosition();
 	}
 	else {
 		m_cursorpos.x = m_width;
-		m_cursor_idx = strlen(m_text);
+		m_cursor_idx = g_utf8_strlen(m_text, -1);
 	}
 }
 
@@ -361,7 +475,7 @@ void NedPangoCairoText::setCursorToFirstPos() {
 	if (m_text == NULL) return;
 	bool RTL = (pango_find_base_dir(m_text, -1) == PANGO_DIRECTION_RTL);
 	if (RTL) {
-		m_cursor_idx = strlen(m_text);
+		m_cursor_idx = g_utf8_strlen(m_text, -1);
 		computeCursorPosition();
 	}
 	else {
@@ -444,7 +558,7 @@ void NedPangoCairoText::computeCursorPosition() {
 	}
 }
 
-char *NedPangoCairoText::inspect_text(char *text) {
+char *NedPangoCairoText::inspect_text(const char *text) {
 	char Str[1024];
 	char *cptr, *cptr2, *cptr3, *cptr4;
 	specLetter *let;
@@ -473,6 +587,7 @@ char *NedPangoCairoText::inspect_text(char *text) {
 						  	cptr - Str,
 						  	SEGNO_X_OFFSET, SEGNO_Y_OFFSET);
 						  break;
+/*
 					case '#': let = new specLetter(
 						  	"\100",
 						  	cptr - Str,
@@ -493,6 +608,9 @@ char *NedPangoCairoText::inspect_text(char *text) {
 						  	NEUTRAL_Y_OFFSET);
 							small_letter = true;
 						  break;
+*/
+					default: handle_spec_utf8_chars(cptr, cptr2);
+							 break;
 				}
 				if (let != NULL) {
 					*cptr = ' ';
@@ -521,7 +639,46 @@ char *NedPangoCairoText::inspect_text(char *text) {
 	}
 	return strdup(Str);
 }
+
 	
+
+void NedPangoCairoText::handle_spec_utf8_chars(char *cptr, char *cptr2) {
+	char *cptr3;
+	int i;
+	int len_dest, len_src;
+	struct utf8_element *utf8_ptr;
+
+	for (utf8_ptr = utf8code_array; utf8_ptr->utf8code[3]; utf8_ptr++) { 
+		if (!utf8_ptr->short_chars) continue;
+		if (!strncmp(cptr2, utf8_ptr->short_chars, strlen(utf8_ptr->short_chars))) {
+			len_dest = 4;
+			for (i = 0; i < 4; i++) {
+				if (!utf8_ptr->utf8code[i]) len_dest--;
+			}
+			if (len_dest < 1) {
+				NedResource::Abort("NedPangoCairoText::handle_spec_utf8_chars");
+			}
+			len_src = strlen(utf8_ptr->short_chars) + 1;
+			if (len_dest > len_src) {
+				for (cptr3 = cptr + strlen(cptr) + (len_dest - len_src); cptr3 >= cptr + (len_dest - len_src); cptr3--) {
+					*cptr3 = *(cptr3 - (len_dest - len_src));
+				}
+			}
+			else if (len_dest < len_src) {
+				for (cptr3 = cptr + len_dest; *(cptr3 + (len_src - len_dest)); cptr3++) {
+					*cptr3 = *(cptr3 + (len_src - len_dest));
+				}
+				*(cptr3) = '\0';
+			}
+			for (i = 4 - len_dest; i < 4; i++, cptr++) {
+				*cptr = utf8_ptr->utf8code[i];
+			}
+			return;
+		}
+	}
+}
+				
+			
 NedPangoCairoText::specLetter::specLetter(const char *let, int ix, double xo, double yo) :
 		letter(let), idx(ix), xoffs(xo), yoffs(yo) {}
 
