@@ -45,7 +45,7 @@
 
 
 #define PS_ZOOM 24.0
-#define PS_ZOOM2 23.0
+#define PS_ZOOM2 22.4
 	
 
 #define LINE_DIST 0.22
@@ -55,7 +55,7 @@
 
 #define STEM_DEFAULT_HEIGHT (3.4 * LINE_DIST)
 #define STEM_THICK 0.04
-#define HEAD_THICK (1.12 * LINE_DIST)
+#define HEAD_THICK (1.2 * LINE_DIST)
 #define HEAD_THICK_HALF (1.09 * HEAD_THICK)
 #define FLAG_DIST (0.87 * LINE_DIST)
 
@@ -71,6 +71,7 @@
 
 
 #define BRACE_SPACE 0.05
+#define BRACE_X_OFFSET 5.0
 #define BRACKET_INDENT 0.2
 #define NESTED_BRACE_INDENT 0.18
 
@@ -125,7 +126,9 @@
 #define AUX_LINE_OVER (0.435 * LINE_DIST)
 #define AUX_LINE_THICK (0.087 * LINE_DIST)
 
-#define TIE_X_START (0.435 * LINE_DIST)
+#define TIE_X_START (0.456 * LINE_DIST)
+#define TIE_X_START_LARGE (0.6 * LINE_DIST)
+#define MIN_TIE_LEN 1.2
 #define TIE_Y_OFFS1 (1.74 * LINE_DIST)
 #define TIE_Y_OFFS2 (2.174 * LINE_DIST)
 #define TIE_X_RIGHT_OVER (3.48 * LINE_DIST)
@@ -183,6 +186,11 @@
 #define STAFF_SHORT_NAME_FONT_SLANT PANGO_STYLE_NORMAL
 #define STAFF_SHORT_NAME_FONT_WEIGHT PANGO_WEIGHT_NORMAL
 
+#define TEXT_FONT_NAME_DEFAULT "Sans"
+#define TEXT_FONT_SIZE_DEFAULT 0.5
+#define TEXT_FONT_SLANT_DEFAULT PANGO_STYLE_NORMAL
+#define TEXT_FONT_WEIGHT_DEFAULT PANGO_WEIGHT_NORMAL 
+
 #define SYSTEM_INTENT_SEC_SPACE 0.2
 
 #define SCORE_INFO_EXTRA_DIST 0.1
@@ -210,6 +218,8 @@
 #define NOTE_64 (NOTE_32/2)
 #define NOTE_128 (NOTE_64/2)
 
+#define GRACE_DURATION NOTE_32
+
 #define NORMAL_NOTE  0
 #define CROSS_NOTE1  1
 #define CROSS_NOTE2  2
@@ -223,6 +233,11 @@
 #define QUAD_NOTE2  10 
 #define GUITAR_NOTE_STEM  11 
 #define GUITAR_NOTE_NO_STEM  12 
+#define OPEN_HIGH_HAT 13
+#define CLOSED_HIGH_HAT 14
+
+#define COUNT_OF_DRUMS_TO_CHOOSE_DURING_MIDI_IMPORT 9
+#define OFFSET_TO_BE_USED_DURING_MIDI_IMPORT 1007
 
 #define STROKEN_GRACE (NOTE_64/2)
 #define GRACE_8 (STROKEN_GRACE/2)
@@ -240,6 +255,12 @@
 #define GRACE_BEAM_THICK 0.06
 #define GRACE_BEAM_DIST 0.1
 
+#define SIGN_BREATH1 3
+#define SIGN_BREATH2 4
+
+#define LILY_BREAH_SCRIPT_RCOMMA 0
+#define LILY_BREAH_SCRIPT_UPBOW 1
+
 #define SHIFT_MODE  (1 << 0)
 #define INSERT_EREASE_MODE  (1 << 1)
 #define KEYBOARD_CTRL_MODE (1 << 2)
@@ -290,6 +311,10 @@
 //#define EXTRA_SPACE (0.2 * LINE_DIST)
 #define WHOLE_EXTRA_SPACE (2.6 * LINE_DIST)
 
+#define TIME_SYMBOL_NONE 0
+#define TIME_SYMBOL_COMMON_TIME 1
+#define TIME_SYMBOL_CUT_TIME 2
+
 // real measure numbers
 #define MEASURE_NUMBER_X_POS -0.45
 #define MEASURE_NUMBER_Y_POS -0.45
@@ -310,7 +335,9 @@
 
 #define MIN_NEAR_DIST 0.4
 #define MIN_STAFF_BORDER_DIST 0.3
-#define SEC_DIST 0.2
+#define SEC_DIST_BELOW 0.3 
+#define SEC_DIST_ABOVE 0.1 
+#define MIN_SEC_DIST (2 * LINE_DIST)
 #define REP_NUM_XPOS 0.09
 #define REP_NUM_YPOS 0.4
 
@@ -325,9 +352,10 @@
 #define TENOR_CLEF 4
 #define NEUTRAL_CLEF1 5
 #define NEUTRAL_CLEF2 6
+#define NEUTRAL_CLEF3 7
 
 #define MINCLEF_NUMBER TREBLE_CLEF
-#define MAXCLEF_NUMBER NEUTRAL_CLEF2
+#define MAXCLEF_NUMBER NEUTRAL_CLEF3
 
 #define OCTUP_Y_TREBLE_DIST_DOWN 0.8
 #define OCTUP_Y_TREBLE_DIST_UP -1.05
@@ -344,11 +372,19 @@
 #define DEMO_LINE_DIST 9.6
 #define DEMO_POS_FAK 44.0
 
+#define MAX_MIDI_INPUT_CHORD 25
+
 #define MAX_RECENT_FILES 8
 
 #define START_EXTRA_SPACE (1.5 * LINE_DIST)
+#define OLDVALUES
+#ifdef OLDVALUES
 #define MINIMUM_EXTRA_SPACE (2.0 * LINE_DIST)
 #define MAXIMUM_EXTRA_SPACE (10.0 * LINE_DIST)
+#else
+#define MINIMUM_EXTRA_SPACE (0.0 * LINE_DIST)
+#define MAXIMUM_EXTRA_SPACE (5.0 * LINE_DIST)
+#endif
 
 #define MIN_SYSTEM_Y_DIST (1.2 * LINE_DIST)
 #define MAX_SYSTEM_Y_DIST (1.8 * LINE_DIST)
@@ -385,6 +421,10 @@
 #define HIDD_GREEN     0
 #define HIDD_BLUE    255
 
+#define NOTE_ACTIVE_STATE_NONE 0
+#define NOTE_ACTIVE_STATE_ON 1
+#define NOTE_ACTIVE_STATE_OFF 2
+
 
 #define TOLERANCE 0.05
 #define MIN_REST_Y_VOICE_DIST 0.1
@@ -402,6 +442,8 @@
 #define X11_MINI_ICONS_DIM 20
 #define X11_ICONS_BACK_MINI_DIM 25
 
+#define MAX_SCREEN_DIM 100000
+
 #define DBG_NO 0
 #define DBG_TESTING 1
 #define DBG_CRITICAL 2
@@ -417,13 +459,19 @@
 #define SEGNO_AL_CODA	     (1 << 8)
 #define SEGNO_AL_CODA2	     (1 << 9)  // "valid" al coda (during replay)
 
+
+/* Debug macros */
+#define LAUS(var) (var), (var) / NOTE_64, (var) / NOTE_4
+/*              */
 class NedNote;
 class NedChordOrRest;
 class NedMainWindow;
 class NedSystem;
+class NedStaff;
 class NedPage;
 class NedMeasure;
 class NedTools;
+class NedChordName;
 
 
 struct MidiPortStruct;
@@ -449,6 +497,7 @@ struct MidiEventStruct {
 #define PSEUDO_EVENT_MEASURE (PSEUDO_FLAG | 0x1)
 #define PSEUDO_EVENT_TEMPO_CHANGE (PSEUDO2_FLAG | 0x2)
 #define PSEUDO_EVENT_KEY_CHANGE (PSEUDO2_FLAG | 0x4)
+#define PSEUDO_EVENT_PGM_CHANGE (PSEUDO2_FLAG | 0x8)
 	int data1, data2;
 	NedNote *note;
 	NedMeasure *measure;
@@ -470,8 +519,6 @@ class NedResource {
 	public:
 		static void initialize();
 		static void cleanup();
-		static void setWindow(NedMainWindow *ned_main_window) {m_main_window = ned_main_window;}
-		static NedMainWindow *getMainWindow() {return m_main_window;}
 		static char *getFontFileName() {return m_font_file_name;}
 		static cairo_scaled_font_t *getScaledFont(int zoom_level);
 		static double getStartZoomScale() {return m_zoom_factors[START_ZOOM_LEVEL];}
@@ -494,14 +541,17 @@ class NedResource {
 		static GdkPixbuf *m_32ndnote_icon;
 		static GdkPixbuf *m_64thnote_icon;
 		static GdkPixbuf *m_eightnote_icon;
+		static GdkPixbuf *m_eightnote_dot_icon;
 		static GdkPixbuf *m_fullnote_icon;
 		static GdkPixbuf *m_halfnote_icon;
+		static GdkPixbuf *m_halfnote_dot_icon;
 		static GdkPixbuf *m_quarternote_icon;
+		static GdkPixbuf *m_quarternote_dot_icon;
 		static GdkPixbuf *m_stroken_grace_icon;
 		static GdkPixbuf *m_grace_eighth_icon;
 		static GdkPixbuf *m_grace_sixth_icon;
-
 		static GdkPixbuf *m_sixteenthnote_icon;
+		static GdkPixbuf *m_sixteenthnote_dot_icon;
 
 		static GdkPixbuf *m_rest_icon;
 
@@ -545,9 +595,26 @@ class NedResource {
 		static GdkPixbuf *m_drum8_icon;
 		static GdkPixbuf *m_drum9_icon;
 		static GdkPixbuf *m_drum10_icon;
+		static GdkPixbuf *m_drum11_icon;
+		static GdkPixbuf *m_drum12_icon;
 		static GdkPixbuf *m_guitar_note_icon;
 		static GdkPixbuf *m_guitar_note_no_stem_icon;
 
+		
+		static GdkPixbuf *m_drum1_icon_small;
+		static GdkPixbuf *m_drum2_icon_small;
+		static GdkPixbuf *m_drum4_icon_small;
+		static GdkPixbuf *m_drum6_icon_small;
+		static GdkPixbuf *m_drum8_icon_small;
+		static GdkPixbuf *m_drum3_icon_small;
+		static GdkPixbuf *m_drum5_icon_small;
+		static GdkPixbuf *m_drum7_icon_small;
+		static GdkPixbuf *m_drum9_icon_small;
+		static GdkPixbuf *m_drum10_icon_small;
+		static GdkPixbuf *m_drum11_icon_small;
+		static GdkPixbuf *m_drum12_icon_small;
+		static GdkPixbuf *m_normal_note_icon_small;
+
 		static GdkPixbuf *m_dcross_icon;
 		static GdkPixbuf *m_dflat_icon;
 		static GdkPixbuf *m_stacc_icon;
@@ -574,12 +641,20 @@ class NedResource {
 		static GdkPixbuf *m_fermata_icon;
 		static GdkPixbuf *m_arpeggio_icon;
 
+		static GdkPixbuf *m_comm_icon;
+		static GdkPixbuf *m_cut_icon;
+
+		static GdkPixbuf *m_midikeyboard_icon;
+		static GdkPixbuf *m_record_icon;
+
 		static GdkPixbuf *m_lines;
 		static int m_lines_buf_width, m_lines_buf_height;
 
 		static GdkPixbuf *m_signs;
 		static int m_signs_buf_width, m_signs_buf_height;
 
+		static int m_drum_icons_width, m_drum_icons_height;
+
 		static GdkPixbuf *m_background1;
 		static GdkPixbuf *m_mini_background1;
 		static GdkPixbuf *m_mini_background2;
@@ -592,9 +667,12 @@ class NedResource {
 
 		static NedBbox m_acc_bbox;
 		static double getFlagWidth() {return m_flag_width;}
+		static char my_getc(FILE *fp);
+		static void my_ungetc(char c, FILE *fp);
 		static void unreadWord(char *word);
 		static bool readWord(FILE *fp, char *word);
 		static bool readWordWithNum(FILE *fp, char *word);
+		static bool readWordWithAllChars(FILE *fp, char *word);
 		static bool readWordOfLength(FILE *fp, char *word);
 		static bool readString(FILE *fp, char *word);
 		static bool readTillEnd(FILE *fp, char *word);
@@ -604,8 +682,11 @@ class NedResource {
 		static bool readHex(FILE *fp, int *val);
 		static bool readFloat(FILE *fp, double *val);
 		static bool removeWhiteSpaces(char **s);
+		static void removeNonAsciiFromUTF8String(const gchar *s);
 		static const char *m_error_message;
 		static int m_input_line;
+		static void startMidiInput();
+		static void stopMidiInput();
 		static void close_sequ();
 		static void MidiProgramChange(int channel, int pgm);
 		static void MidiCtrl(unsigned int ctrl_param, int channel, int val);
@@ -623,9 +704,10 @@ class NedResource {
 		static void addMeasureEntry(NedMeasure *measure);
 		static void addTempoChange(double tempoinverse);
 		static void addFermata(unsigned int offset);
+		static void addPgmChange(unsigned int offset, int chn, int pgm);
 		static void correctFermataTempo();
 		static void handleSegnos();
-		static void prepareMidiEventList(NedMainWindow *main_window, double tempo_inverse);
+		static void prepareMidiEventList(double tempo_inverse);
 		static void startReplay(NedNote *startNote, double tempo_inverse);
 		static void collectTempoEvents(GList **tempo_chords);
 		static void copyAllBetweenMeasures(int start_measure_number, int alternative1_measure_number, int end_measure_number);
@@ -635,8 +717,10 @@ class NedResource {
 		static void increaseSystemStartTime(unsigned long long system_duration);
 		static const char *GM_Instruments[];
 		static int getNumInstruments();
-		static GList *listMidiDevices(int *index_of_subscribed_device);
-		static void subscribe(int idx);
+		static GList *listMidiOutDevices(int *index_of_subscribed_out_device);
+		static GList *listMidiInDevices(int *index_of_subscribed_in_device);
+		static void subscribe_out(int idx);
+		static void subscribe_in(int idx);
 		static void prepareAddrStruct();
 		static int addAddr(void *addr);
 		static void addAddr(int marker, void *addr);
@@ -648,8 +732,9 @@ class NedResource {
 		static void addToRecentFiles(char *fname);
 		static int getClefDist(int oldclef, int old_oct_offs, int newclef, int new_oct_offs);
 		static void reset_tools();
-		static void playImmediately(int channel, int pgm, int pitch, int vol);
-		static bool alsaSequencerOk() {return m_sequ != NULL;}
+		static void playImmediately(int channel, int pgm, int num_midi_input_notes, int *midi_input_chord, int vol);
+		static bool alsaSequencerOutOk() {return m_sequ_out != NULL;}
+		static bool alsaSequencerInOk() {return m_sequ_in != NULL;}
 		static bool m_avoid_immadiate_play;
 		static struct paper_info_struct *getPaperInfo(const char *papername);
 		static const char *m_print_cmd;
@@ -661,7 +746,8 @@ class NedResource {
 #ifdef YELP_PATH
 		static char *m_yelparg;
 #endif
-		static GList *m_main_windows, *m_window_with_last_focus;
+		static GList *m_main_windows;
+		static NedMainWindow *m_window_with_last_focus;
 		static void appendTempFileName(char *fname);
 		static const char *m_last_folder;
 		static char *m_last_xml_dir;
@@ -674,6 +760,9 @@ class NedResource {
 		static void setLengthField(unsigned int length_field_value) {m_length_field_value = length_field_value;}
 		static unsigned int getPartLength(int *dotcount);
 		static unsigned int getPartLength(int *dotcount, bool is_rest, bool build_whole_rests, unsigned int measure_duration);
+		static void setLengthField2(unsigned long long measure_start, unsigned long long note_start, unsigned int length_field_value);
+		static unsigned int getPartLength2(int *dotcount);
+		static unsigned int getPartLength2(int *dotcount, bool is_rest, bool build_whole_rests, unsigned int measure_duration);
 		static bool fittingDuration(unsigned int duration);
 		static bool fittingPosition(unsigned long long position);
 		static unsigned int subtime(struct timeval *future, struct timeval *now);
@@ -683,6 +772,7 @@ class NedResource {
 		static const char *getLilyPondClefName(int clef_number);
 		static const char *getLilyPondKeySigName(int keysig_number);
 		static void makeLilyString(char *ori, char *buffer);
+		static void writeLilyFreeChord(FILE *fp, NedChordName *chordname, unsigned int length, unsigned int duration, int dotcount, unsigned int *midi_len);
 		static unsigned int getCurrentLength();
 		static int getCurrentNoteHead();
 		static unsigned int getStatus();
@@ -695,6 +785,7 @@ class NedResource {
 		static void popup_tools(NedMainWindow *w);
 		static void resetSomeButtons();
 		static void resetButtons();
+		static void resetTieButton();
 		static void toggleButton(int nr);
 		static void toggleRest();
 		static void toggleTie();
@@ -703,7 +794,11 @@ class NedResource {
 		static void toggleNatural();
 		static void toggleFlat();
 		static NedMainWindow *getMainwWindowWithLastFocus();
+		static NedStaff *m_recorded_staff;
 		static void setMainwWindowWithFocus(NedMainWindow *w);
+		static snd_seq_t *getMidiInSequ() {return m_sequ_in;}
+		static snd_seq_t *getMidiOutSequ() {return m_sequ_out;}
+		static gint m_toolboxX, m_toolboxY;
 	private:
 		static void printMidiEvent(MidiEventStruct *ev);
 		static void removeAllPseudosEvents();
@@ -715,13 +810,19 @@ class NedResource {
 		static void handleDSalFine(bool da_capo);
 		static void handleDSalCoda(bool da_capo);
 		static struct paper_info_struct m_paper_info[];
+		static void process_midi_event();
+		static gboolean do_poll_midi_device(void *data);
 		static gboolean stopImmediate(void *data);
-		static GList *midi_devices;
+		static int m_number_of_midi_poll_fds;
+		static bool m_run_midi_in;
+		static GList *m_midi_out_devices, *m_midi_in_devices;
 		static GList *m_midi_events;
 		static GList *m_play_ptr;
 		static int m_handler_id;
-		static snd_seq_addr_t m_alsaaddr;
-		static snd_seq_addr_t m_alsa_dest_addr;
+		static snd_seq_addr_t m_alsa_out_addr;
+		static snd_seq_addr_t m_alsa_in_addr;
+		static snd_seq_addr_t m_alsa_out_dest_addr;
+		static snd_seq_addr_t m_alsa_in_src_addr;
 		static unsigned long long m_system_start_time;
 		static bool m_channel_list[16];
 		static GList *m_addr_rel_list;
@@ -729,7 +830,6 @@ class NedResource {
 		static void read_config();
 		static bool permtest(snd_seq_port_info_t *pinfo, unsigned int perm);
 		static void placeMidiEvent(MidiEventStruct *ev);
-		static NedMainWindow *m_main_window;
 		static int m_marker_counter;
 		static gboolean playNext(void *data);
 		static bool m_playing;
@@ -745,6 +845,7 @@ class NedResource {
 		static cairo_scaled_font_t *createScaledFont(int zoom_level);
 
 		static unsigned int m_length_field_value;
+		static unsigned int m_part_start, m_part_end;
 
 		static void init_fonts(char *font_file_name);
 		static int loadfontface(char *font_file_name);
@@ -754,9 +855,10 @@ class NedResource {
 		static NedTools *m_tools;
 		static double m_flag_width;
 		static char m_word_buffer[128];
-		static snd_seq_t *m_sequ;
-		static int m_last_immediate_pitch;
-		static int m_last_immediate_channel;
+		static snd_seq_t *m_sequ_out;
+		static snd_seq_t *m_sequ_in;
+		static int m_count_of_last_imm_pitches;
+		static snd_seq_event_t m_last_imm_notes[MAX_MIDI_INPUT_CHORD];
 		static GList *m_temp_file_names;
 		static double m_tempo_inverse;
 		static unsigned long long m_accel_midi_start;
