@@ -189,7 +189,7 @@ void NedStaff::collectSelectionRectangleElements(double xp, double yp, NedBbox *
 		bool is_first_selected, bool is_last_selected) {
 	int i;
 	if (is_first_selected && m_ypos + 4 * LINE_DIST < yp) return;
-	if (is_last_selected && yp + sel_rect->height < m_ypos) return;
+	if (is_last_selected && yp + sel_rect->height < m_ypos)  return;
 	for (i = 0; i < VOICE_COUNT; i++) {
 		 m_voices[i]->collectSelectionRectangleElements(xp, yp, sel_rect, sel_group, is_first_selected, is_last_selected);
 	}
@@ -317,7 +317,7 @@ GList *NedStaff::getStaffElements(unsigned long long midi_time) {
 }
 
 
-void NedStaff::draw(cairo_t *cr, double botpos, double botpos2, bool first_page, double indent, bool *freetexts_or_lyrics_present) {
+void NedStaff::draw(cairo_t *cr, double botpos, bool first_page, double indent, bool *freetexts_or_lyrics_present) {
 	GList *lptr;
 	double leftx = getMainWindow()->getLeftX();
 	double topy = getMainWindow()->getTopY();
@@ -394,9 +394,9 @@ void NedStaff::draw(cairo_t *cr, double botpos, double botpos2, bool first_page,
 						 oct_shift = true; break;
 				  }
 				clef_y_offs = -LINE_DIST;break;
-		case NEUTRAL_CLEF1: glyph[0].index = BASE + 27;
-				break;
-		case NEUTRAL_CLEF2: glyph[0].index = BASE + 27;
+		case NEUTRAL_CLEF1: 
+		case NEUTRAL_CLEF2: 
+		case NEUTRAL_CLEF3: glyph[0].index = BASE + 27;
 				break;
 		default: glyph[0].index = BASE + 3;
 				  switch (m_clef_octave_shift) {
@@ -439,40 +439,58 @@ void NedStaff::draw(cairo_t *cr, double botpos, double botpos2, bool first_page,
 		cairo_stroke(cr);
 	}
 	if (first_page || getMainWindow()->isTimsigChangingMeasure(getSystem()->getNumberOfFirstMeasure())) {
-		int n = getMainWindow()->getNumerator(getSystem()->getNumberOfFirstMeasure());
-		if (n > 9) {
-			glyph[0].index = NUMBERBASE + n / 10;
-			glyph[0].x =  X_POS_PAGE_REL(sig_xpos - SIG_HALF);
-			glyph[0].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
-			glyph[1].index = NUMBERBASE + n % 10;
-			glyph[1].x =  X_POS_PAGE_REL(sig_xpos + SIG_HALF);
-			glyph[1].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
-			cairo_show_glyphs(cr, glyph, 2);
-		}
-		else {
-	
-			glyph[0].index = NUMBERBASE + n;
-			glyph[0].x =  X_POS_PAGE_REL(sig_xpos);
-			glyph[0].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
-			cairo_show_glyphs(cr, glyph, 1);
-		}
-		n = getMainWindow()->getDenominator(getSystem()->getNumberOfFirstMeasure());
-		if (n > 9) {
-			glyph[0].index = NUMBERBASE + n / 10;
-			glyph[0].x =  X_POS_PAGE_REL(sig_xpos - SIG_HALF);
-			glyph[0].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
-			glyph[1].index = NUMBERBASE + n % 10;
-			glyph[1].x =  X_POS_PAGE_REL(sig_xpos + SIG_HALF);
-			glyph[1].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
-			cairo_show_glyphs(cr, glyph, 2);
-		}
-		else {
-			glyph[0].index = NUMBERBASE + n;
-			glyph[0].x =  X_POS_PAGE_REL(sig_xpos);
-			glyph[0].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
-			cairo_show_glyphs(cr, glyph, 1);
-		}
-		cairo_stroke(cr);
+		switch (getMainWindow()->getTimesigSymbol(getSystem()->getNumberOfFirstMeasure())) {
+			case TIME_SYMBOL_COMMON_TIME:
+				glyph[0].index = BASE + 56;
+				glyph[0].x = X_POS_PAGE_REL(sig_xpos);
+				glyph[0].y = Y_POS_STAFF_REL(-3 * LINE_DIST/2.0);
+				cairo_show_glyphs(cr, glyph, 1);
+				cairo_stroke(cr);
+				break;
+			case TIME_SYMBOL_CUT_TIME:
+				glyph[0].index = BASE + 57;
+				glyph[0].x = X_POS_PAGE_REL(sig_xpos);
+				glyph[0].y = Y_POS_STAFF_REL(-3 * LINE_DIST/2.0);
+				cairo_show_glyphs(cr, glyph, 1);
+				cairo_stroke(cr);
+				break;
+			default:
+				int n = getMainWindow()->getNumerator(getSystem()->getNumberOfFirstMeasure());
+				if (n > 9) {
+					glyph[0].index = NUMBERBASE + n / 10;
+					glyph[0].x =  X_POS_PAGE_REL(sig_xpos - SIG_HALF);
+					glyph[0].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
+					glyph[1].index = NUMBERBASE + n % 10;
+					glyph[1].x =  X_POS_PAGE_REL(sig_xpos + SIG_HALF);
+					glyph[1].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
+					cairo_show_glyphs(cr, glyph, 2);
+				}
+				else {
+			
+					glyph[0].index = NUMBERBASE + n;
+					glyph[0].x =  X_POS_PAGE_REL(sig_xpos);
+					glyph[0].y = Y_POS_STAFF_REL(Y_NUMRATOR_POS);
+					cairo_show_glyphs(cr, glyph, 1);
+				}
+				n = getMainWindow()->getDenominator(getSystem()->getNumberOfFirstMeasure());
+				if (n > 9) {
+					glyph[0].index = NUMBERBASE + n / 10;
+					glyph[0].x =  X_POS_PAGE_REL(sig_xpos - SIG_HALF);
+					glyph[0].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
+					glyph[1].index = NUMBERBASE + n % 10;
+					glyph[1].x =  X_POS_PAGE_REL(sig_xpos + SIG_HALF);
+					glyph[1].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
+					cairo_show_glyphs(cr, glyph, 2);
+				}
+				else {
+					glyph[0].index = NUMBERBASE + n;
+					glyph[0].x =  X_POS_PAGE_REL(sig_xpos);
+					glyph[0].y = Y_POS_STAFF_REL(Y_DENOMINATOR_POS);
+					cairo_show_glyphs(cr, glyph, 1);
+				}
+				cairo_stroke(cr);
+				break;
+	    	} // switch
 	}
 	
 	m_freetexts_or_lyrics_present = false;
@@ -480,12 +498,12 @@ void NedStaff::draw(cairo_t *cr, double botpos, double botpos2, bool first_page,
 		m_voices[i]->draw(cr, &m_freetexts_or_lyrics_present);
 
 	}
-	int mcount = getSystem()->getMeasureCount();
+	int mcount = getSystem()->getFullMeasureCount();
 	bool rep_line_needed = getMainWindow()->needsARepLine(getSystem()->getMeasureNr(0)->getMeasureNumber());
-	getSystem()->getMeasureNr(0)->draw(cr, SPECIAL_POSITION_START, m_system->m_system_start + REP_THICK_LINE_DIST - indent, getPage()->getContentXpos() + indent, getSystem()->getYPos(), m_ypos, botpos, botpos2, getBottomPos(), leftx, topy, m_top_y_border, zoom_factor, zoom_level, rep_line_needed, false);
+	getSystem()->getMeasureNr(0)->draw(cr, SPECIAL_POSITION_START, m_system->m_system_start + REP_THICK_LINE_DIST - indent, getPage()->getContentXpos() + indent, getSystem()->getYPos(), m_ypos - LINE_THICK / 2.0, botpos + LINE_THICK / 2.0, getBottomPos(), leftx, topy, m_top_y_border, zoom_factor, zoom_level, rep_line_needed, getSystem()->getSpecialMeasuresAtEnd(), false);
 	for (i = 1; i <= mcount; i++) {
 		rep_line_needed = getMainWindow()->needsARepLine(getSystem()->getMeasureNr(i)->getMeasureNumber());
-		getSystem()->getMeasureNr(i)->draw(cr, (i == mcount) ? SPECIAL_POSITION_END : SPECIAL_POSITION_NONE, m_width, getPage()->getContentXpos(),  getSystem()->getYPos(), m_ypos, botpos, botpos2, getBottomPos(), leftx, topy, m_top_y_border, zoom_factor, zoom_level, rep_line_needed, getSystem()->getSystemNumber() == 4);
+		getSystem()->getMeasureNr(i)->draw(cr, (i == mcount) ? SPECIAL_POSITION_END : SPECIAL_POSITION_NONE, m_width, getPage()->getContentXpos(),  getSystem()->getYPos(), m_ypos - LINE_THICK / 2.0, botpos + LINE_THICK / 2.0, getBottomPos(), leftx, topy, m_top_y_border, zoom_factor, zoom_level, rep_line_needed, getSystem()->getSpecialMeasuresAtEnd(), false);
 	}
 	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
 		if (((NedChordOrRest *) lptr->data)->getMidiTime() == 0 && ((NedChordOrRest *) lptr->data)->getType() == TYPE_CLEF) continue;
@@ -563,7 +581,7 @@ double NedStaff::getRealYPosOfLine(int line) {
 	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
 	double topy = getMainWindow()->getTopY();
 
-	return Y_POS_SYS_REL(m_ypos + 4 * LINE_DIST + line * LINE_DIST / 2.0);
+	return getMainWindow()->getCurrentScale() * Y_POS_SYS_REL(m_ypos + 4 * LINE_DIST + line * LINE_DIST / 2.0);
 }
 
 void NedStaff::empty() {
@@ -635,6 +653,14 @@ void NedStaff::zoomFreeReplaceables(double zoom, double scale) {
 	}
 }
 
+void NedStaff::recomputeFreeReplaceables() {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->recomputeFreeReplaceables();
+	}
+}
+
 void NedStaff::testTies() {
 	int i;
 
@@ -730,10 +756,17 @@ void NedStaff::appendElementsOfMeasureLength(part *part_ptr, unsigned int meas_d
 	}
 }
 
-bool NedStaff::tryInsertOrErease(double x, double y, NedChordOrRest **newObj, bool force_rest) {
-	return m_voices[getMainWindow()->getCurrentVoice()]->tryInsertOrErease(x, y, newObj, force_rest);
+void NedStaff::copyDataOfWholeStaff() {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->copyDataOfWholeVoice();
+	}
 }
 
+bool NedStaff::tryInsertOrErease(double x, double y, int num_midi_input_notes, int *midi_input_chord /* given if insert from midikeyboard */, NedChordOrRest **newObj, bool force_rest) {
+	return m_voices[getMainWindow()->getCurrentVoice()]->tryInsertOrErease(x, y, num_midi_input_notes, midi_input_chord, newObj, force_rest);
+}
 
 bool NedStaff::findLine(double x, double y, double *ypos, int *line, double *bottom) {
 	double topy = getMainWindow()->getTopY();
@@ -785,11 +818,17 @@ void NedStaff::computeTies() {
 	}
 }
 
-void NedStaff::computeTuplets() {
+void NedStaff::computeTuplets(bool has_repeat_lines) {
 	int i;
 	for (i = 0; i < VOICE_COUNT; i++) {
 		m_voices[i]->computeTuplets(&m_top_y_border, &m_bottom_y_border);
 	}
+	if (has_repeat_lines) {
+		if (m_top_y_border > -MIN_SEC_DIST) {
+			m_top_y_border = -MIN_SEC_DIST;
+		}
+		m_top_y_border -= SEC_DIST_BELOW + SEC_DIST_ABOVE;
+	}
 }
 
 double NedStaff::placeStaff(double staffpos) {
@@ -831,7 +870,7 @@ double NedStaff::placeStaff(double staffpos) {
 }
 
 void NedStaff::insertStaffElement(NedChordOrRest *elem, bool redraw /* = true */) {
-	m_staffelems = g_list_insert_sorted(m_staffelems, elem, (gint (*)(const void*, const void*)) NedChordOrRest::compareMidiTimes);
+	m_staffelems = g_list_insert_sorted(m_staffelems, elem, (gint (*)(const void*, const void*)) NedChordOrRest::compareMidiTimesAndTypes);
 	if (redraw) {
 		getMainWindow()->setAndUpdateClefTypeAndKeySig();
 		getMainWindow()->reposit(NULL, NULL, NULL);
@@ -860,7 +899,6 @@ void NedStaff::assignMidiTimes() {
 
 void NedStaff::deleteStaffElements(bool unexecute, GList *elems, unsigned long long reftime, unsigned long duration) {
 	GList *lptr1, *lptr2;
-	NedMeasure *measure;
 
 	for (lptr1 = g_list_first(elems); lptr1; lptr1 = g_list_next(lptr1)) {
 		if ((lptr2 = g_list_find(m_staffelems, lptr1->data)) == NULL) {
@@ -927,8 +965,8 @@ void NedStaff::collectChordsAndRests(NedClipBoard *board, unsigned long long mid
 	NedClipBoardElement *element = new NedClipBoardElement();
 	element->from_staff = this;
 	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
-		if (((NedChordOrRest *) lptr->data)->getMidiTime() < midi_start) continue;
-		if (((NedChordOrRest *) lptr->data)->getMidiTime() >= midi_end) break;
+		if (((NedChordOrRest *) lptr->data)->getMidiTime() <= midi_start) continue;
+		if (((NedChordOrRest *) lptr->data)->getMidiTime() > midi_end) break;
 		element->chords_or_rests_to_move = g_list_append(element->chords_or_rests_to_move, lptr->data);
 	}
 	board->m_elements = g_list_append(board->m_elements, element);
@@ -986,6 +1024,22 @@ int NedStaff::computeVolume(unsigned long long midi_time) {
 	return volume;
 }
 
+void NedStaff::prepareMutedReplay() {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->prepareMutedReplay();
+	}
+}
+
+void NedStaff::resetActiveFlags() {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->resetActiveFlags();
+	}
+}
+
 void NedStaff::prepareReplay(bool with_keysig) {
 	int i;
 	GList *lptr;
@@ -1047,6 +1101,18 @@ void NedStaff::adjustAccidentals() {
 	}
 }
 
+void NedStaff::changeAccidentals(NedCommandList *command_list, int preferred_offs, GList *selected_group) {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->setInternalPitch(0, selected_group);
+	}
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->changeAccidentals(command_list, preferred_offs, selected_group);
+	}
+	removeUnneededAccidentals(selected_group);
+}
+
 
 void NedStaff::transpose(int pitchdist, GList *selected_group) {
 	int i;
@@ -1109,16 +1175,22 @@ bool NedStaff::detectVoices(unsigned int *voice_mask, unsigned long long *e_time
 }
 
 void NedStaff::exportLilyPond(FILE *fp, int voice_nr, int *last_line, unsigned int *midi_len,
-		bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames) {
+		bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur,
+		unsigned int *lyrics_map, bool with_break, bool *guitar_chordnames, bool *chordnames, int *breath_script, bool keep_beams) {
 	if (voice_nr < 0 || voice_nr >= VOICE_COUNT) {
 		NedResource::Abort("NedStaff::exportLilyPond");
 	}
-	m_voices[voice_nr]->exportLilyPond(fp, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map, with_break, chordnames);
+	m_voices[voice_nr]->exportLilyPond(fp, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map, with_break, guitar_chordnames, chordnames, breath_script, keep_beams);
 }
 
-void NedStaff::exportLilyChordnames(FILE *fp, int *last_line, unsigned int *midi_len,
+void NedStaff::exportLilyGuitarChordnames(FILE *fp, int *last_line, unsigned int *midi_len,
 		bool last_system, unsigned long long end_time, bool *in_alternative, bool with_break) {
-	m_voices[0]->exportLilyChordnames(fp, last_line, midi_len, last_system, end_time, in_alternative, with_break);
+	m_voices[0]->exportLilyGuitarChordnames(fp, last_line, midi_len, last_system, end_time, in_alternative, with_break);
+}
+
+void NedStaff::exportLilyFreeChordName(FILE *fp, int *last_line, unsigned int *midi_len,
+		bool last_system, unsigned long long end_time, bool *in_alternative, bool with_break) {
+	m_voices[0]->exportLilyFreeChordName(fp, last_line, midi_len, last_system, end_time, in_alternative, with_break);
 }
 
 void NedStaff::exportLilyLyrics(FILE *fp, bool last_system, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count) {
@@ -1128,25 +1200,69 @@ void NedStaff::exportLilyLyrics(FILE *fp, bool last_system, int voice_nr, int li
 	m_voices[voice_nr]->exportLilyLyrics(fp, last_system, line_nr, end_time, sil_count);
 }
 	
+void NedStaff::collectLyrics(NedLyricsEditor *leditor) {
+	m_voices[0]->collectLyrics(leditor);
+}
 
-int NedStaff::assignElementsToMeasures(NedMeasure *measures, int staff_offs, bool use_upbeat) {
+void NedStaff::setLyrics(NedCommandList *command_list, NedLyricsEditor *leditor) {
+	m_voices[0]->setLyrics(command_list, leditor);
+}
+	
+
+int NedStaff::assignElementsToMeasures(NedMeasure *measures, int staff_offs, bool use_upbeat, int *special_measure_at_end, bool out /* = false */) {
 	int mesure_count = 0, mc;
+	NedMeasure *measure;
+	NedChordOrRest *element;
 	GList *lptr;
 	int curr_measure = 0;
+	bool last_measure_has_keysig_or_timesig = false;
+	NedChordOrRest *last_keysig = NULL;
 	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
-		while (((NedChordOrRest *) lptr->data)->getMidiTime() > measures[curr_measure].midi_end) {
-			curr_measure++;
+		element = (NedChordOrRest *) lptr->data;
+		switch (element->getType()) {
+			case TYPE_KEYSIG:  while (((NedChordOrRest *) lptr->data)->getMidiTime() >= measures[curr_measure].midi_end) {
+							curr_measure++; // KEYSIG after bar
+							last_keysig = (NedChordOrRest *) lptr->data;
+							if (((NedChordOrRest *) lptr->data)->getMidiTime() == measures[curr_measure].midi_start) {
+#define KEYSIG_EXTRA_WIDTH 0.2
+								measures[curr_measure].m_staff_elements_width = ((NedChordOrRest *) lptr->data)->getBBox()->width + KEYSIG_EXTRA_WIDTH;
+							}
+				
+				  }
+				  break;
+			default: while (((NedChordOrRest *) lptr->data)->getMidiTime() > measures[curr_measure].midi_end) {
+					curr_measure++; // CLEF before bar
+				  }
+				  break;
 		}
-		if (((NedChordOrRest *) lptr->data)->getMidiTime() > measures[curr_measure].midi_end) continue;
-		if (((NedChordOrRest *) lptr->data)->getMidiTime() == 0 && ((NedChordOrRest *) lptr->data)->getType() == TYPE_CLEF) continue;
-		measures[curr_measure].m_position_array.insertStaffElement(
-			(NedChordOrRest *) lptr->data, staff_offs / VOICE_COUNT, measures[curr_measure].getMeasureNumber());
-		((NedChordOrRest *) lptr->data)->m_measure = &(measures[curr_measure]);
+		measure = measures + curr_measure;
+		//if (((NedChordOrRest *) lptr->data)->getMidiTime() > measures[curr_measure].midi_end) continue;
+		//if (((NedChordOrRest *) lptr->data)->getMidiTime() == 0 && ((NedChordOrRest *) lptr->data)->getType() == TYPE_CLEF) continue;
+		//if (((NedChordOrRest *) lptr->data)->getMidiTime() == 0 && (((NedChordOrRest *) lptr->data)->getType() & (TYPE_CLEF | TYPE_KEYSIG)) != 0) continue;
+		measure->m_position_array.insertStaffElement(
+			element, staff_offs / VOICE_COUNT, measure->getMeasureNumber());
+		element->m_measure = measure;
 		if (curr_measure > mesure_count) mesure_count = curr_measure;
 	}
+	if (last_keysig && last_keysig->getMidiTime() == measures[curr_measure].midi_start) {
+		last_measure_has_keysig_or_timesig = true;
+	}
 	for (int i = 0; i < VOICE_COUNT; i++) {
 		mc = m_voices[i]->assignElementsToMeasures(measures, staff_offs + i, use_upbeat);
-		if (mc > mesure_count) mesure_count = mc;
+		if (mc > mesure_count) {
+			mesure_count = mc;
+		}
+		if (mc >= mesure_count) {
+			last_measure_has_keysig_or_timesig = false;
+		}
+	}
+	if (last_measure_has_keysig_or_timesig) { // the last measure isn't regulaer
+		mesure_count--;
+		*special_measure_at_end = 1;
+	}
+	else if ((measures[mesure_count+1].getSpecial() & TIMESIG_TYPE_MASK) == TIMESIG) {
+		*special_measure_at_end = 1;
+		last_measure_has_keysig_or_timesig = true;
 	}
 	return mesure_count;
 }
@@ -1164,7 +1280,7 @@ double NedStaff::computeMidDist(double y) {
 }
 
 void NedStaff::saveStaff(FILE *fp) {
-	int i;
+	int i, ll;
 	GList *lptr;
 	fprintf(fp, "STAFF %d\n", m_staff_number);
 	if (g_list_length(m_staffelems) > 0) {
@@ -1181,12 +1297,16 @@ void NedStaff::saveStaff(FILE *fp) {
 						case TENOR_CLEF: fprintf(fp, "TENOR_CLEF, "); break;
 						case NEUTRAL_CLEF1: fprintf(fp, "NEUTRAL_CLEF1, "); break;
 						case NEUTRAL_CLEF2: fprintf(fp, "NEUTRAL_CLEF2, "); break;
+						case NEUTRAL_CLEF3: fprintf(fp, "NEUTRAL_CLEF3, "); break;
 					}
 					fprintf(fp, " %d, ", ((NedChordOrRest *) lptr->data)->getDotCount()); // octave shift
 					break;
 				case TYPE_KEYSIG:
-					int ll = ((NedChordOrRest *) lptr->data)->getLength();
-					fprintf(fp, "KEYSIG, %d, ", ll); break;
+					ll = ((NedChordOrRest *) lptr->data)->getLength();
+					fprintf(fp, "KEYSIG, %d, ", ll); 
+					break;
+				case TYPE_STAFF_SIGN:
+					fprintf(fp, "STAFF_SIGN, %d, ", ((NedChordOrRest *) lptr->data)->getDotCount()); // sub_type
 					break;
 			}
 			fprintf(fp, "%llu ) ", ((NedChordOrRest *) lptr->data)->getMidiTime());
@@ -1204,7 +1324,7 @@ void NedStaff::restoreStaff(FILE *fp) {
 	bool was_clef;
 	int octave_shift;
 	char buffer[128];
-	int cleftype, keysignum;
+	int staff_elem_type, keysignum, staff_sign_sub_type;
 	unsigned long long miditime;
 	NedChordOrRest *staffelem;
 	
@@ -1225,42 +1345,46 @@ void NedStaff::restoreStaff(FILE *fp) {
 			return;
 		}
 		while (!strcmp(buffer, "(")) {
-			if (!NedResource::readWord(fp, buffer)) {
-				NedResource::m_error_message = "TREBLE_CLEF, BASS_CLEF, ALTO_CLEF, SOPRAN_CLEF, TENOR_CLEF, NEUTRAL_CLEF1, NEUTRAL_CLEF2, or KEYSIG expected";
+			if (!NedResource::readWordWithNum(fp, buffer)) {
+				NedResource::m_error_message = "TREBLE_CLEF, BASS_CLEF, ALTO_CLEF, SOPRAN_CLEF, TENOR_CLEF, NEUTRAL_CLEF1, NEUTRAL_CLEF2, NEUTRAL_CLEF3, KEYSIG, or STAFF_SIGN  expected";
 				return;
 			}
 			was_clef = false;
 			octave_shift = 0;
 			if (!strcmp(buffer, "TREBLE_CLEF")) {
-				cleftype = TREBLE_CLEF;
+				staff_elem_type = TREBLE_CLEF;
 				was_clef = true;
 			}
 			else if (!strcmp(buffer, "BASS_CLEF")) {
-				cleftype = BASS_CLEF;
+				staff_elem_type = BASS_CLEF;
 				was_clef = true;
 			}
 			else if (!strcmp(buffer, "ALTO_CLEF")) {
-				cleftype = ALTO_CLEF;
+				staff_elem_type = ALTO_CLEF;
 				was_clef = true;
 			}
 			else if (!strcmp(buffer, "SOPRAN_CLEF")) {
-				cleftype = SOPRAN_CLEF;
+				staff_elem_type = SOPRAN_CLEF;
 				was_clef = true;
 			}
 			else if (!strcmp(buffer, "TENOR_CLEF")) {
-				cleftype = TENOR_CLEF;
+				staff_elem_type = TENOR_CLEF;
 				was_clef = true;
 			}
 			else if (!strcmp(buffer, "NEUTRAL_CLEF1")) {
-				cleftype = NEUTRAL_CLEF1;
+				staff_elem_type = NEUTRAL_CLEF1;
 				was_clef = true;
 			}
 			else if (!strcmp(buffer, "NEUTRAL_CLEF2")) {
-				cleftype = NEUTRAL_CLEF2;
+				staff_elem_type = NEUTRAL_CLEF2;
+				was_clef = true;
+			}
+			else if (!strcmp(buffer, "NEUTRAL_CLEF3")) {
+				staff_elem_type = NEUTRAL_CLEF3;
 				was_clef = true;
 			}
 			else if (!strcmp(buffer, "KEYSIG")) {
-				cleftype = TYPE_KEYSIG;
+				staff_elem_type = TYPE_KEYSIG;
 				if (!NedResource::readWord(fp, buffer) || strcmp(buffer, ",")) {
 					NedResource::m_error_message = ", expected";
 					return;
@@ -1270,8 +1394,20 @@ void NedStaff::restoreStaff(FILE *fp) {
 					return;
 				}
 			}
+			else if (!strcmp(buffer, "STAFF_SIGN")) {
+				staff_elem_type = TYPE_STAFF_SIGN;
+				if (!NedResource::readWord(fp, buffer) || strcmp(buffer, ",")) {
+					NedResource::m_error_message = ", expected";
+					return;
+				}
+				if (!NedResource::readInt(fp, &staff_sign_sub_type)) {
+					NedResource::m_error_message = "staff sign number expected";
+					return;
+				}
+			}
 			else {
-				NedResource::m_error_message = "TREBLE_CLEF, BASS_CLEF,  ALTO_CLEF, SOPRAN_CLEF, TENOR_CLEF, NEUTRAL_CLEF1, NEUTRAL_CLEF2, or KEYSIG expected";
+				printf("buffer = %s\n", buffer); fflush(stdout);
+				NedResource::m_error_message = "TREBLE_CLEF, BASS_CLEF,  ALTO_CLEF, SOPRAN_CLEF, TENOR_CLEF, NEUTRAL_CLEF1, NEUTRAL_CLEF2, NEUTRAL_CLEF3, KEYSIG, or STAFF_SIGN expected";
 				return;
 			}
 			if (was_clef && getMainWindow()->getFileVersion() > 4) {
@@ -1297,13 +1433,16 @@ void NedStaff::restoreStaff(FILE *fp) {
 				NedResource::m_error_message = ") expected";
 				return;
 			}
-			if (cleftype == TYPE_KEYSIG) {
+			if (staff_elem_type == TYPE_KEYSIG) {
 				staffelem = new NedChordOrRest(m_voices[0], TYPE_KEYSIG, keysignum, 0, miditime);
 			}
-			else {
-				staffelem = new NedChordOrRest(m_voices[0], TYPE_CLEF, cleftype, octave_shift, miditime, true);
+			else if (staff_elem_type >= TREBLE_CLEF && staff_elem_type <= NEUTRAL_CLEF3) {
+				staffelem = new NedChordOrRest(m_voices[0], TYPE_CLEF, staff_elem_type, octave_shift, miditime, true);
 			}
-			m_staffelems = g_list_insert_sorted(m_staffelems, staffelem, (gint (*)(const void*, const void*)) NedChordOrRest::compareMidiTimes);
+			else {
+				staffelem = new NedChordOrRest(m_voices[0], TYPE_STAFF_SIGN, staff_sign_sub_type, miditime);
+			}
+			m_staffelems = g_list_insert_sorted(m_staffelems, staffelem, (gint (*)(const void*, const void*)) NedChordOrRest::compareMidiTimesAndTypes);
 			if (!NedResource::readWord(fp, buffer)) {
 				NedResource::m_error_message = "( or VOICE expected";
 				return;
