@@ -35,6 +35,7 @@ class NedVoice;
 class NedChordOrRest;
 class NedClipBoard;
 class NedCommandList;
+class NedLyricsEditor;
 struct VoiceInfoContainer;
 class NedMeasure;
 class NedSlur;
@@ -46,7 +47,7 @@ class NedStaff {
 		~NedStaff();
 		NedStaff *clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedSystem *p_system, int staff_number);
 		void adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist);
-		void draw(cairo_t *cr, double botpos, double botpos2, bool first_page, double indent, bool *freetexts_or_lyrics_present);
+		void draw(cairo_t *cr, double botpos, bool first_page, double indent, bool *freetexts_or_lyrics_present);
 		void drawTexts(cairo_t *cr, bool first_page, double scale);
 		NedMainWindow *getMainWindow();
 		NedPage *getPage();
@@ -56,13 +57,14 @@ class NedStaff {
 		void setStaffNumber(int nr) {m_staff_number = nr;}
 		void prepareForImport();
 		void appendElementsOfMeasureLength(part *part_ptr, unsigned int meas_duration, int meas_num);
+		void copyDataOfWholeStaff();
 		void removeLastImported();
 		bool handleImportedTuplets();
 		void handleStaffElements();
 		//bool find_new_cursor_pos(guint keyval, int *x, int *y, double *dist);
 		bool trySelect (double x, double y, bool only_free_placeables);
 		NedChordOrRest *findNearestElement(double x, double y, NedStaff *oldstaff, double *mindist, double *ydist);
-		bool tryInsertOrErease (double x, double y, NedChordOrRest **newObj, bool force_rest);
+		bool tryInsertOrErease (double x, double y, int num_midi_input_notes, int *midi_input_chord/* given if insert from midikeyboard */, NedChordOrRest **newObj, bool force_rest);
 		bool tryErease (double x, double y);
 		bool findLine(double x, double y, double *ypos, int *line, double *bottom);
 		bool findFirstElementeGreaterThan(double x, NedChordOrRest **element);
@@ -79,15 +81,16 @@ class NedStaff {
 		void cutEmptyVoices();
 		void handleEmptyMeasures();
 		void zoomFreeReplaceables(double zoom, double scale);
+		void recomputeFreeReplaceables();
 		void computeBeams();
-		void computeTuplets();
+		void computeTuplets(bool has_repeat_lines);
 		void computeTies();
 		unsigned int getStaffDurationTotal();
 		void assignMidiTimes();
 		double computeMidDist(double y);
 		void collectChordsAndRests(NedClipBoard *board, unsigned long long midi_start, unsigned long long midi_end);
 		void collectDestinationVoices(NedClipBoard *board);
-		int assignElementsToMeasures(NedMeasure *measures, int staff_offs, bool use_upbeat);
+		int assignElementsToMeasures(NedMeasure *measures, int staff_offs, bool use_upbeat, int *special_measure_at_end, bool out = false);
 		bool tryChangeLength(NedChordOrRest *chord_or_rest);
 		void searchForBeamGroups(unsigned int midi_start_time);
 		double getTopYBorder() {return m_top_y_border;}
@@ -98,10 +101,13 @@ class NedStaff {
 		void saveStaff(FILE *fp);
 		void restoreStaff(FILE *fp);
 		int computeVolume(unsigned long long midi_time);
+		void prepareMutedReplay();
+		void resetActiveFlags();
 		void prepareReplay(bool with_keysig);
 		void findAccidentals(char offs_array[115], NedMeasure *meas_info, unsigned long long midi_time, bool including);
 		void setInternalPitches();
 		void adjustAccidentals();
+		void changeAccidentals(NedCommandList *command_list, int preferred_offs, GList *selected_group);
 		void transpose(int pitchdist, GList *selected_group);
 		void hideRests(NedCommandList *command_list, bool unhide, int voice_nr);
 		bool shiftNotes(unsigned long long start_time, int linedist);
@@ -139,10 +145,15 @@ class NedStaff {
 		void determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found);
 		bool detectVoices(unsigned int *voice_mask, unsigned long long *e_time);
 		void exportLilyPond(FILE *fp, int voice_nr, int *last_line, unsigned int *midi_len,
-				bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames);
-		void exportLilyChordnames(FILE *fp, int *last_line, unsigned int *midi_len,
+				bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break,
+					 bool *guitar_chordnames, bool *chordnames, int *breath_script, bool keep_beams);
+		void exportLilyGuitarChordnames(FILE *fp, int *last_line, unsigned int *midi_len,
+				bool last_system, unsigned long long end_time, bool *in_alternative, bool with_break);
+		void exportLilyFreeChordName(FILE *fp, int *last_line, unsigned int *midi_len,
 				bool last_system, unsigned long long end_time, bool *in_alternative, bool with_break);
 		void exportLilyLyrics(FILE *fp, bool last_system, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count);
+		void collectLyrics(NedLyricsEditor *leditor);
+		void setLyrics(NedCommandList *command_list, NedLyricsEditor *leditor);
 		GList *getStaffElements(unsigned long long midi_time);
 		static const char m_sharpPos[7][7];
 		static const char m_flatPos[7][7];
