@@ -36,9 +36,9 @@
 #include "appendstaffcommand.h"
 #include "mainwindow.h"
 #include "musicxmlimport.h"
+#include "pangocairotext.h"
 
 #define DEFAULT_SYSTEM_DIST (4 * LINE_DIST)
-#define STAFF_START_LEFT_THICK 0.08
 
 #define X_POS_PAGE_REL(p) ((getPage()->getContentXpos() + (p)) * zoom_factor - leftx)
 
@@ -52,8 +52,8 @@
 
 NedSystem::NedSystem(NedPage *page, double ypos, double width, int nr, unsigned int start_measure_number, bool start) :
 m_system_start(CLEF_SPACE), m_is_positioned(false), m_staffs(NULL), m_deleted_staffs(NULL),
-m_ypos(ypos), m_width(width), m_system_number(nr),
-m_page(page), m_measure_count(0), m_only_whole_elements(FALSE), m_endMeasure(NULL) {
+m_ypos(ypos), m_width(width), m_system_number(nr), m_has_repeat_lines(false),
+m_page(page), m_measure_count(0), m_extra_space(0.0), m_only_whole_elements(FALSE), m_endMeasure(NULL), m_special_measure_at_end(0) {
 	int i;
 	double staff_pos = 0;
 	NedStaff *staff;
@@ -128,6 +128,7 @@ NedStaff *NedSystem::getStaff(int nr) {
 	return (NedStaff *) lptr->data;
 }
 
+
 void NedSystem::fill_up(NedCommandList *command_list) {
 	GList *lptr;
 	do {
@@ -209,9 +210,8 @@ void NedSystem::draw(cairo_t *cr, bool first_page, bool show_measure_numbers, bo
 			botp = botp1;
 		}
 
-		((NedStaff *) lptr1->data)->draw(cr, botp, botp2, first_page, indent, &m_freetexts_or_lyrics_present);
+		((NedStaff *) lptr1->data)->draw(cr, botp,  first_page, indent, &m_freetexts_or_lyrics_present);
 	}
-	if (m_freetexts_or_lyrics_present) *freetexts_or_lyrics_present = true;
 	firststaff = (NedStaff *) g_list_first(m_staffs)->data;
 	laststaff = (NedStaff *) g_list_last(m_staffs)->data;
 	if (show_measure_numbers && (m_system_number != 0 || !first_page)) {
@@ -230,15 +230,23 @@ void NedSystem::draw(cairo_t *cr, bool first_page, bool show_measure_numbers, bo
 	}
 	cairo_new_path(cr);
 	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+#ifdef BADLINEPOLICY
 	cairo_set_line_width(cr, zoom_factor * STAFF_START_LEFT_THICK);
+#else
+	cairo_set_line_width(cr, zoom_factor * MEASURE_LINE_THICK);
+#endif
 	cairo_move_to(cr, X_POS_PAGE_REL(indent),
 		 Y_POS(m_ypos + firststaff->getTopPos() - LINE_THICK / 2.0));
 	cairo_line_to(cr,  X_POS_PAGE_REL(indent),
 		 Y_POS(m_ypos + laststaff->getBottomPos() + LINE_THICK / 2.0));
-	cairo_move_to(cr, X_POS_PAGE_REL(m_width),
-		 Y_POS(m_ypos + firststaff->getTopPos() - LINE_THICK / 2.0));
-	cairo_line_to(cr,  X_POS_PAGE_REL(m_width),
-		 Y_POS(m_ypos + laststaff->getBottomPos() + LINE_THICK / 2.0));
+#ifdef BADLINEPOLICY
+	if (!m_special_measure_at_end) { // i.e there is a key signature at the end
+			cairo_move_to(cr, X_POS_PAGE_REL(m_width),
+		 	Y_POS(m_ypos + firststaff->getTopPos() - LINE_THICK / 2.0));
+		cairo_line_to(cr,  X_POS_PAGE_REL(m_width),
+		 	Y_POS(m_ypos + laststaff->getBottomPos() + LINE_THICK / 2.0));
+	}
+#endif
 	cairo_stroke(cr);
 //#define SHOW_MEASURES_END
 #ifdef SHOW_MEASURES_END
@@ -264,6 +272,14 @@ void NedSystem::draw(cairo_t *cr, bool first_page, bool show_measure_numbers, bo
 		if ((getMainWindow()->m_staff_contexts[i].m_flags & BRACE_END) != 0) {
 			if (brace_start_y_pos >= 0.0) {
 				drawBrace(cr, leftx, indent - (inside_bracket ? 2 * NESTED_BRACE_INDENT : BRACE_SPACE) , topy, zoom_factor, staff->getBottomPos() - brace_start_y_pos, brace_start_y_pos);
+				if (first_page) {
+					if (getMainWindow()->m_staff_contexts[i].m_group_name != NULL) {
+						m_freetexts_or_lyrics_present = true;
+					}
+				}
+				else if (getMainWindow()->m_staff_contexts[i].m_group_short_name != NULL) {
+						m_freetexts_or_lyrics_present = true;
+				}
 				brace_start_y_pos = -1.0;
 			}
 		}
@@ -274,11 +290,20 @@ void NedSystem::draw(cairo_t *cr, bool first_page, bool show_measure_numbers, bo
 		if ((getMainWindow()->m_staff_contexts[i].m_flags & BRACKET_END) != 0) {
 			if (bracket_start_y_pos >= 0.0) {
 				drawBracket(cr, leftx, indent, topy, zoom_factor, bracket_start_y_pos, staff->getBottomPos());
+				if (first_page) {
+					if (getMainWindow()->m_staff_contexts[i].m_group_name != NULL) {
+						m_freetexts_or_lyrics_present = true;
+					}
+				}
+				else if (getMainWindow()->m_staff_contexts[i].m_group_short_name != NULL) {
+						m_freetexts_or_lyrics_present = true;
+				}
 				bracket_start_y_pos = -1.0;
 			}
 			inside_bracket = false;
 		}
 	}
+	if (m_freetexts_or_lyrics_present) *freetexts_or_lyrics_present = true;
 	if (m_endMeasure != NULL) {
 		cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 		cairo_rectangle (cr,  X_POS_PAGE_REL(m_endMeasure->start + REP_THICK_LINE_DIST / 3.0), Y_POS(m_ypos + firststaff->getTopPos() + firststaff->getTopYBorder() - 2 * REP_THICK_LINE_DIST ),
@@ -291,11 +316,67 @@ void NedSystem::draw(cairo_t *cr, bool first_page, bool show_measure_numbers, bo
 };
 
 void NedSystem::drawTexts(cairo_t *cr, bool first_page, double scale) {
-	GList *lptr;
+	double leftx = getMainWindow()->getLeftX();
+	double topy = getMainWindow()->getTopY();
+	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
+	NedStaff *staff;
+	double brace_start_y_pos = -1.0;
+	double bracket_start_y_pos = -1.0;
+	int i;
+	bool group_written;
 
 	if (!m_freetexts_or_lyrics_present) return;
-	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
-		((NedStaff *) lptr->data)->drawTexts(cr, first_page, scale);
+	for (i = 0; i < getMainWindow()->getStaffCount(); i++) {
+		staff = (NedStaff *) g_list_nth(m_staffs, i)->data;
+		if ((getMainWindow()->m_staff_contexts[i].m_flags & BRACE_START) != 0) {
+			brace_start_y_pos = staff->getTopPos();
+		}
+		group_written = false;
+		if ((getMainWindow()->m_staff_contexts[i].m_flags & BRACE_END) != 0) {
+			if (brace_start_y_pos >= 0.0) {
+				if (first_page) {
+					if (getMainWindow()->m_staff_contexts[i].m_group_name != NULL) {
+					group_written = true;
+					getMainWindow()->m_staff_contexts[i].m_group_name->draw(cr,
+					((getPage()->getContentXpos()) * zoom_factor - leftx - BRACE_X_OFFSET) * scale +
+					(getMainWindow()->getNettoIndent() - getMainWindow()->m_staff_contexts[i].m_group_name->getInternalWidth()),
+		 			Y_POS(m_ypos + brace_start_y_pos + (staff->getTopPos() + 4 * LINE_DIST - brace_start_y_pos - LINE_DIST) / 2.0) * scale, zoom_factor, scale);
+					}
+				}
+				else if (getMainWindow()->m_staff_contexts[i].m_group_short_name != NULL) {
+					group_written = true;
+					getMainWindow()->m_staff_contexts[i].m_group_short_name->draw(cr,
+					((getPage()->getContentXpos()) * zoom_factor  - leftx - BRACE_X_OFFSET) * scale +
+					(getMainWindow()->get2ndNettoIndent() - getMainWindow()->m_staff_contexts[i].m_group_short_name->getInternalWidth()),
+		 			Y_POS(m_ypos + brace_start_y_pos + (staff->getTopPos() + 4 * LINE_DIST - brace_start_y_pos - LINE_DIST) / 2.0)  * scale, zoom_factor, scale);
+				}
+			}
+		}
+		if ((getMainWindow()->m_staff_contexts[i].m_flags & BRACKET_START) != 0) {
+			bracket_start_y_pos = staff->getTopPos();
+		}
+		if ((getMainWindow()->m_staff_contexts[i].m_flags & BRACKET_END) != 0) {
+			if (bracket_start_y_pos >= 0.0) {
+				if (!group_written) {
+					if (first_page) {
+						if (getMainWindow()->m_staff_contexts[i].m_group_name != NULL) {
+						getMainWindow()->m_staff_contexts[i].m_group_name->draw(cr,
+						((getPage()->getContentXpos()) * zoom_factor - leftx) * scale +
+						(getMainWindow()->getNettoIndent() - getMainWindow()->m_staff_contexts[i].m_group_name->getInternalWidth()),
+		 				Y_POS(m_ypos + bracket_start_y_pos + (staff->getTopPos() + 4 * LINE_DIST - bracket_start_y_pos - LINE_DIST) / 2.0)  * scale, zoom_factor, scale);
+						}
+					}
+					else if (getMainWindow()->m_staff_contexts[i].m_group_short_name != NULL) {
+						getMainWindow()->m_staff_contexts[i].m_group_short_name->draw(cr,
+						((getPage()->getContentXpos()) * zoom_factor  - leftx) * scale +
+						(getMainWindow()->get2ndNettoIndent() - getMainWindow()->m_staff_contexts[i].m_group_short_name->getInternalWidth()),
+		 				Y_POS(m_ypos + bracket_start_y_pos + (staff->getTopPos() + 4 * LINE_DIST - bracket_start_y_pos - LINE_DIST) / 2.0)  * scale, zoom_factor, scale);
+					}
+					bracket_start_y_pos = -1.0;
+				}
+			}
+		}
+		staff->drawTexts(cr, first_page, scale);
 	}
 };
 
@@ -426,7 +507,7 @@ double NedSystem::computeMidDist(double y) {
 bool NedSystem::testLineOverflow() {
 	compute_extra_space(false);
 	//do_reposit(false);
-	return m_extra_space < MINIMUM_EXTRA_SPACE;
+	return m_extra_space < MINIMUM_EXTRA_SPACE && m_measure_count > 1;
 }
 
 void NedSystem::removeLastImported() {
@@ -477,6 +558,16 @@ void NedSystem::appendElementsOfMeasureLength(part *parts, unsigned int meas_dur
 	}
 }
 
+void NedSystem::copyDataOfWholeStaff(int staff_nr) {
+	GList *lptr;
+
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::copyDataOfWholeStaff");
+	}
+
+	((NedStaff *) lptr->data)->copyDataOfWholeStaff();
+}
+
 void NedSystem::handleStaffElements() {
 	GList *lptr;
 
@@ -501,6 +592,16 @@ void NedSystem::adjustAccidentals() {
 	}
 }
 
+void NedSystem::changeAccidentals(NedCommandList *command_list, int preferred_offs, bool *staff_list, GList *selected_group) {
+	GList *lptr;
+	int i;
+
+	for (i = 0, lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr), i++) {
+		if (!staff_list[i]) continue;
+		((NedStaff *) lptr->data)->changeAccidentals(command_list, preferred_offs, selected_group);
+	}
+}
+
 void NedSystem::transpose(int pitchdist, bool *staff_list, GList *selected_group) {
 	GList *lptr;
 	int i;
@@ -544,7 +645,7 @@ void NedSystem::removeUnneededAccidentals() {
 }
 
 
-bool NedSystem::tryInsertOrErease(double x, double y, NedChordOrRest **newObj, bool force_rest) {
+bool NedSystem::tryInsertOrErease(double x, double y, int num_midi_input_notes, int *midi_input_chord /* given if insert from midikeyboard */, NedChordOrRest **newObj, bool force_rest) {
 	GList *lptr;
 	double mindist, d;
 	lptr = g_list_first(m_staffs);
@@ -556,13 +657,12 @@ bool NedSystem::tryInsertOrErease(double x, double y, NedChordOrRest **newObj, b
 			mindist = d;
 		}
 	}
-	if (nearest_staff->tryInsertOrErease(x, y, newObj, force_rest)) {
-		return TRUE;
+	if (nearest_staff->tryInsertOrErease(x, y, num_midi_input_notes, midi_input_chord, newObj, force_rest)) {
+		return true;
 	}
-	return FALSE;
+	return false;
 }
 
-
 bool NedSystem::findLine(double x, double y, double *ypos, int *line, double *bottom, NedStaff **staff) {
 	GList *lptr;
 	double mindist, d;
@@ -706,6 +806,14 @@ void NedSystem::zoomFreeReplaceables(double zoom, double scale) {
 	}
 }
 
+void NedSystem::recomputeFreeReplaceables() {
+	GList *lptr;
+
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		((NedStaff *) lptr->data)->recomputeFreeReplaceables();
+	}
+}
+
 void NedSystem::testTies() {
 	GList *lptr;
 
@@ -792,9 +900,9 @@ bool NedSystem::reposit(int measure_number, NedCommandList *command_list /* = NU
 	unsigned long long midi_time = 0;
 	unsigned int meas_duration;
 	bool use_upbeat = (m_system_number == 0 && m_page->getPageNumber() == 0);
+	int critical_measure;
 	//bool changed = false;
 	//bool out = (m_system_number == 6 && m_page->getPageNumber() == 2);
-
 	
 
 	if (next_system != NULL) *next_system = m_page->getNextSystem(this);
@@ -809,25 +917,36 @@ bool NedSystem::reposit(int measure_number, NedCommandList *command_list /* = NU
 		m_measures[i].midi_start = midi_time;
 		meas_duration = getMainWindow()->getMeasureDuration(measure_number);
 		m_measures[i].setMeasureNumber(measure_number++, getMainWindow()->m_special_measures /* friend !! */, true);
+		if (!m_measures[i].m_special_spread) {
+			m_measures[i].m_spread_fac = getMainWindow()->getGlopabSpreadFac();
+		}
 		midi_time += meas_duration;
 		m_measures[i].midi_end = midi_time;
 	}
 		
 	if (command_list == NULL) {
 		do_reposit(use_upbeat);
-		return FALSE;
+		return false;
 	}
 	compute_extra_space(use_upbeat);
 	//do_reposit(use_upbeat);
-	m_check_line_compression = TRUE;
+	m_check_line_compression = true;
 
-	if (m_extra_space < MINIMUM_EXTRA_SPACE && m_measure_count > 1) {
+	if (m_extra_space < MINIMUM_EXTRA_SPACE) {
+	
+	
+	   if (m_measure_count > 1) {
+	//if (m_extra_space < MINIMUM_EXTRA_SPACE && m_measure_count > 1) {
 		//printf("pag: %d sys: %d: m_extra_space(%f) < MINIMUM_EXTRA_SPACE (%f)reposit\n", m_page->getPageNumber(), m_system_number, m_extra_space, MINIMUM_EXTRA_SPACE); fflush(stdout);
 		m_check_line_compression = FALSE;
+		//critical_measure = m_special_measure_at_end ? m_measure_count-2 : m_measure_count-1;
+		critical_measure = m_measure_count-1;
+		//int xx = 0;
 		NedClipBoard *board = new NedClipBoard();
-		board->setRefTimeAndDuration(m_measures[m_measure_count-1].midi_start, m_measures[m_measure_count-1].midi_end - m_measures[m_measure_count-1].midi_start);
+		if (m_measures[critical_measure].m_measure_has_only_staff_members) critical_measure--;
+		board->setRefTimeAndDuration(m_measures[critical_measure].midi_start, m_measures[critical_measure].midi_end - m_measures[critical_measure].midi_start);
 		for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
-			((NedStaff *) lptr->data)->collectChordsAndRests(board, m_measures[m_measure_count-1].midi_start,  m_measures[m_measure_count-1].midi_end);
+			((NedStaff *) lptr->data)->collectChordsAndRests(board, m_measures[critical_measure].midi_start,  m_measures[critical_measure].midi_end);
 		}
 		if ((board->onlyWholeRests() && m_page->isLastSystem(this))) {
 			board->setDeleteOnly();
@@ -855,11 +974,12 @@ bool NedSystem::reposit(int measure_number, NedCommandList *command_list /* = NU
 		command_list->addCommand(command);
 		command->execute();
 	}
+	}
 	if (m_check_line_compression && m_extra_space > MAXIMUM_EXTRA_SPACE) {
 		//printf("pag: %d sys: %d : m_extra_space(%f) > MAXIMUM_EXTRA_SPACE (%f)reposit\n", m_page->getPageNumber(), m_system_number, m_extra_space, MAXIMUM_EXTRA_SPACE); fflush(stdout);
 		if ((other_system = m_page->getNextSystem(this)) != NULL) {
 			NedClipBoard *board = new NedClipBoard();
-			if (other_system->collectFirstMeasure(board, getSystemEndTime())) {
+			if (other_system->collectFirstMeasure(board, getMainWindow()->getMeasureDuration(m_measures[m_measure_count].getMeasureNumber()), getSystemEndTime())) {
 				collectDestinationVoices(board);
 				if (board->onlyWholeRests() && m_page->isLastSystem(this)) {
 					board->setInsertOnly();
@@ -872,15 +992,17 @@ bool NedSystem::reposit(int measure_number, NedCommandList *command_list /* = NU
 					compute_extra_space(use_upbeat);
 					//do_reposit(use_upbeat);
 					other_system->do_reposit(false); // the ping-pong elements have bad x position
-					m_check_line_compression = FALSE;
+					m_check_line_compression = false;
 				}
 				else {
+					//board->unexecute_reverse();
 					NedMoveChordsAndRestsReverseCommand *command = new NedMoveChordsAndRestsReverseCommand(board);
 					for (lptr = g_list_first(other_system->m_staffs); lptr; lptr = g_list_next(lptr)) {
 						((NedStaff *) lptr->data)->assignMidiTimes();
 					}
 					//changed = true;
 					command_list->addCommand(command);
+					//command->execute(); do not execut because is already done
 					command_list->addSystemForRepositioning(other_system);
 					if (board->onlyWholeRests() && m_page->isLastSystem(this)) {
 						board->setInsertOnly();
@@ -942,6 +1064,7 @@ void NedSystem::renumberMeasures(int *measure_number, GList *special_measures, b
 void NedSystem::setNewMeasureSpread(double spread) {
 	for (int i = 0; i < MAX_MEASURES; i++) {
 		m_measures[i].m_spread_fac = spread;
+		m_measures[i].m_special_spread = false;
 	}
 }
 
@@ -970,9 +1093,12 @@ double NedSystem::placeStaffs(double staffpos) {
 	return yy + m_ypos + SYSTEM_MIN_DIST;
 }
 
-bool NedSystem::collectFirstMeasure(NedClipBoard *board, unsigned long long length_of_previous_system) {
+bool NedSystem::collectFirstMeasure(NedClipBoard *board, unsigned long long length_of_first_measure, unsigned long long length_of_previous_system) {
 	GList *lptr;
 	if (m_measure_count < 2) return FALSE;
+	m_measures[0].midi_start = 0;
+	m_measures[0].midi_end = length_of_first_measure; // preliminary times importatnt is the correct duration. Under some circumstances
+	                                                  // collectFirstMeasure is called before the measure got their time
 	board->setRefTimeAndDuration(length_of_previous_system, m_measures[0].midi_end);
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
 		((NedStaff *) lptr->data)->collectChordsAndRests(board, m_measures[0].midi_start,  m_measures[0].midi_end);
@@ -988,10 +1114,11 @@ void NedSystem::collectSelectionRectangleElements(double xp, NedBbox *sel_rect,
 
 	firststaff = (NedStaff *) g_list_first(m_staffs)->data;
 	laststaff = (NedStaff *) g_list_last(m_staffs)->data;
-	double yp = firststaff->getTopPos() + m_ypos;
+	double yp = firststaff->getTopPos() + firststaff->getTopYBorder() + m_ypos;
 	if (is_last_selected_page && sel_rect->y + sel_rect->height < yp) return;
-	if (is_first_selected_page && yp + laststaff->getBottomPos() - firststaff->getTopPos() < sel_rect->y) return;
-	yp = sel_rect->y - (firststaff->getTopPos() + m_ypos);
+
+	if (is_first_selected_page && m_ypos + laststaff->getBottomPos() < sel_rect->y) return;
+	yp = sel_rect->y - m_ypos;
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
 		((NedStaff *) lptr->data)->collectSelectionRectangleElements(xp, yp, sel_rect, sel_group,
 		is_first_selected_page && (first_selected_system == this), is_last_selected_page && (last_selected_system == this));
@@ -1212,7 +1339,8 @@ void NedSystem::do_reposit(bool use_upbeat) {
 	int new_measure_count;
 	int grace_positions[MAX_POSITIONS];
 	double extra_space_for_accidentals;
-	new_measure_count = 0;
+	bool measure_has_only_staff_members;
+	m_special_measure_at_end = new_measure_count = 0;
 	if (m_measures[0].getStoredStaffCount() != g_list_length(m_staffs)) {
 		for (i = 0; i < MAX_MEASURES; i++) {
 			m_measures[i].m_position_array.setNewStaffCount(g_list_length(m_staffs));
@@ -1220,6 +1348,7 @@ void NedSystem::do_reposit(bool use_upbeat) {
 	}
 	for (i = 0; i < MAX_MEASURES; i++) {
 		m_measures[i].m_position_array.empty();
+		m_measures[i].m_staff_elements_width = 0.0;
 	}
 	current_position = m_system_start - m_measures[0].getNeededSpaceAfter();
 	if (m_measures[0].getSpecial() == REPEAT_OPEN || m_measures[0].getSpecial() == REPEAT_OPEN_CLOSE) {
@@ -1231,42 +1360,56 @@ void NedSystem::do_reposit(bool use_upbeat) {
 	}
 	memset(grace_positions, 0, sizeof(grace_positions));
 	for (i = 0, lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr), i++) {
-		new_measure_count = ((NedStaff *) lptr->data)->assignElementsToMeasures(m_measures, VOICE_COUNT * i, use_upbeat);
+		new_measure_count = ((NedStaff *) lptr->data)->assignElementsToMeasures(m_measures, VOICE_COUNT * i, use_upbeat, &m_special_measure_at_end, m_system_number == 0);
+		new_measure_count++;
+		m_measure_count = new_measure_count;
 	}
-	m_measure_count = new_measure_count + 1;
-	if (((m_measures[m_measure_count-1].getSpecial() & REP_TYPE_MASK) == REPEAT_CLOSE) || ((m_measures[m_measure_count-1].getSpecial() & REP_TYPE_MASK) == REPEAT_OPEN_CLOSE)) {
-		extra_width_at_end = m_measures[m_measure_count-1].getNeededSpaceBefore();
+	if (m_special_measure_at_end) {
+		m_measures[m_measure_count+1].m_staff_elements_width = 0.2;
+		m_measures[m_measure_count+1].m_position_array.empty();
+	}
+	if (((m_measures[m_measure_count+m_special_measure_at_end].getSpecial() & REP_TYPE_MASK) == REPEAT_CLOSE) || ((m_measures[m_measure_count+m_special_measure_at_end].getSpecial() & REP_TYPE_MASK) == REPEAT_OPEN_CLOSE) ||
+		(m_measures[m_measure_count+m_special_measure_at_end].getSpecial() & REP_TYPE_MASK) == END_BAR) {
+		extra_width_at_end = m_measures[m_measure_count+m_special_measure_at_end].getNeededSpaceBefore();
 	}
 	else {
 		extra_width_at_end = 0.0;
 	}
 	num_elements = 0;
 	extra_space_for_accidentals = 0.0;
-	current_position = m_measures[0].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals,
-		false);
+	current_position = m_measures[0].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals,  &measure_has_only_staff_members, false);
+	if (m_measures[0].getSpecial() & (START_TYPE_MASK | END_TYPE_MASK)) {
+	}
 	m_endMeasure = NULL;
-	for (i = 1; i < m_measure_count; i++) {
-		current_position = m_measures[i].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals,
-			i == 2);
+	m_has_repeat_lines = false;
+	for (i = 1; i < m_measure_count+m_special_measure_at_end; i++) {
+		measure_has_only_staff_members = true;
+		current_position = m_measures[i].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals, &measure_has_only_staff_members, false);
 			if (m_measures[i].barRequiresHideFollowing()) {
 				m_endMeasure = &(m_measures[i]);
 			}
 			m_measures[i-1].end = m_measures[i].start;
+			if (m_measures[i].getSpecial() & (START_TYPE_MASK | END_TYPE_MASK)) {
+				m_has_repeat_lines = true;
+			}
 	}
 	m_measures[i-1].end = m_width - RIGHT_DIST;
-	num_elements += m_measure_count;
-	m_extra_space = m_width - current_position - extra_space_for_accidentals - m_measures[m_measure_count-1].getNeededSpaceBefore() - RIGHT_DIST;
+	if (m_measures[i].getSpecial() & (START_TYPE_MASK | END_TYPE_MASK)) {
+		m_has_repeat_lines = true;
+	}
+	num_elements += m_measure_count+m_special_measure_at_end;
+	m_extra_space = m_width - current_position - extra_space_for_accidentals - m_measures[m_measure_count+m_special_measure_at_end+1].getNeededSpaceBefore() - RIGHT_DIST - extra_width_at_end;
 	/*
 	if (m_page->getPageNumber() == 0 && m_system_number == 0) {
 		printf("m_extra_space(%f) = m_width(%f) - current_position(%f) - extra_space_for_accidentals(%f) - nb(%f) -  RIGHT_DIST(%f)\n",
 				m_extra_space, m_width, current_position, extra_space_for_accidentals, 
-					m_measures[m_measure_count-1].getNeededSpaceBefore(), RIGHT_DIST); fflush(stdout);
+					m_measures[m_measure_count+m_special_measure_at_end+1].getNeededSpaceBefore(), RIGHT_DIST); fflush(stdout);
 	}
 	*/
 	extra_space_divided = m_extra_space / (num_elements + 2);
 	shift = 0;
 	if (extra_space_divided > 0.0) {
-		for (i = 0; i < m_measure_count; i++) {
+		for (i = 0; i < m_measure_count+m_special_measure_at_end+1; i++) {
 #define BBB
 #ifdef BBB
 			shift = m_measures[i].shiftXpos(extra_space_divided, shift, m_page->getPageNumber() == 0 && m_system_number == 0);
@@ -1276,13 +1419,43 @@ void NedSystem::do_reposit(bool use_upbeat) {
 			}
 		}
 	}
-	m_measures[i-1].end = m_measures[i].start = m_width - RIGHT_DIST;
+	m_measures[i-2].end = m_measures[i-1].start = m_width - RIGHT_DIST;
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
 		((NedStaff *) lptr->data)->searchForBeamGroups(use_upbeat ? getMainWindow()->getUpBeatInverse() : 0);
 		((NedStaff *) lptr->data)->computeBeams();
-		((NedStaff *) lptr->data)->computeTuplets();
+		((NedStaff *) lptr->data)->computeTuplets(m_has_repeat_lines);
 		((NedStaff *) lptr->data)->computeTies();
 	}
+	if (m_special_measure_at_end) { /* means last measure consists of only a key signature */
+		/*
+		m_measures[i-2].start -= 0.15;
+		m_measures[i-1].end = m_measures[i-2].start;
+		*/
+	}
+}
+
+void NedSystem::computeTuplets() {
+	GList *lptr;
+	int i;
+	
+	m_has_repeat_lines = false;
+	for (i = 0; i < m_measure_count; i++) {
+		if (m_measures[i].getSpecial() & (START_TYPE_MASK | END_TYPE_MASK)) {
+			m_has_repeat_lines = true;
+		}
+	}
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		((NedStaff *) lptr->data)->computeBeams();
+		((NedStaff *) lptr->data)->computeTuplets(m_has_repeat_lines);
+	}
+}
+
+void NedSystem::resetActiveFlags() {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+		((NedStaff *) lptr->data)->resetActiveFlags();
+	}
 }
 
 void NedSystem::compute_extra_space(bool use_upbeat) {
@@ -1290,12 +1463,11 @@ void NedSystem::compute_extra_space(bool use_upbeat) {
 	int num_elements;
 	int i;
 	double current_position;
-	double shift;
-	double extra_space_divided;
 	double extra_width_at_end;
 	int new_measure_count;
 	int grace_positions[MAX_POSITIONS];
 	double extra_space_for_accidentals;
+	bool measure_has_only_staff_members;
 	new_measure_count = 0;
 	if (m_measures[0].getStoredStaffCount() != g_list_length(m_staffs)) {
 		for (i = 0; i < MAX_MEASURES; i++) {
@@ -1304,6 +1476,7 @@ void NedSystem::compute_extra_space(bool use_upbeat) {
 	}
 	for (i = 0; i < MAX_MEASURES; i++) {
 		m_measures[i].m_position_array.empty();
+		m_measures[i].m_staff_elements_width = 0.0;
 	}
 	current_position = m_system_start - m_measures[0].getNeededSpaceAfter();
 	if (m_measures[0].getSpecial() == REPEAT_OPEN || m_measures[0].getSpecial() == REPEAT_OPEN_CLOSE) {
@@ -1315,43 +1488,49 @@ void NedSystem::compute_extra_space(bool use_upbeat) {
 	}
 	memset(grace_positions, 0, sizeof(grace_positions));
 	for (i = 0, lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr), i++) {
-		new_measure_count = ((NedStaff *) lptr->data)->assignElementsToMeasures(m_measures, VOICE_COUNT * i, use_upbeat);
+		new_measure_count = ((NedStaff *) lptr->data)->assignElementsToMeasures(m_measures, VOICE_COUNT * i, use_upbeat, &m_special_measure_at_end, m_system_number == 0);
+		new_measure_count++;
+		m_measure_count = new_measure_count;
 	}
-	m_measure_count = new_measure_count + 1;
-	if (((m_measures[m_measure_count-1].getSpecial() & REP_TYPE_MASK) == REPEAT_CLOSE) || ((m_measures[m_measure_count-1].getSpecial() & REP_TYPE_MASK) == REPEAT_OPEN_CLOSE)) {
-		extra_width_at_end = m_measures[m_measure_count-1].getNeededSpaceBefore();
+	if (m_special_measure_at_end) {
+		m_measures[m_measure_count+1].m_staff_elements_width = 0.2;
+		m_measures[m_measure_count+1].m_position_array.empty();
+	}
+	if (((m_measures[m_measure_count+m_special_measure_at_end].getSpecial() & REP_TYPE_MASK) == REPEAT_CLOSE) || ((m_measures[m_measure_count+m_special_measure_at_end].getSpecial() & REP_TYPE_MASK) == REPEAT_OPEN_CLOSE) ||
+		(m_measures[m_measure_count+m_special_measure_at_end].getSpecial() & REP_TYPE_MASK) == END_BAR) {
+		extra_width_at_end = m_measures[m_measure_count+m_special_measure_at_end].getNeededSpaceBefore();
 	}
 	else {
 		extra_width_at_end = 0.0;
 	}
 	num_elements = 0;
 	extra_space_for_accidentals = 0.0;
-	current_position = m_measures[0].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals,
-		m_page->getPageNumber() == 0 && m_system_number == 0);
-	for (i = 1; i < m_measure_count; i++) {
-		if (m_measure_count >= MAX_MEASURES) {
-			NedResource::Abort("Too many measures");
-		}
-		current_position = m_measures[i].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals,
-			m_page->getPageNumber() == 0 && m_system_number == 0);
+	current_position = m_measures[0].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals,  &measure_has_only_staff_members, false);
+	m_endMeasure = NULL;
+	for (i = 1; i < m_measure_count+m_special_measure_at_end; i++) {
+		measure_has_only_staff_members = true;
+		current_position = m_measures[i].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals, &measure_has_only_staff_members, false);
+			if (m_measures[i].barRequiresHideFollowing()) {
+				m_endMeasure = &(m_measures[i]);
+			}
 			m_measures[i-1].end = m_measures[i].start;
 	}
 	m_measures[i-1].end = m_width - RIGHT_DIST;
-	num_elements += m_measure_count;
-	m_extra_space = m_width - current_position - extra_space_for_accidentals - m_measures[m_measure_count-1].getNeededSpaceBefore() - RIGHT_DIST;
-	extra_space_divided = m_extra_space / (num_elements + 2);
-	shift = 0;
-	if (extra_space_divided > 0.0) {
-		for (i = 0; i < m_measure_count; i++) {
-#ifdef BBB
-			shift = m_measures[i].shiftXpos(extra_space_divided, shift, m_page->getPageNumber() == 9 && m_system_number == 0);
-#endif
-			if (i > 0) {
-				m_measures[i-1].end = m_measures[i].start;
-			}
-		}
+	num_elements += m_measure_count+m_special_measure_at_end;
+	m_extra_space = m_width - current_position - extra_space_for_accidentals - m_measures[m_measure_count+m_special_measure_at_end+1].getNeededSpaceBefore() - RIGHT_DIST - extra_width_at_end;
+	/*
+	if (m_page->getPageNumber() == 0 && m_system_number == 0) {
+		printf("m_extra_space(%f) = m_width(%f) - current_position(%f) - extra_space_for_accidentals(%f) - nb(%f) -  RIGHT_DIST(%f)\n",
+				m_extra_space, m_width, current_position, extra_space_for_accidentals, 
+					m_measures[m_measure_count+1].getNeededSpaceBefore(), RIGHT_DIST); fflush(stdout);
 	}
-	m_measures[i-1].end = m_measures[i].start = m_width - RIGHT_DIST;
+	*/
+	//if (measure_has_only_staff_members) { /* means last measure consists of only a key signature */
+		/*
+		m_measures[i-2].start -=  0.15;
+		m_measures[i].end = m_measures[i-2].start;
+		*/
+//	}
 }
 
 unsigned int NedSystem::getWholeMidiLength() {
@@ -1366,10 +1545,10 @@ NedMeasure *NedSystem::getMeasure(unsigned long long  midi_time) {
 			return (&(m_measures[i]));
 		}
 	}
-#ifdef AAA
-	NedResource::DbgMsg(DBG_TESTING, "pag: %d, sys. %d, midi_time = %u , m_measures[m_measure_count(%d)].midi_start = %llu, m_measures[m_measure_count(%d)].midi_end = %llu, NOTE_4 = %u\n",
+#ifdef VVV
+	NedResource::DbgMsg(DBG_TESTING, "pag: %d, sys. %d, midi_time = %llu(%llu, %llu) , m_measures[m_measure_count(%d)].midi_start = %llu(%llu,%llu), m_measures[m_measure_count(%d)].midi_end = %llu(%llu, %llu)\n",
 	m_page->getPageNumber(), m_system_number,
-	midi_time, m_measure_count, m_measures[m_measure_count].midi_start, m_measure_count, m_measures[m_measure_count].midi_end, NOTE_4); 
+	LAUS(midi_time), m_measure_count, m_measures[m_measure_count].midi_start, m_measure_count, m_measures[m_measure_count].midi_end, NOTE_4); 
 	if (m_measure_count >= 0) {
 		NedResource::DbgMsg(DBG_TESTING, "m_measures[m_measure_count - 1(%d)].midi_start = %llu,  m_measures[m_measure_count - 1(%d)].midi_end = %llu\n",
 		m_measure_count-1, m_measures[m_measure_count-1].midi_start, m_measure_count-1, m_measures[m_measure_count-1].midi_end); 
@@ -1388,8 +1567,12 @@ void NedSystem::prepareReplay(bool with_keysig) {
 	GList *lptr;
 
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
-		if (getMainWindow()->m_staff_contexts[((NedStaff *) lptr->data)->getStaffNumber()].m_muted) continue;
-		((NedStaff *) lptr->data)->prepareReplay(with_keysig);
+		if (getMainWindow()->m_staff_contexts[((NedStaff *) lptr->data)->getStaffNumber()].m_muted){
+			((NedStaff *) lptr->data)->prepareMutedReplay();
+		}
+		else {
+			((NedStaff *) lptr->data)->prepareReplay(with_keysig);
+		}
 	}
 }
 
@@ -1528,23 +1711,35 @@ void NedSystem::detectVoices(int staff_nr, unsigned int *voice_mask, NedSystem *
 }
 
 void NedSystem::exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
-		NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames) {
+		NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur,
+			unsigned int *lyrics_map, bool with_break, bool *guitar_chordnames, bool *chordnames, int *breath_script, bool keep_beams) {
 	GList *lptr;
 
 	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
 		NedResource::Abort("NedSystem::exportLilyPond");
 	}
-	((NedStaff *) lptr->data)->exportLilyPond(fp, voice_nr, last_line, midi_len, last_system == this, end_time, in_alternative, lily_slur, lyrics_map, with_break, chordnames);
+	((NedStaff *) lptr->data)->exportLilyPond(fp, voice_nr, last_line, midi_len, last_system == this, end_time, in_alternative,
+		lily_slur, lyrics_map, with_break, guitar_chordnames, chordnames, breath_script, keep_beams);
 }
 
-void NedSystem::exportLilyChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+void NedSystem::exportLilyGuitarChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
 		NedSystem *last_system, unsigned long long end_time, bool *in_alternative, bool with_break) {
 	GList *lptr;
 
 	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
-		NedResource::Abort("NedSystem::exportLilyChordnames");
+		NedResource::Abort("NedSystem::exportLilyGuitarChordnames");
 	}
-	((NedStaff *) lptr->data)->exportLilyChordnames(fp, last_line, midi_len, last_system == this, end_time, in_alternative, with_break);
+	((NedStaff *) lptr->data)->exportLilyGuitarChordnames(fp, last_line, midi_len, last_system == this, end_time, in_alternative, with_break);
+}
+
+void NedSystem::exportLilyFreeChordName(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+		NedSystem *last_system, unsigned long long end_time, bool *in_alternative, bool with_break) {
+	GList *lptr;
+
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::exportLilyFreeChordName");
+	}
+	((NedStaff *) lptr->data)->exportLilyFreeChordName(fp, last_line, midi_len, last_system == this, end_time, in_alternative, with_break);
 }
 
 void NedSystem::exportLilyLyrics(FILE *fp, bool last_system, int staff_nr, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count) {
@@ -1555,3 +1750,23 @@ void NedSystem::exportLilyLyrics(FILE *fp, bool last_system, int staff_nr, int v
 	}
 	((NedStaff *) lptr->data)->exportLilyLyrics(fp, last_system, voice_nr, line_nr, end_time, sil_count);
 }
+
+void NedSystem::collectLyrics(NedLyricsEditor *leditor, int staff_nr) {
+	GList *lptr;
+
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::collectLyrics");
+	}
+
+	((NedStaff *) lptr->data)->collectLyrics(leditor);
+}
+
+void NedSystem::setLyrics(NedCommandList *command_list, NedLyricsEditor *leditor, int staff_nr) {
+	GList *lptr;
+
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::setLyrics");
+	}
+
+	((NedStaff *) lptr->data)->setLyrics(command_list, leditor);
+}
