@@ -34,12 +34,13 @@ class NedStaff;
 class NedClipBoard;
 class NedCommandList;
 class NedChordOrRest;
+class NedLyricsEditor;
 class NedBbox;
 class NedSlur;
 class part;
 
 #define MAX_POSITIONS 4096
-#define MAX_MEASURES 80
+#define MAX_MEASURES 160
 //#define WITH_WEIGHTS
 
 
@@ -55,8 +56,10 @@ class NedSystem {
 		NedPage *getPage() {return m_page;}
 		void assignMidiTimes();
 		void prepareForImport();
+		void setInternalPitches();
 		bool testLineOverflow();
 		void appendElementsOfMeasureLength(part *parts, unsigned int meas_duration, int meas_num);
+		void copyDataOfWholeStaff(int staff_nr);
 		void removeLastImported();
 		bool handleImportedTuplets();
 		void handleStaffElements();
@@ -64,12 +67,13 @@ class NedSystem {
 		void hideRests(NedCommandList *command_list, bool unhide, int staff_nr, int voice_nr);
 		bool shiftNotes(unsigned long long start_time, int linedist, int staff_number);
 		void adjustAccidentals();
-		void setInternalPitches();
+		void changeAccidentals(NedCommandList *command_list, int preferred_offs, bool *staff_list, GList *selected_group);
+		void testBeams(char *s);
 		void removeUnneededAccidentals();
 		bool trySelect (double x, double y, bool only_free_placeables);
 		NedChordOrRest *findNearestElement(int staff_nr, double x, double y, NedStaff *oldstaff, double *mindist, double *ydist);
 		NedChordOrRest *getFirstObjectInNextSystem(int staff_nr);
-		bool tryInsertOrErease (double x, double y, NedChordOrRest **newObj, bool force_rest);
+		bool tryInsertOrErease (double x, double y, int num_midi_input_notes, int *midi_input_chord /* given if insert from midikeyboard */, NedChordOrRest **newObj, bool force_rest);
 		bool findLine(double x, double y, double *ypos, int *line, double *bottom, NedStaff **staff);
 		bool findElement(guint keyval, double x, double y, NedChordOrRest **element, NedStaff **staff);
 		double getHeight();
@@ -83,12 +87,15 @@ class NedSystem {
 		void cutEmptyVoices();
 		void handleEmptyMeasures();
 		void zoomFreeReplaceables(double zoom, double scale);
+		void recomputeFreeReplaceables();
 		void testTies();
 		void removeLastStaff();
 		void deleteStaff(int staff_number);
 		void restoreStaff(int staff_number);
 		void shiftStaff(int staff_number, unsigned int position);
 		bool reposit(int measure_number, NedCommandList *command_list = NULL, NedSystem **next_system  = NULL);
+		void computeTuplets();
+		void resetActiveFlags();
 		void renumberMeasures(int *measure_number, GList *special_measures, bool force);
 		void setNewMeasureSpread(double spread);
 		int getNumberOfFirstMeasure();
@@ -96,7 +103,7 @@ class NedSystem {
 		unsigned long long getSystemEndTime();
 		void checkForElementsToSplit(NedCommandList *command_list, int *measure_number);
 		void collectDestinationVoices(NedClipBoard *board);
-		bool collectFirstMeasure(NedClipBoard *board, unsigned long long length_of_previous_system);
+		bool collectFirstMeasure(NedClipBoard *board, unsigned long long length_of_first_measure, unsigned long long length_of_previous_system);
 		void findSelectedSystems(NedBbox *sel_rect, int *number_of_first_selected_staff, 
 			int *number_of_last_selected_staff, NedSystem **first_selected_system, NedSystem **last_selected_system);
 		void collectSelectionRectangleElements(double xp, NedBbox *sel_rect, GList **sel_group,
@@ -118,6 +125,8 @@ class NedSystem {
 		NedMeasure *getLastMeasure() {return &(m_measures[m_measure_count]);}
 		bool findTimeOfMeasure(int meas_num, unsigned long long *meas_time);
 		int getMeasureCount() {return m_measure_count;}
+		int getFullMeasureCount() {return m_measure_count + m_special_measure_at_end;}
+		int getSpecialMeasuresAtEnd() {return m_special_measure_at_end;}
 		bool tryChangeLength(NedChordOrRest *chord_or_rest);
 		double placeStaffs(double staffpos);
 		NedStaff *findStaff(double x, double y, NedMeasure **measure);
@@ -140,11 +149,16 @@ class NedSystem {
 		void determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found);
 		void detectVoices(int staff_nr, unsigned int *voice_mask, NedSystem **last_system, unsigned long long *end_time);
 		void exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
-			NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames);
-		void exportLilyChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+			NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur,
+				unsigned int *lyrics_map, bool with_break, bool *guitar_chordnames, bool *chordnames, int *breath_script, bool keep_beams);
+		void exportLilyGuitarChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+			NedSystem *last_system, unsigned long long end_time, bool *in_alternative, bool with_break);
+		void exportLilyFreeChordName(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
 			NedSystem *last_system, unsigned long long end_time, bool *in_alternative, bool with_break);
 		void exportLilyLyrics(FILE *fp, bool last_system, int staff_nr, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count);
 		double m_system_start;
+		void collectLyrics(NedLyricsEditor *leditor, int staff_nr);
+		void setLyrics(NedCommandList *command_list, NedLyricsEditor *leditor, int staff_nr);
 		bool m_is_positioned;
 	private:
 		void drawBrace (cairo_t *cr, double leftx, double indent, double topy, double zoom_factor, double height, double toppos);
@@ -158,6 +172,7 @@ class NedSystem {
 		double m_ypos;
 		double m_width;
 		int m_system_number;
+		bool m_has_repeat_lines;
 		NedPage *m_page;
 		NedMeasure m_measures[MAX_MEASURES];
 		int m_measure_count;
@@ -166,6 +181,7 @@ class NedSystem {
 		bool m_only_whole_elements;
 		NedMeasure *m_endMeasure;
 		bool m_freetexts_or_lyrics_present;
+		int m_special_measure_at_end; // last "measure" aith only keysig and/or tinmesig ?
 };
 
 #endif /* SYSTEM_H */
