@@ -30,10 +30,10 @@
 #define X_POS_PAGE_REL(p) ((m_element->getPage()->getContentXpos() + (p)) * zoom_factor - leftx)
 #define Y_POS_PAGE_REL(p) ((p) * zoom_factor - topy)
 
-NedTempoSign::NedTempoSign(unsigned int kind, unsigned int tempo) : NedFreeReplaceable(), m_kind(kind), m_tempo(tempo), m_number_len(0)  {computeGlypths();}
-NedTempoSign::NedTempoSign(unsigned int kind, unsigned int tempo, NedChordOrRest *element) : NedFreeReplaceable(element), m_kind(kind), m_tempo(tempo), m_number_len(0)   {computeGlypths();}
+NedTempoSign::NedTempoSign(unsigned int kind, unsigned int tempo) : NedFreeReplaceable(), m_kind(kind), m_tempo(tempo), m_number_len(0), m_dot(0)  {computeGlypths();}
+NedTempoSign::NedTempoSign(unsigned int kind, unsigned int tempo, NedChordOrRest *element) : NedFreeReplaceable(element), m_kind(kind), m_tempo(tempo), m_number_len(0), m_dot(0)   {computeGlypths();}
 NedTempoSign::NedTempoSign(unsigned int kind, unsigned int tempo, NedChordOrRest *element, double x, double y, bool relative) :
-	NedFreeReplaceable(element, x, y, relative), m_kind(kind), m_tempo(tempo), m_number_len(0)  {computeGlypths();}
+	NedFreeReplaceable(element, x, y, relative), m_kind(kind), m_tempo(tempo), m_number_len(0), m_dot(0)  {computeGlypths();}
 
 
 NedFreeReplaceable *NedTempoSign::clone(struct addr_ref_str **slurlist, NedChordOrRest *newelement, GtkWidget *drawing_area /* = NULL */, NedMainWindow *main_window /* = NULL */) {
@@ -51,10 +51,25 @@ void NedTempoSign::setTempo(unsigned int kind, unsigned int tempo) {
 }
 
 double NedTempoSign::getTempoInverse() {
+	if (m_dot) {
+		return (double) NOTE_4 / (double) (3 * m_kind / 2) * 60000.0 / (double) m_tempo;
+	}
 	return (double) NOTE_4 / (double) m_kind * 60000.0 / (double) m_tempo;
 }
 
 void NedTempoSign::computeGlypths() {
+	switch (m_kind) {
+		case WHOLE_NOTE: m_dot = 0; break;
+		case NOTE_2: m_dot = 0; break;
+		case 3 * NOTE_2 / 2: m_dot = 1; m_kind = NOTE_2; break;
+		case NOTE_4: m_dot = 0; break;
+		case 3 * NOTE_4 / 2: m_dot = 1; m_kind = NOTE_4; break;
+		case NOTE_8: m_dot = 0; break;
+		case 3 * NOTE_8 / 2: m_dot = 1; m_kind = NOTE_8; break;
+		case NOTE_16: m_dot = 0; break;
+		case 3 * NOTE_16 / 2: m_dot = 1; m_kind = NOTE_16; break;
+		default: NedResource::Abort("NedTempoSign::computeGlypths"); break;
+	}
 	if (m_tempo < 10) {
 		m_number_len = 1;
 		m_glyphs[0].index = m_tempo + NUMBERBASE;
@@ -75,14 +90,15 @@ void NedTempoSign::computeGlypths() {
 void NedTempoSign::startContextDialog(GtkWidget *ref) {
 	bool state;
 	unsigned int newtempo, newkind;
+	bool newdot;
 
-	NedTempoDialog *tempo_dialog = new NedTempoDialog(GTK_WINDOW(ref), m_kind, m_tempo);
+	NedTempoDialog *tempo_dialog = new NedTempoDialog(GTK_WINDOW(ref), m_kind, m_dot, m_tempo);
 
-	tempo_dialog->getValues(&state, &newkind, &newtempo);
+	tempo_dialog->getValues(&state, &newkind, &newdot, &newtempo);
 	delete tempo_dialog;
 
 	if (state) {
-		m_kind = newkind;
+		m_kind = newdot ? 3 * newkind / 2 : newkind;
 		m_tempo = newtempo;
 		computeGlypths();
 	}
@@ -96,6 +112,7 @@ void NedTempoSign::draw(cairo_t *cr, double leftx, double topy, double zoom_fact
 	double extra_offs = 0.0;
 	cairo_glyph_t glyph;
 	cairo_matrix_t m;
+	double extra_x_offs = 0.0;
 
 	getXYPos(&xpos, &ypos);
 	
@@ -127,19 +144,28 @@ void NedTempoSign::draw(cairo_t *cr, double leftx, double topy, double zoom_fact
 #define NUMBER_DIST 0.55
 #define DIGIT_DIST 0.3
 #define NUMBER_Y_POS 0.2
+#define TEMPO_DOT_DIST (1.3 * LINE_DIST)
+#define TEMPO_DOT_RAD (0.2 * LINE_DIST)
+#define TEMPO_DOT_EXTRA_X_OFFS (0.4 * LINE_DIST)
+	if (m_dot) {
+		extra_x_offs = TEMPO_DOT_EXTRA_X_OFFS;
+		cairo_new_path(cr);
+		cairo_arc(cr, zoom_factor * TEMPO_DOT_DIST, 0.0, zoom_factor * TEMPO_DOT_RAD, 0.9, 2.0 * M_PI);
+		cairo_fill(cr);
+	}
 	glyph.index = BASE + 5;
 	if (m_kind < NOTE_4) {
-		glyph.x = zoom_factor * (HEAD_THICK_HALF - 0.15);
+		glyph.x = zoom_factor * (extra_x_offs + HEAD_THICK_HALF - 0.15);
 		glyph.y = -zoom_factor * 0.8 * STEM_DEFAULT_HEIGHT;
 		cairo_show_glyphs(cr, &glyph, 1);
 	}
 	if (m_kind < NOTE_8) {
-		glyph.x = zoom_factor * (HEAD_THICK_HALF - 0.15);
+		glyph.x = zoom_factor * (extra_x_offs + HEAD_THICK_HALF - 0.15);
 		glyph.y = -zoom_factor * 0.6 * STEM_DEFAULT_HEIGHT;
 		cairo_show_glyphs(cr, &glyph, 1);
 	}
 	for (i = 0; i < 3;i++) {
-		m_glyphs[i].x = zoom_factor * (NUMBER_DIST + extra_offs + i * DIGIT_DIST);
+		m_glyphs[i].x = zoom_factor * (extra_x_offs + NUMBER_DIST + extra_offs + i * DIGIT_DIST);
 		m_glyphs[i].y = zoom_factor * NUMBER_Y_POS;
 	}
 	cairo_show_glyphs(cr, m_glyphs, m_number_len);
@@ -153,10 +179,10 @@ void NedTempoSign::draw(cairo_t *cr, double leftx, double topy, double zoom_fact
 	}
 	cairo_new_path(cr);
 	cairo_set_line_width(cr, zoom_factor * STEM_THICK);
-	cairo_move_to(cr, zoom_factor * (EQU_XOFFS + EQU_LEN + extra_offs), - zoom_factor * QE_1_POS);
-	cairo_line_to(cr, zoom_factor * (EQU_XOFFS + extra_offs), - zoom_factor * QE_1_POS);
-	cairo_move_to(cr, zoom_factor * (EQU_XOFFS + EQU_LEN + extra_offs), - zoom_factor * QE_2_POS);
-	cairo_line_to(cr, zoom_factor * (EQU_XOFFS + extra_offs), - zoom_factor * QE_2_POS);
+	cairo_move_to(cr, zoom_factor * (extra_x_offs + EQU_XOFFS + EQU_LEN + extra_offs), - zoom_factor * QE_1_POS);
+	cairo_line_to(cr, zoom_factor * (extra_x_offs + EQU_XOFFS + extra_offs), - zoom_factor * QE_1_POS);
+	cairo_move_to(cr, zoom_factor * (extra_x_offs + EQU_XOFFS + EQU_LEN + extra_offs), - zoom_factor * QE_2_POS);
+	cairo_line_to(cr, zoom_factor * (extra_x_offs + EQU_XOFFS + extra_offs), - zoom_factor * QE_2_POS);
 	cairo_stroke(cr);
 	if (isActive()) {
 		cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
