@@ -35,6 +35,7 @@ class NedChordOrRest;
 class NedCommandList;
 class NedBeam;
 class NedClipBoard;
+class NedLyricsEditor;
 class NedVoice;
 class NedBbox;
 class NedMeasure;
@@ -60,6 +61,7 @@ class NedVoice {
 		void draw(cairo_t *cr, bool *freetexts_or_lyrics_present);
 		void drawTexts(cairo_t *cr, double scale);
 		int getBeamCount();
+		int getPosition(NedChordOrRest *chord_or_rest);
 		int getVoiceNumber() {return m_voice_nr;}
 		void setVoiceNumber(int nr) {m_voice_nr = nr;}
 		NedMainWindow *getMainWindow();
@@ -69,14 +71,16 @@ class NedVoice {
 		int getNumElements();
 		void prepareForImport() {m_start_of_last_imported = m_end_of_last_imported = NULL;}
 		void appendElementsOfMeasureLength(part *part_ptr, unsigned int meas_duration, int meas_num);
+		void copyDataOfWholeVoice();
 		bool handleImportedTuplets();
 		void handleEmptyMeasures();
 		void zoomFreeReplaceables(double zoom, double scale);
+		void recomputeFreeReplaceables();
 		void removeLastImported();
 		void handleStaffElements();
 		bool trySelect (double x, double y, bool only_free_placeables);
 		NedChordOrRest *findNearestElement(double x, double y, double factor, double *mindist, double *ydist);
-		bool tryInsertOrErease (double x, double y, NedChordOrRest **newObj, bool force_rest);
+		bool tryInsertOrErease (double x, double y, int num_midi_input_notes, int *midi_input_chord /* given if insert from midikeyboard */, NedChordOrRest **newObj, bool force_rest);
 		bool tryConvertToTuplet(int method, int tuplet_val, NedChordOrRest *templ);
 		double findTimePos(unsigned int time);
 		void addBeam(NedBeam *beam);
@@ -107,12 +111,15 @@ class NedVoice {
 		void restoreTuplets(FILE *fp);
 		void restoreFreePlaceables(FILE *fp);
 		void collectVolumeChanges(GList **volume_changes);
+		void prepareMutedReplay();
+		void resetActiveFlags();
 		void prepareReplay();
 		void findAccidentals(char offs_array[115], NedMeasure *meas_info, unsigned long long midi_time, bool including);
 		void removeUnneededAccidentals(GList *selected_group);
 		void setInternalPitches();
 		void hideRests(NedCommandList *command_list, bool unhide);
 		void adjustAccidentals();
+		void changeAccidentals(NedCommandList *command_list, int preferred_offs, GList *selected_group);
 		void setInternalPitch(int pitchdist, GList *selected_group);
 		void adjustAccidentals(int pitchdist, GList *selected_group);
 		void shiftNotes(unsigned long long start_time, int linedist, unsigned long long stop_time, bool stop_at);
@@ -147,21 +154,29 @@ class NedVoice {
 		void determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found);
 		bool detectVoices(unsigned int *voice_mask, unsigned long long *e_time);
 		void exportLilyPond(FILE *fp, int *last_line, unsigned int *midi_len, bool last_system,
-				unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordname);
-		void exportLilyChordnames(FILE *fp, int *last_line, unsigned int *midi_len, bool last_system,
+				unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur,
+				unsigned int *lyrics_map, bool with_break, bool *guitar_chordnames, bool *chordname, int *breath_script, bool keep_beams);
+		void exportLilyGuitarChordnames(FILE *fp, int *last_line, unsigned int *midi_len, bool last_system,
 				unsigned long long end_time, bool *in_alternative, bool with_break);
+		void exportLilyFreeChordName(FILE *fp, int *last_line, unsigned int *midi_len, bool last_system,
+			unsigned long long end_time, bool *in_alternative, bool with_break);
 		void exportLilyLyrics(FILE *fp, bool last_system, int line_nr, unsigned long long end_time, int *sil_count);
+		void collectLyrics(NedLyricsEditor *leditor);
+		void setLyrics(NedCommandList *command_list, NedLyricsEditor *leditor);
 	private:
 		bool beamEndRequired(GList *beamlist_so_far, unsigned int beats, NedMeasure *measure);
 		NedChordOrRest *handleGapAfterInsertion(NedCommandList *command_list, int *pos /* in/out */, int sum_of_chords_and_rests_to_delete,
 			NedChordOrRest *chord_or_rest_to_distribute, int tuplet_val,  bool force_wholes, bool do_execute);
+		NedChordOrRest *handleGapAfterInsertion2(NedCommandList *command_list, unsigned long long offs, int *pos /* in/out */,
+			int sum_of_chords_and_rests_to_delete, NedChordOrRest *chord_or_rest_to_distribute,
+			int tuplet_val, bool force_wholes, bool do_execute);
 
 		bool searchForBreakableTuplet(NedCommandList *command_list, unsigned long long critical_time);
 		void searchForBreakableTies(NedCommandList *command_list, unsigned int dir, unsigned long long start_time, unsigned long long end_time);
 		void testForTupletEnd(unsigned long long start_time, unsigned int duration, unsigned long long *end_time);
 		void testAllMeasuresForFillup(NedCommandList *command_list);
 		void exportLilypondSpecMeasure(FILE *fp, unsigned int special, SpecialMeasure *spec_measure, bool *in_alternative, int measure_number);
-		bool insertGrace(double x, double y);
+		bool insertGrace(double x, double y, int pitch /* given if insert from midikeyboard */);
 		GList *m_chord_or_rests;
 		GList *m_beam_list;
 		GList *m_tuplet_list;
